package jlx.asal.j;

import java.lang.reflect.*;

import jlx.blocks.ibd1.*;
import jlx.common.ReflectionUtils;
import jlx.common.reflection.FieldReflectionException;

public abstract class JBuilder {
	protected JTrigger none(JStat... statements) {
		return new JTrigger.NONE(JBool.TRUE, seq(statements));
	}
	
	protected JTrigger finalized(JStat... statements) {
		return new JTrigger.FINALIZED(seq(statements));
	}
	
	protected JTrigger guard(JBool guard, JStat... statements) {
		return new JTrigger.NONE(guard, seq(statements));
	}
	
	protected JTrigger when(JBool predicate, JBool guard, JStat... statements) {
		return new JTrigger.WHEN(predicate, guard, seq(statements));
	}
	
	protected JTrigger when(JBool predicate, JStat... statements) {
		return new JTrigger.WHEN(predicate, JBool.TRUE, seq(statements));
	}
	
	protected JTrigger when(InPort<JPulse> predicate, JBool guard, JStat... statements) {
		return new JTrigger.WHEN(predicate, guard, seq(statements));
	}
	
	protected JTrigger when(InPort<JPulse> predicate, JStat... statements) {
		return new JTrigger.WHEN(predicate, JBool.TRUE, seq(statements));
	}
	
	protected JTrigger after(InPort<JInt> timeout, JBool guard, JStat... cmds) {
		return new JTrigger.AFTER(timeout, guard, seq(cmds));
	}
	
	protected JTrigger after(InPort<JInt> timeout, JStat... statements) {
		return new JTrigger.AFTER(timeout, JBool.TRUE, seq(statements));
	}
	
	protected JStat empty() {
		return new JStat.EMPTY();
	}
	
	protected <T extends JType> JStat assign(InPort<T> var, T expr) {
		return new JStat.ASSIGNMENT(var, expr);
	}
	
	protected <T extends JType> JStat assign(OutPort<T> var, T expr) {
		return new JStat.ASSIGNMENT(var, expr);
	}
	
	protected <T extends JType> JStat assign(T var, T expr) {
		return new JStat.ASSIGNMENT(var, expr);
	}
	
	protected JStat assign(JBool var, JPulse expr) {
		return new JStat.ASSIGNMENT(var, expr);
	}
	
	protected JStat assign(JPulse var, JBool expr) {
		return new JStat.ASSIGNMENT(var, expr);
	}
	
	protected JStat $if(JBool condition, JStat ifBranch) {
		return new JStat.IF(condition, ifBranch, new JStat.EMPTY());
	}
	
	protected JStat $if(JBool condition, JStat ifBranch, JStat elseBranch) {
		return new JStat.IF(condition, ifBranch, elseBranch);
	}
	
	protected JStat $return() {
		return new JStat.RETURN_VOID();
	}
	
	protected JStat $return(JType expr) {
		return new JStat.RETURN(expr);
	}
	
	protected JStat seq(JStat... statements) {
		if (statements.length == 0) {
			return new JStat.EMPTY();
		}
		
		JStat result = statements[0];
		
		for (int index = 1; index < statements.length; index++) {
			result = new JStat.SEQ(result, statements[index]);
		}
		
		return result;
	}
	
	protected JPulse toPulse(JBool x) {
		return new JPulse.BOOL2PULSE(x);
	}
	
	protected JBool not(JBool x) {
		return new JBool.UNARY(JOperation.NOT, x);
	}
	
	protected JBool and(JBool x, JBool y) {
		return new JBool.BINARY(JOperation.AND, x, y);
	}
	
	protected <T extends JType> JBool eq(T x, T y) {
		return new JBool.EQ(x, y);
	}
	
	protected <T extends JType> JBool neq(T x, T y) {
		return new JBool.NEQ(x, y);
	}
	
	private Field findOperationField(Operation<?> operation) {
		for (Field f : getClass().getFields()) {
			if (Operation.class.isAssignableFrom(f.getType())) {
				try {
					if (f.get(this) == operation) {
						return f;
					}
				} catch (IllegalArgumentException | IllegalAccessException e) {
					//Do nothing, just ignore the field.
				}
			}
		}
		
		throw new Error("Operation is not in scope!");
	}
	
//	private Field findPortField(PrimitivePort<?> port) {
//		for (Field f : getClass().getFields()) {
//			if (PrimitivePort.class.isAssignableFrom(f.getType())) {
//				try {
//					if (f.get(this) == port) {
//						return f;
//					}
//				} catch (IllegalArgumentException | IllegalAccessException e) {
//					//Do nothing, just ignore the field.
//				}
//			}
//		}
//		
//		throw new Error("Port is not in scope!");
//	}
	
	protected JBool isSubtypeOf(JType expr, Class<JType> targetClz) {
		return new JBool.IS(expr, JType.createVar(targetClz));
	}
	
	protected <T extends JType> T cast(JType expr, Class<T> targetClz) {
		return JType.createCast(expr, targetClz);
	}
	
	protected <T extends JType> T param(String paramName, Class<T> paramType) {
		return JType.createParam(paramName, paramType);
	}
	
	protected JStat invokeStat(Operation<? extends JType> operation, JType... paramValues) {
		return new JStat.INVOKE(invoke(operation, paramValues));
	}
	
	@SuppressWarnings("unchecked")
	protected <T extends JType> T invoke(Operation<T> operation, JType... paramValues) {
		Field f = findOperationField(operation);
		
		try {
			Operation<? extends JType> fieldValue = (Operation<? extends JType>)f.get(this);
			Class<? extends JType> returnType;
			
			if (fieldValue instanceof Initialization) {
				returnType = JVoid.class;
			} else {
				returnType = ReflectionUtils.getTypeParam(f).asSubclass(JType.class);
			}
			
			if (paramValues.length != fieldValue.getParams().size()) {
				throw new Error("Expected " + fieldValue.getParams().size() + " parameters but found " + paramValues.length + " in invocation of the " + f.getName() + " operation!");
			}
			
			//We cannot type-check here, because parameter values could be ports/properties:
			
//			for (int index = 0; index < paramValues.length; index++) {
//				JType paramValue = paramValues[index];
//				JType param = fieldValue.getParams().get(index);
//				
//				if (!param.getClass().isAssignableFrom(paramValue.getClass())) {
//					throw new Error("Cannot assign " + paramValue.getClass().getCanonicalName() + " to parameter " + param.getParamName() + " in invocation of the " + f.getName() + " operation!");
//				}
//			}
			
			return (T)JType.createOpInvocation(f.getName(), returnType, paramValues);
		} catch (IllegalAccessException | FieldReflectionException | IllegalArgumentException | SecurityException e) {
			throw new Error("Unexpected error in operation call!", e);
		}
	}
}

