package jlx.behave.proto;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.time.LocalTime;
import java.util.*;

public class DecaFourInputPath {
	private final List<DecaFourStateConfig> cfgs;
	private final List<PulsePackMap> inputs;
	private final List<Boolean> nonDets;
	private final DecaFourStateConfig firstCfg;
	private final int hashCode;
	
	public DecaFourInputPath(DecaFourStateConfig startCfg) {
		cfgs = new ArrayList<DecaFourStateConfig>();
		cfgs.add(startCfg);
		inputs = new ArrayList<PulsePackMap>();
		nonDets = new ArrayList<Boolean>();
		firstCfg = startCfg;
		
		hashCode = Objects.hash(cfgs, inputs);
	}
	
	public DecaFourInputPath(DecaFourInputPath source, PulsePackMap input, boolean nonDet, DecaFourStateConfig succCfg) {
		cfgs = new ArrayList<DecaFourStateConfig>(source.cfgs);
		cfgs.add(succCfg);
		inputs = new ArrayList<PulsePackMap>(source.inputs);
		inputs.add(input);
		nonDets = new ArrayList<Boolean>();
		nonDets.add(nonDet);
		firstCfg = succCfg;
		
		hashCode = Objects.hash(cfgs, inputs);
	}
	
	public List<DecaFourStateConfig> getCfgs() {
		return cfgs;
	}
	
	public DecaFourStateConfig getFirstCfg() {
		return firstCfg;
	}
	
	public DecaFourStateConfig getLastCfg() {
		return cfgs.get(cfgs.size() - 1);
	}
	
	public List<PulsePackMap> getInputs() {
		return inputs;
	}
	
	public List<Boolean> getNonDets() {
		return nonDets;
	}
	
	public int getNonDetCount() {
		int result = 0;
		
		for (Boolean nd : nonDets) {
			if (nd) {
				result++;
			}
		}
		
		return result;
	}
	
	@Override
	public int hashCode() {
		return hashCode;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DecaFourInputPath other = (DecaFourInputPath) obj;
		return Objects.equals(cfgs, other.cfgs);
	}
	
	public boolean isNonDet() {
		List<Set<DecaFourStateConfig>> result = new ArrayList<Set<DecaFourStateConfig>>();
		result.add(Collections.singleton(firstCfg));
		
		for (int index = 0; index < inputs.size(); index++) {
			Set<DecaFourStateConfig> tgtCfgs = new HashSet<DecaFourStateConfig>();
			PulsePackMap input = inputs.get(index);
			
			for (DecaFourStateConfig srcCfg : result.get(index)) {
				tgtCfgs.addAll(srcCfg.computeTransitiveSuccsViaInputVal(input));
			}
			
			if (tgtCfgs.size() > 1) {
				return true;
			}
			
//			System.out.println("[" + LocalTime.now() + "] #tgtCfgs[" + (index + 1) + " / " + inputs.size() + "] = " + tgtCfgs.size());
			result.add(tgtCfgs);
		}
		
		return false;
	}
	
	public List<Set<DecaFourStateConfig>> computePotentialCfgs() {
		List<Set<DecaFourStateConfig>> result = new ArrayList<Set<DecaFourStateConfig>>();
		result.add(Collections.singleton(firstCfg));
		
		for (int index = 0; index < inputs.size(); index++) {
			Set<DecaFourStateConfig> tgtCfgs = new HashSet<DecaFourStateConfig>();
			PulsePackMap input = inputs.get(index);
			
			for (DecaFourStateConfig srcCfg : result.get(index)) {
				tgtCfgs.addAll(srcCfg.computeTransitiveSuccsViaInputVal(input));
			}
			
			System.out.println("[" + LocalTime.now() + "] #tgtCfgs[" + (index + 1) + " / " + inputs.size() + "] = " + tgtCfgs.size());
			result.add(tgtCfgs);
		}
		
		return result;
	}
	
	public Set<DecaFourStateConfig> computeFinalPotentialCfgs() {
		return computePotentialCfgs().get(cfgs.size() - 1);
	}
	
	public void saveToFile(String filename) {
		try {
			int slashIndex = filename.lastIndexOf("/");
			
			if (slashIndex >= 0) {
				String dirName = filename.substring(0, slashIndex);
				File dir = new File(dirName);
				dir.mkdir();
			}
			
			PrintStream out = new PrintStream(filename);
			
			for (int i = 0; i < inputs.size(); i++) {
				out.println(i + ". " + cfgs.get(i));
				out.println(inputs.get(i).toString());
			}
			
			{
				int i = cfgs.size() - 1;
				out.println(i + ". " + cfgs.get(i));
			}
			
			out.close();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
}

