package jlx.behave.proto;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import jlx.behave.proto.gui.DecaFourScenarioPlayer;
import jlx.models.UnifyingBlock;
import jlx.models.UnifyingBlock.ReprBlock;
import jlx.utils.HashMaps;
import jlx.utils.Texts;

public class DecaFourScenarioPlayerComp {
	private final DecaFourScenarioPlayer[] players;
	private final List<Row> rows;
	
	public static class Row {
		public final Class<?> filter;
		public final boolean[] occurrencePerStat;
		public final Set<?> elems;
		public final int maxPrintCount;
		
		public Row(Class<?> filter, boolean[] occurrencePerStat, Set<?> elems, int maxPrintCount) {
			this.filter = filter;
			this.occurrencePerStat = occurrencePerStat;
			this.elems = elems;
			this.maxPrintCount = maxPrintCount;
		}
	}
	
	public DecaFourScenarioPlayerComp(DecaFourScenarioPlayer... players) {
		this.players = players;
		
		rows = new ArrayList<Row>();
	}
	
	public void addProtoComparison(int maxPrintCount) {
		Map<Class<?>, Set<Integer>> occurrencesPerElem = new HashMap<Class<?>, Set<Integer>>();
		
		for (int index = 0; index < players.length; index++) {
			for (LineageObject x : players[index].getLineageObjects()) {
				if (ProtoVertex.class.isAssignableFrom(x.getClass())) {
					ProtoVertex v = ProtoVertex.class.cast(x);
					HashMaps.inject(occurrencesPerElem, v.sysmlClz, index);
				}
			}
		}
		
		Map<Set<Integer>, Set<Class<?>>> elemsPerOccurrences = HashMaps.invert(occurrencesPerElem);
		
		for (Map.Entry<Set<Integer>, Set<Class<?>>> e : elemsPerOccurrences.entrySet()) {
			boolean[] xs = new boolean[players.length];
			
			for (int i : e.getKey()) {
				xs[i] = true;
			}
			
			rows.add(new Row(Object.class, xs, e.getValue(), maxPrintCount));
		}
	}
	
	public <T extends LineageObject> void addComparison(Class<T> filter, int maxPrintCount) {
		Map<T, Set<Integer>> occurrencesPerElem = new HashMap<T, Set<Integer>>();
		
		for (int index = 0; index < players.length; index++) {
			for (LineageObject x : players[index].getLineageObjects()) {
				if (filter.isAssignableFrom(x.getClass())) {
					HashMaps.inject(occurrencesPerElem, filter.cast(x), index);
				}
			}
		}
		
		Map<Set<Integer>, Set<T>> elemsPerOccurrences = HashMaps.invert(occurrencesPerElem);
		
		for (Map.Entry<Set<Integer>, Set<T>> e : elemsPerOccurrences.entrySet()) {
			boolean[] xs = new boolean[players.length];
			
			for (int i : e.getKey()) {
				xs[i] = true;
			}
			
			rows.add(new Row(filter, xs, e.getValue(), maxPrintCount));
		}
	}
	
	public <T extends LineageObject> void addCfgDescComparison(DecaFourStateMachines sms4, int maxPrintCount) {
		Map<String, Set<Integer>> occurrencesPerElem = new HashMap<String, Set<Integer>>();
		
		for (int index = 0; index < players.length; index++) {
			for (DecaFourStateConfig cfg : players[index].getPotVisitedCfgs()) {
				HashMaps.inject(occurrencesPerElem, cfg.getDescription(), index);
			}
		}
		
		Map<Set<Integer>, Set<String>> elemsPerOccurrences = HashMaps.invert(occurrencesPerElem);
		
		for (Map.Entry<Set<Integer>, Set<String>> e : elemsPerOccurrences.entrySet()) {
			boolean[] xs = new boolean[players.length];
			
			for (int i : e.getKey()) {
				xs[i] = true;
			}
			
			rows.add(new Row(String.class, xs, e.getValue(), maxPrintCount));
		}
	}
	
	public Set<DecaFourStateConfig> addCfgComparison(DecaFourStateMachines sms4, int maxPrintCount) {
		Set<DecaFourStateConfig> result = new HashSet<DecaFourStateConfig>();
		Map<DecaFourStateConfig, Set<Integer>> occurrencesPerElem = new HashMap<DecaFourStateConfig, Set<Integer>>();
		
		for (int index = 0; index < players.length; index++) {
			for (DecaFourStateConfig cfg : players[index].getPotVisitedCfgs()) {
				HashMaps.inject(occurrencesPerElem, cfg, index);
			}
		}
		
		Map<Set<Integer>, Set<DecaFourStateConfig>> elemsPerOccurrences = HashMaps.invert(occurrencesPerElem);
		
		for (Map.Entry<Set<Integer>, Set<DecaFourStateConfig>> e : elemsPerOccurrences.entrySet()) {
			boolean[] xs = new boolean[players.length];
			
			for (int i : e.getKey()) {
				xs[i] = true;
			}
			
			rows.add(new Row(DecaFourStateConfig.class, xs, e.getValue(), maxPrintCount));
			
			if (e.getKey().contains(1) && !e.getKey().contains(0)) {
//				Set<DecaFourInputPath> paths = sms4.computePathsFromInitToTarget(e.getValue().iterator().next());
//				System.out.println("#paths = " + paths.size());
//				
//				for (DecaFourInputPath p : paths) {
//					p.saveToFile("sttt/debug-input-path.txt");
//					break;
//				}
				
				result.addAll(e.getValue());
			}
		}
		
		return result;
	}
	
	public void addCfgPairComparison(DecaFourStateMachines sms4, int maxPrintCount) {
		Map<DecaFourCfgPair, Set<Integer>> occurrencesPerElem = new HashMap<DecaFourCfgPair, Set<Integer>>();
		
		for (int index = 0; index < players.length; index++) {
			for (DecaFourCfgPair cfgPair : players[index].getCfgPairs()) {
				HashMaps.inject(occurrencesPerElem, cfgPair, index);
			}
		}
		
		Map<Set<Integer>, Set<DecaFourCfgPair>> elemsPerOccurrences = HashMaps.invert(occurrencesPerElem);
		
		for (Map.Entry<Set<Integer>, Set<DecaFourCfgPair>> e : elemsPerOccurrences.entrySet()) {
			boolean[] xs = new boolean[players.length];
			
			for (int i : e.getKey()) {
				xs[i] = true;
			}
			
			rows.add(new Row(DecaFourCfgPair.class, xs, e.getValue(), maxPrintCount));
		}
	}
	
	public Set<Class<?>> getSysmlClzs(UnifyingBlock ub) {
		Set<Class<?>> result = new HashSet<Class<?>>();
		
		for (ReprBlock rb : ub.getBlocks()) {
			if (rb.getStateMachine() != null) {
				for (DecaTritoVertex v : rb.getDecaTritoStateMachine().vertices.values()) {
					result.addAll(v.getSysmlClzs());
				}
			}
		}
		
		return result;
	}
	
	private static String indexToStr(int index, int max) {
		String result = Integer.toString(index);
		int length = Integer.toString(max).length();
		
		while (result.length() < length) {
			result = "0" + result;
		}
		
		return result;
	}
	
	private static List<String> getBaseColumnValues(Row row) {
		List<String> columnValues = new ArrayList<String>();
		
		for (int i = 0; i < row.occurrencePerStat.length; i++) {
			columnValues.add(row.occurrencePerStat[i] ? "1" : "0");
		}
		
		columnValues.add(row.filter.getCanonicalName()); //type
		columnValues.add(Integer.toString(row.elems.size())); //#elements
		return columnValues;
	}
	
	public void saveToFile(String filename, int maxPrintCount) {
		try {
			int slashIndex = filename.lastIndexOf("/");
			
			if (slashIndex >= 0) {
				String dirName = filename.substring(0, slashIndex);
				File dir = new File(dirName);
				dir.mkdir();
			}
			
			PrintStream out = new PrintStream(filename);
			
			{
				List<String> columnValues = new ArrayList<String>();
				
				for (int i = 0; i < players.length; i++) {
					columnValues.add(players[i].getName());
				}
				
				columnValues.add("Element type"); //type
				columnValues.add("#Elements"); //#elements
				columnValues.add("Element index"); //index
				columnValues.add("Element description"); //description
				
				out.println(Texts.concat(columnValues, "\t"));
			}
			
			for (Row row : rows) {
				final int __maxPrintCount = Math.min(maxPrintCount, row.maxPrintCount);
				
				if (row.elems.size() < __maxPrintCount || (row.elems.size() == 1 && __maxPrintCount == 0)) {
					int j = 0;
					
					for (Object x : row.elems) {
						List<String> columnValues = getBaseColumnValues(row);
						columnValues.add(indexToStr(j, row.elems.size())); //index
						columnValues.add(x.toString()); //description
						out.println(Texts.concat(columnValues, "\t"));
						
						j++;
					}
				} else {
					int j = 0;
					
					for (Object x : row.elems) {
						if (j >= __maxPrintCount) {
							break;
						}
						
						List<String> columnValues = getBaseColumnValues(row);
						columnValues.add(indexToStr(j, row.elems.size())); //index
						columnValues.add(x.toString()); //description
						out.println(Texts.concat(columnValues, "\t"));
						
						j++;
					}
					
					{
						List<String> columnValues = getBaseColumnValues(row);
						columnValues.add(Integer.toString(row.elems.size())); //index
						columnValues.add(". . . " + (row.elems.size() - __maxPrintCount) + " more . . ."); //description
						out.println(Texts.concat(columnValues, "\t"));
					}
				}
			}
			
			out.close();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
}

