package jlx.behave.proto;

import java.util.*;

public class DecaFourStateConfigPath {
	private final List<DecaFourStateConfig> cfgs;
	private final DecaFourCoverageStats coverageStats;
	private final DecaFourStateConfig lastCfg;
	
	private final PulsePackMap lastInput;
	private final int lastInputValId;
	
	private final int hashCode;
	
	public DecaFourStateConfigPath(DecaFourStateConfigPath source, DecaFourCoverageStats sourceCoverageStats, PulsePackMap lastInput, int lastInputValId) {
		cfgs = new ArrayList<DecaFourStateConfig>(source.cfgs);
		coverageStats = new DecaFourCoverageStats(sourceCoverageStats);
		lastCfg = source.lastCfg;
		
		this.lastInput = lastInput;
		this.lastInputValId = lastInputValId;
		
		hashCode = extractHashCode();
	}
	
//	public DecaFourStateConfigPath(DecaFourStateConfig prevCfg, DecaFourStateConfigPath source, int nextInputValId) {
//		cfgs = new ArrayList<DecaFourStateConfig>(source.cfgs);
//		cfgs.add(0, prevCfg);
//		coverageStats = new DecaFourCoverageStats();
//		lastCfg = source.lastCfg;
//		
//		hashCode = Objects.hash(cfgs, coverageStats, lastInputValId);
//	}
	
	public DecaFourStateConfigPath(DecaFourStateConfig startCfg, PulsePackMap startInput, int startInputValId) {
		cfgs = new ArrayList<DecaFourStateConfig>();
		cfgs.add(startCfg);
		coverageStats = new DecaFourCoverageStats();
		lastCfg = startCfg;
		
		lastInput = startInput;
		lastInputValId = startInputValId;
		
		hashCode = extractHashCode();
	}
	
	public DecaFourStateConfigPath(DecaFourStateConfigPath source, DecaFourStateConfig nextCfg, DecaFourCoverageStats nextCoverageStats, PulsePackMap nextInput, int nextInputValId) {
		cfgs = new ArrayList<DecaFourStateConfig>(source.cfgs);
		cfgs.add(nextCfg);
		coverageStats = new DecaFourCoverageStats(source.coverageStats);
		coverageStats.addAll(nextCoverageStats);
		lastCfg = nextCfg;
		
		lastInput = nextInput;
		lastInputValId = nextInputValId;
		
		hashCode = extractHashCode();
	}
	
	private int extractHashCode() {
		return Objects.hash(cfgs, coverageStats, lastInput, lastInputValId);
	}
	
	public List<DecaFourStateConfig> getCfgs() {
		return cfgs;
	}
	
	public DecaFourCoverageStats getCoverageStats() {
		return coverageStats;
	}
	
	public DecaFourStateConfig getFirstCfg() {
		return cfgs.get(0);
	}
	
	public DecaFourStateConfig getLastCfg() {
		return lastCfg;
	}
	
	public int getLastInputValId() {
		return lastInputValId;
	}
	
	public PulsePackMap getLastInputs() {
		return lastInput;
	}
	
	@Override
	public int hashCode() {
		return hashCode;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DecaFourStateConfigPath other = (DecaFourStateConfigPath) obj;
		return Objects.equals(cfgs, other.cfgs) && Objects.equals(coverageStats, other.coverageStats) && Objects.equals(lastInput, other.lastInput) && lastInputValId == other.lastInputValId;
	}
	
	public void print(String cfgName) {
		int currIndex = 0;
		
		for (DecaFourStateConfig cfg : cfgs) {
			System.out.println(cfgName + "[" + currIndex + "] =\n\t\t" + cfg.getDescription("\n\t\t"));
			currIndex++;
		}
	}
}

