package jlx.behave.proto;

import java.io.*;
import java.util.*;

import jlx.asal.j.*;
import jlx.asal.rewrite.ASALSymbolicValue;
import jlx.asal.vars.*;
import jlx.behave.StateMachine;
import jlx.models.UnifyingBlock.*;
import jlx.utils.*;
import jlx.utils.ldd.*;

/**
 * 
 */
public class DecaFourStateMachine implements LineageObject {
	public final JScope scope;
	public final StateMachine instance;
	public final Set<InputEquivClz> inputEquivClzs;
	public final Map<DecaFourTransitionReqSet, Set<InputEquivClz>> inputEquivClzsPerReqSet;
	public final Set<DecaFourVertex> vertices;
	public final DecaFourVertex initialVertex;
	public final PulsePackMap initialInputs;
	public final LDDEncoding<ReprPort, PulsePack> externalInputValEncoding;
	public final Set<DecaFourTransition> transitions;
	public final Set<ProtoTransition> initProtoTransitions;
	public final Map<ASALPort, ASALPort> timeoutPortPerDurationPort;
	public final DecaTritoStateMachine legacy;
//	public final Set<DecaFourTransitionReqSet> reqSets;
	public final Map<Set<DecaFourVertex>, DecaFourTgtGrp> tgtGrpPerVtxs;
	
//	public final Map<ReprPort, Set<PulsePack>> pulsePacksPerExternalInputPort;
	
	public DecaFourStateMachine(DecaTritoStateMachine sm, Map<JScope, DecaTritoStateMachine> smPerScope, Map<DecaTritoVertex, DecaFourVertex> newVertexPerOldVertex, LDDEncoding<JScope, DecaFourVertex> cfgEncoding) {
		instance = sm.instance;
		scope = sm.scope;
		legacy = sm;
		
		timeoutPortPerDurationPort = new HashMap<ASALPort, ASALPort>(sm.timeoutPortPerDurationPort);
		vertices = new HashSet<DecaFourVertex>();
		
		for (DecaTritoVertex oldVertex : sm.vertices.values()) {
			vertices.add(newVertexPerOldVertex.get(oldVertex));
		}
		
		Map<ReprPort, List<PulsePack>> pvsPerPort = new HashMap<ReprPort, List<PulsePack>>();
		
		for (Map.Entry<ReprPort, List<PulsePack>> e : sm.inputValEncoding.getOrderedValuesPerKey().entrySet()) {
			if (e.getKey().isPortToEnvironment()) {
				pvsPerPort.put(e.getKey(), e.getValue());
			}
		}
		
		externalInputValEncoding = new LDDEncoding<ReprPort, PulsePack>(pvsPerPort);
		
		inputEquivClzs = new HashSet<InputEquivClz>();
		initialInputs = sm.initialInputs;
		initProtoTransitions = new HashSet<ProtoTransition>(sm.initProtoTransitions);
		initialVertex = newVertexPerOldVertex.get(sm.initialVertex);
		
		//Determine all "output state machines", i.e. state machines that generate outputs that become inputs for this state machine.
		//Per output state machine X, determine the input ports of this state machine to which X's output ports are connected:
		Map<DecaTritoStateMachine, Map<ReprPort, Set<ReprPort>>> inputsPerOutputPerOutputSm = new HashMap<DecaTritoStateMachine, Map<ReprPort, Set<ReprPort>>>();
		
		for (DecaTritoStateMachine outputSm : smPerScope.values()) {
			Map<ReprPort, Set<ReprPort>> inputsPerOutput = new HashMap<ReprPort, Set<ReprPort>>();
			
			for (ASALVariable v : outputSm.scope.getVariablePerName().values()) {
				if (v instanceof ReprPort) {
					ReprPort rp = (ReprPort)v;
					
					for (ReprFlow rf : rp.getOutgoingFlows()) {
						if (rf.target.getReprOwner() == scope) {
							HashMaps.inject(inputsPerOutput, rp, rf.target);
						}
					}
				}
			}
			
			if (inputsPerOutput.size() > 0) {
				inputsPerOutputPerOutputSm.put(outputSm, inputsPerOutput);
			}
		}
		
		System.out.println("2");
		
		//Group vertices of each output state machine by the (internal) output valuation that it generates for ONLY THIS state machine:
		Map<DecaTritoStateMachine, Map<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>>> vtxsPerInternalOutputValPerOutputSm = new HashMap<DecaTritoStateMachine, Map<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>>>();
		
		for (Map.Entry<DecaTritoStateMachine, Map<ReprPort, Set<ReprPort>>> e : inputsPerOutputPerOutputSm.entrySet()) {
			Map<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>> vtxsPerInternalOutputVal = new HashMap<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>>();
			Set<ReprPort> outputs = e.getValue().keySet();
			
			for (DecaTritoVertex v : e.getKey().vertices.values()) {
				Map<ReprPort, ASALSymbolicValue> internalOutputVal = new HashMap<ReprPort, ASALSymbolicValue>();
				
				for (Map.Entry<ReprPort, ASALSymbolicValue> e2 : v.getOutputVal().entrySet()) {
					if (outputs.contains(e2.getKey())) {
						internalOutputVal.put(e2.getKey(), e2.getValue());
					}
				}
				
				if (newVertexPerOldVertex.get(v) == null) {
					for (Class<?> c : v.getSysmlClzs()) {
						System.out.println("error clz = " + c.getCanonicalName());
					}
					
					throw new Error("Should not happen!");
				}
				
				internalOutputVal = PulsePackMap.from(internalOutputVal, Dir.OUT).extractValuation();
				HashMaps.inject(vtxsPerInternalOutputVal, internalOutputVal, newVertexPerOldVertex.get(v));
			}
			
			vtxsPerInternalOutputValPerOutputSm.put(e.getKey(), vtxsPerInternalOutputVal);
		}
		
		System.out.println("3");
		
//		Map<Map<JScope, DecaFourVertex>, Set<PulsePackMap>> inputValsPerVtxCfg = new HashMap<Map<JScope, DecaFourVertex>, Set<PulsePackMap>>();
		Map<PulsePackMap, DecaFourTransitionReqSet> reqSetPerInputVal = new HashMap<PulsePackMap, DecaFourTransitionReqSet>();
		
		for (PulsePackMap input : sm.allInputs) {
			Map<ReprPort, ASALSymbolicValue> inputVal = input.extractValuation();
			
			//Compute per output state machine the vertices that generate outputs that match the currently considered input valuation:
			Map<JScope, Set<DecaFourVertex>> vtxsPerOutputSm = new HashMap<JScope, Set<DecaFourVertex>>();
			boolean isValid = true;
			
			for (Map.Entry<DecaTritoStateMachine, Map<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>>> e : vtxsPerInternalOutputValPerOutputSm.entrySet()) {
				Map<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>> vtxsPerInternalOutputVal = e.getValue();
				Map<ReprPort, Set<ReprPort>> inputsPerOutput = inputsPerOutputPerOutputSm.get(e.getKey());
				Set<DecaFourVertex> vtxs = new HashSet<DecaFourVertex>();
				
				for (Map.Entry<Map<ReprPort, ASALSymbolicValue>, Set<DecaFourVertex>> e2 : vtxsPerInternalOutputVal.entrySet()) {
					Map<ReprPort, ASALSymbolicValue> internalOutputVal = e2.getKey();
					boolean matches = true;
					
					for (Map.Entry<ReprPort, ASALSymbolicValue> e3 : internalOutputVal.entrySet()) {
						for (ReprPort inputRp : inputsPerOutput.get(e3.getKey())) {
							ASALSymbolicValue v = inputVal.get(inputRp);
							
							//Two possibilities:
							// - Value is not null.
							//    Then the value must match the output value.
							// - Value is null.
							//    Then it must be a DT-port of which the T-port is FALSE.
							//    We will encounter a mismatch for the T-port values, or
							//    the output value is a dummy DT-port value that accompanies a FALSE T-port.
							if (v != null && !e3.getValue().equals(v)) {
								matches = false;
								break;
							}
						}
						
						if (!matches) {
							break;
						}
					}
					
					if (matches) {
						vtxs.addAll(e2.getValue());
					}
				}
				
				//No state of the current output state machine supports the input valuation:
				if (vtxs.isEmpty()) {
//					System.out.println("No output matches input " + input.extractInternalMap());
//					
//					CLI.waitForEnter();
					
					isValid = false;
					break;
				}
				
				vtxsPerOutputSm.put(e.getKey().scope, vtxs);
			}
			
			if (isValid) {
				reqSetPerInputVal.put(input, DecaFourTransitionReqSet.addLDDPerScopes(reqSetPerInputVal.get(input), vtxsPerOutputSm.keySet(), new LDD<JScope, DecaFourVertex>(cfgEncoding, vtxsPerOutputSm)));
			}
		}
		
		System.out.println("4");
		
		Set<Set<PulsePackMap>> inputEquivClzSets = new HashSet<Set<PulsePackMap>>();
		
		for (DecaTritoTransition t : sm.transitions) {
			inputEquivClzSets.add(t.getInputVals());
		}
		
		System.out.println("#deca-four-inputEquivClzSets = " + inputEquivClzSets.size());
		
//		reqSets = new HashSet<DecaFourTransitionReqSet>();
		Map<Set<PulsePackMap>, DecaFourTransitionReqSet> reqSetPerInputEquivClzSet = new HashMap<Set<PulsePackMap>, DecaFourTransitionReqSet>();
		
		for (Set<PulsePackMap> is : inputEquivClzSets) {
			if (is.isEmpty()) {
				throw new Error("Should not happen!");
			}
			
			DecaFourTransitionReqSet reqSet = null;
			
			for (PulsePackMap i : is) {
				reqSet = DecaFourTransitionReqSet.union(reqSet, reqSetPerInputVal.get(i));
			}
			
			reqSetPerInputEquivClzSet.put(is, reqSet);
		}
		
		System.out.println("#deca-four-reqSets = " + reqSetPerInputEquivClzSet.size());
		
		tgtGrpPerVtxs = new HashMap<Set<DecaFourVertex>, DecaFourTgtGrp>();
		
//		pulsePacksPerExternalInputPort = new HashMap<ReprPort, Set<PulsePack>>();
		
		//Transitions:
		transitions = new HashSet<DecaFourTransition>();
		
		for (DecaTritoTransition t : sm.transitions) {
			DecaFourVertex srcVertex = newVertexPerOldVertex.get(t.getSrc());
			DecaFourTransitionReqSet reqSet = reqSetPerInputEquivClzSet.get(t.getInputVals());
			
			if (reqSet == null) {
				//throw new Error("Cannot happen?");
				//This *CAN* happen: This means that other state machines prevent this transition from ever being enabled!
//				System.out.println("( Found blocked transition )");
				continue;
			}
			
			Set<DecaFourVertex> tgtVertices = new HashSet<DecaFourVertex>();
			
			for (DecaTritoVertex c : t.getTgts()) {
				tgtVertices.add(newVertexPerOldVertex.get(c));
			}
			
			DecaFourTgtGrp tgtGrp = tgtGrpPerVtxs.get(tgtVertices);
			
			if (tgtGrp == null) {
				tgtGrp = new DecaFourTgtGrp(scope, tgtVertices, tgtGrpPerVtxs.size() + 1);
				tgtGrpPerVtxs.put(tgtVertices, tgtGrp);
			}
			
			DecaFourTransition newTr = new DecaFourTransition(t, reqSet, t.getInputVals(), srcVertex, tgtGrp);
			HashMaps.inject(srcVertex.getNondetOutgoingPerReqSet(), reqSet, newTr);
			srcVertex.getOutgoing().add(newTr);
			
			if (tgtVertices.size() == 1 && tgtVertices.contains(srcVertex)) {
				HashMaps.injectAll(srcVertex.getSelfLoopInputsPerReqSet(), reqSet, newTr.getInputs());
			}
			
			for (PulsePackMap i : newTr.getInputs()) {
				HashMaps.injectInject(srcVertex.getInputsPerNonDetTgtsPerReqSet(), reqSet, tgtGrp, i);
				
				if (tgtVertices.size() == 1) {
					HashMaps.injectInject(srcVertex.getInputsPerDetTgtsPerReqSet(), reqSet, tgtGrp, i);
				}
				
				PulsePackMap e = i.extractExternalMap();
				
				if (e != null) { //Never null . . .
					HashMaps.injectInject(srcVertex.getTrsPerNonDetTgtsPerReqSet(), reqSet, e, newTr);
					HashMaps.injectInject(srcVertex.getExternalInputsPerNonDetTgtsPerReqSet(), reqSet, tgtGrp, e);
				}
			}
			
			for (DecaFourVertex tgtVertex : tgtVertices) {
				tgtVertex.getIncoming().add(newTr);
			}
			
			transitions.add(newTr);
		}
		
		System.out.println("5");
		
		for (DecaFourVertex v : vertices) {
			for (Map.Entry<DecaFourTransitionReqSet, Set<PulsePackMap>> e : v.getSelfLoopInputsPerReqSet().entrySet()) {
				v.getSelfLoopInputsPerReqSet2().put(e.getKey(), new DecaFourPulsePackMaps(e.getValue(), externalInputValEncoding));
			}
			
//			for (Map.Entry<DecaFourTransitionReqSet, Map<DecaFourTgtGrp, Set<PulsePackMap>>> e : v.getInputsPerNonDetTgtsPerReqSet().entrySet()) {
//				for (Map.Entry<DecaFourTgtGrp, Set<PulsePackMap>> e2 : e.getValue().entrySet()) {
//					HashMaps.injectSet(v.getInputsPerNonDetTgtsPerReqSet2(), e.getKey(), e2.getKey(), new DecaFourPulsePackMaps2(e2.getValue(), false));
//				}
//			}
			
			for (Map.Entry<DecaFourTransitionReqSet, Map<DecaFourTgtGrp, Set<PulsePackMap>>> e : v.getExternalInputsPerNonDetTgtsPerReqSet().entrySet()) {
				for (Map.Entry<DecaFourTgtGrp, Set<PulsePackMap>> e2 : e.getValue().entrySet()) {
					HashMaps.injectSet(v.getExternalInputsPerNonDetTgtsPerReqSet2(), e.getKey(), e2.getKey(), new DecaFourPulsePackMaps(e2.getValue(), externalInputValEncoding));
				}
			}
		}
		
		System.out.println("6");
		
		inputEquivClzsPerReqSet = Collections.emptyMap();
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(initProtoTransitions);
		result.addAll(transitions);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		
		for (DecaFourTransition t : transitions) {
			result.addAll(t.getInferredElements());
		}
		
		return result;
	}
	
	public Set<ProtoTransition> getAllProtoTransitions() {
		Set<ProtoTransition> result = new HashSet<ProtoTransition>();
		result.addAll(initProtoTransitions);
		
		for (DecaFourTransition t : transitions) {
			result.addAll(t.getProtoTrs());
		}
		
		return result;
	}
	
	public Map<Set<ReprPort>, Integer> getFreqPerDivPorts() {
		Map<Set<ReprPort>, Integer> result = new HashMap<Set<ReprPort>, Integer>();
		Set<ReprPort> pvsPorts = initialVertex.getOutputVal().extractMultiplePvsMap().getPackPerPort().keySet();
		
		for (Set<ReprPort> ps : HashSets.getSubsets(pvsPorts)) {
			if (ps.size() > 0) {
				Set<Map<ReprPort, ASALSymbolicValue>> elems = new HashSet<Map<ReprPort, ASALSymbolicValue>>();
				
				for (DecaFourVertex v : vertices) {
					elems.add(v.getOutputVal().extractValuation(ps));
				}
				
				Set<Set<ReprPort>> keysToRemove = new HashSet<Set<ReprPort>>();
				boolean add = true;
				
				for (Map.Entry<Set<ReprPort>, Integer> e : result.entrySet()) {
					if (e.getKey().size() == ps.size()) {
						if (e.getValue() < elems.size()) {
							keysToRemove.add(e.getKey());
						} else {
							if (e.getValue() > elems.size()) {
								add = false;
							}
						}
					}
				}
				
				if (add) {
					result.keySet().removeAll(keysToRemove);
					result.put(ps, elems.size());
				}
			}
		}
		
		return result;
	}
	
	public void printGraphvizFile() {
		try {
			PrintStream ps = new PrintStream("deca-four-" + scope.getName() + ".gv");
			printGraphvizFile(ps, transitions);
			ps.flush();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
	
	public void printGraphvizFile(PrintStream out, Collection<DecaFourTransition> trs) {
		Set<DecaFourVertex> vtxs = new HashSet<DecaFourVertex>();
		
		for (DecaFourTransition t : trs) {
			vtxs.add(t.getSourceVertex());
			vtxs.addAll(t.getTgtGrp().getVtxs());
		}
		
		Map<DecaFourVertex, String> namePerVertex = new HashMap<DecaFourVertex, String>();
		Map<DecaFourTransition, String> namePerTransition = new HashMap<DecaFourTransition, String>();
		Map<DecaFourTransition, String> colorPerTransition = new HashMap<DecaFourTransition, String>();
//		Map<DecaFourTransition, Set<DecaFourTransition>> notePerTransition = new HashMap<DecaFourTransition, Set<DecaFourTransition>>();
		
		for (DecaFourVertex v : vtxs) {
			namePerVertex.put(v, "V" + namePerVertex.size());
		}
		
		for (DecaFourTransition t : trs) {
			namePerTransition.put(t, "T" + namePerTransition.size());
			colorPerTransition.put(t, Dot.getRandomColor());
		}
		
		out.println("// " + getClass().getCanonicalName());
		out.println("// " + scope.getName());
		out.println("digraph G {");
		
		{
			String s = "(initial)";
			
			for (Map.Entry<ReprPort, ASALSymbolicValue> entry : initialVertex.getOutputVal().extractValuation().entrySet()) {
				s += "\\n" + entry.getKey().getName() + " := " + entry.getValue().toString().replace("\"", "\\\"");
			}
			
			out.println("\tI0 [label=\"" + s + "\", shape=circle, fontsize=10];");
		}
		
		for (DecaFourVertex v : vtxs) {
			String s = "ID " + v.getName();
			s += "\\n" + Texts.concat(v.getSysmlClzs(), " + ", (c) -> { return c.getSimpleName(); });
			
			for (Map.Entry<ReprPort, ASALSymbolicValue> entry : v.getOutputVal().extractValuation().entrySet()) {
				if (!entry.getKey().getType().equals(JPulse.class) || entry.getValue().equals(ASALSymbolicValue.TRUE)) {
					s += "\\n" + Texts._break(entry.getKey().getName() + " := " + entry.getValue().toString().replace("\"", "\\\""), "\\n", 70);
				}
			}
			
			out.println("\t" + namePerVertex.get(v) + " [label=\"" + s + "\", shape=ellipse];");
		}
		
		for (DecaFourTransition t : trs) {
			{
				TextOptions.select(TextOptions.GRAPHVIZ_MIN);
				
				String s = namePerTransition.get(t);
				
				Set<PulsePackMap> zz = new HashSet<PulsePackMap>();
				
				for (PulsePackMap i : t.getInputs()) {
					zz.add(i.extractMultiplePvsMap());
				}
				
				if (zz.size() <= 12) {
					for (PulsePackMap i : zz) {
						s += "\\n" + i.toString();
					}
				}
				
				String colors = "color=\"" + colorPerTransition.get(t) + "\", fontcolor=\"" + colorPerTransition.get(t) + "\"";
				out.println("\t" + namePerTransition.get(t) + " [label=\"" + s + "\", shape=rectangle, fontsize=10, " + colors + ", style=dashed];");
			}
			
//			for (DecaFourTransition t : notePerTransition.get(e.getKey())) {
//				String s = e.getValue();
//				s += "\\n" + t.getInput().createCombinedGuardStr().replace("\"", "\\\"");
//				out.println("\t% " + s);
//			}
		}
		
		for (DecaFourTransition t : trs) {
			String s = namePerTransition.get(t);
			
			if (t.getTgtGrp().getVtxs().size() > 1) {
				String prev_name = namePerVertex.get(t.getSourceVertex());
				out.println("\t" + prev_name + " -> " + s + " [color=\"" + colorPerTransition.get(t) + "\", style=dashed];");
				
				for (DecaFourVertex tgt : t.getTgtGrp().getVtxs()) {
					String next_name = namePerVertex.get(tgt);
					out.println("\t" + s + " -> " + next_name + " [color=\"" + colorPerTransition.get(t) + "\", style=dashed];");
				}
			} else {
				String prev_name = namePerVertex.get(t.getSourceVertex());
				out.println("\t" + prev_name + " -> " + s + " [color=\"" + colorPerTransition.get(t) + "\", style=solid];");
				
				for (DecaFourVertex tgt : t.getTgtGrp().getVtxs()) {
					String next_name = namePerVertex.get(tgt);
					out.println("\t" + s + " -> " + next_name + " [color=\"" + colorPerTransition.get(t) + "\", style=solid];");
				}
			}
		}
		
		if (vtxs.contains(initialVertex)) {
			out.println("\tI0 -> " + namePerVertex.get(initialVertex) + ";");
		}
		
		out.println("}");
	}
}



