package jlx.behave.proto;

import java.util.*;

import jlx.asal.j.JScope;
import jlx.utils.ldd.*;

public class DecaFourTransitionReqSet {
	private final Map<Set<JScope>, LDD<JScope, DecaFourVertex>> lddPerScopes;
	private final int hashCode;
	
	private DecaFourTransitionReqSet(Map<Set<JScope>, LDD<JScope, DecaFourVertex>> lddPerScopes) {
		this.lddPerScopes = lddPerScopes;
		
		hashCode = lddPerScopes.hashCode();
	}
	
	public static DecaFourTransitionReqSet addLDDPerScopes(DecaFourTransitionReqSet reqSet, Set<JScope> scopes, LDD<JScope, DecaFourVertex> ldd) {
//		if (ldd.isEmpty()) {
//			return reqSet;
//		}
		
		Map<Set<JScope>, LDD<JScope, DecaFourVertex>> newLddPerScopes;
		
		if (reqSet != null) {
			newLddPerScopes = new HashMap<Set<JScope>, LDD<JScope, DecaFourVertex>>();
			
			for (Map.Entry<Set<JScope>, LDD<JScope, DecaFourVertex>> e : reqSet.lddPerScopes.entrySet()) {
				if (e.getKey().equals(scopes)) {
					newLddPerScopes.put(e.getKey(), e.getValue().union(ldd));
				} else {
					newLddPerScopes.put(e.getKey(), ldd);
				}
			}
		} else {
			newLddPerScopes = Collections.singletonMap(scopes, ldd);
		}
		
//		System.out.println("keyScopes = " + scopes.size());
		return new DecaFourTransitionReqSet(newLddPerScopes);
	}
	
	public static DecaFourTransitionReqSet union(DecaFourTransitionReqSet x, DecaFourTransitionReqSet y) {
		if (x == null) {
			return y;
		}
		
		if (y == null) {
			return x;
		}
		
		Map<Set<JScope>, LDD<JScope, DecaFourVertex>> newLddPerScopes = new HashMap<Set<JScope>, LDD<JScope, DecaFourVertex>>();
		
		for (Map.Entry<Set<JScope>, LDD<JScope, DecaFourVertex>> e : x.lddPerScopes.entrySet()) {
			newLddPerScopes.put(e.getKey(), e.getValue());
		}
		
		for (Map.Entry<Set<JScope>, LDD<JScope, DecaFourVertex>> e : y.lddPerScopes.entrySet()) {
			LDD<JScope, DecaFourVertex> newLdd = newLddPerScopes.get(e.getKey());
			
			if (newLdd != null) {
				newLddPerScopes.put(e.getKey(), newLdd.union(e.getValue()));
			} else {
				newLddPerScopes.put(e.getKey(), e.getValue());
			}
		}
		
		return new DecaFourTransitionReqSet(newLddPerScopes);
	}
	
	public boolean containsMatch(Map<JScope, DecaFourVertex> subCfg) {
		for (Map.Entry<Set<JScope>, LDD<JScope, DecaFourVertex>> e : lddPerScopes.entrySet()) {
			LDD<JScope, DecaFourVertex> z;
			
			if (e.getKey().isEmpty()) {
				z = new LDD<JScope, DecaFourVertex>(Collections.emptySet(), e.getValue().getEncoding());
			} else {
				Map<JScope, DecaFourVertex> subVtxs = new HashMap<JScope, DecaFourVertex>();
				
				for (JScope s : e.getKey()) {
					subVtxs.put(s, subCfg.get(s));
				}
				
				z = new LDD<JScope, DecaFourVertex>(Collections.singleton(subVtxs), e.getValue().getEncoding());
			}
			
//			System.out.println("#scopes = " + e.getKey().size());
//			System.out.println("#e= " + e.getValue().size());
//			System.out.println("#z= " + z.size());
//			System.out.println("e= " + e.getValue());
//			System.out.println("z= " + z);
			
			if (e.getValue().containsAll(z)) {
				return true;
			}
		}
		
		return false;
	}
	
	@Override
	public int hashCode() {
		return hashCode;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof DecaFourTransitionReqSet)) {
			return false;
		}
		DecaFourTransitionReqSet other = (DecaFourTransitionReqSet) obj;
		return Objects.equals(lddPerScopes, other.lddPerScopes);
	}
}

