package jlx.behave.proto;

import java.util.*;

import jlx.asal.rewrite.ASALSymbolicValue;
import jlx.asal.vars.ASALVariable;
import jlx.common.FileLocation;
import jlx.utils.Texts;

public class DecaOneTransition implements LineageObject {
	private DecaOneVertex sourceVertex;
	private DecaOneVertex targetVertex;
	private ASALSymbolicValue guard;
	private NextStateFct nextStateFct;
	private DecaTransition legacy;
	
	public DecaOneTransition(DecaTransition source, Map<DecaVertex, DecaOneVertex> newVertexPerOldVertex) {
		sourceVertex = newVertexPerOldVertex.get(source.getSourceVertex());
		targetVertex = newVertexPerOldVertex.get(source.getTargetVertex());
		guard = source.getGuard();
		nextStateFct = new NextStateFct(source.getNextStateFct());
		legacy = source;
	}
	
	/**
	 * Transition from which this transition has been derived.
	 * May reference properties that have been rewritten in this transition.
	 */
	public DecaTransition getLegacy() {
		return legacy;
	}
	
	public boolean isIdle() {
		return legacy.isIdle();
	}
	
	public List<FileLocation> getFileLocations() {
		return legacy.getFileLocations();
	}
	
	public DecaOneVertex getSourceVertex() {
		return sourceVertex;
	}
	
	public DecaOneVertex getTargetVertex() {
		return targetVertex;
	}
	
	public ASALSymbolicValue getGuard() {
		return guard;
	}
	
	public NextStateFct getNextStateFct() {
		return nextStateFct;
	}
	
	public boolean removeIdentityAssignments() {
		NextStateFct temp = new NextStateFct();
		boolean result = false;
		
		for (NextStateFct.Entry e : nextStateFct.getEntries()) {
			if (e.getValue().equals(ASALSymbolicValue.from(e.getVariable()))) {
				result = true;
			} else {
				temp.put(e.getVariable(), e.getValue(), e.getDebugText());
			}
		}
		
		nextStateFct = temp;
		return result;
	}
	
	public boolean removeWrittenVars(Collection<ASALVariable> vars) {
		NextStateFct temp = nextStateFct.removeVariables(vars);
		boolean result = temp.getVariables().size() < nextStateFct.getVariables().size();
		nextStateFct = temp;
		return result;
	}
	
	public boolean substitute(Map<ASALVariable, ASALSymbolicValue> subst) {
		boolean result = false;
		ASALSymbolicValue newGuard = guard.substitute(subst);
		
		if (!newGuard.equals(guard)) {
			guard = newGuard;
			result = true;
		}
		
		if (nextStateFct.substitute(subst)) {
			result = true;
		}
		
		return result;
	}
	
	public boolean applyRestr(Map<ASALVariable, Map<ASALSymbolicValue, ASALSymbolicValue>> restrPerVar) {
		return nextStateFct.applyRestrPerVar(restrPerVar);
	}
	
	public Set<ProtoTransition> getProtoTrs() {
		return legacy.getProtoTrs();
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.add(legacy);
		result.add(sourceVertex);
		result.add(targetVertex);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + sourceVertex.toString() + "]";
		result += "---" + Texts.concat(getFileLocations(), ";", (x) -> { return x.tiny(); }) + "-->";
		result += "[" + targetVertex.toString() + "]";
		return result;
	}
}

