package jlx.behave.proto;

import java.util.*;

import jlx.asal.rewrite.ASALSymbolicValue;
import jlx.common.FileLocation;
import jlx.utils.Texts;

public class DecaTransition implements LineageObject {
	private DecaVertex sourceVertex;
	private DecaVertex targetVertex;
	private ASALSymbolicValue guard;
	private NextStateFct nextStateFct;
	private NonaVertex legacyVertex;
	private NonaTransition legacyTransition;
	private Set<ProtoTransition> protoTrs;
	private boolean idle;
	
	public DecaTransition(NonaVertex legacyVertex, NonaTransition legacyTransition, DecaVertex sourceVertex, DecaVertex targetVertex, ASALSymbolicValue guard, NextStateFct nextStateFct, boolean idle, Set<TritoTransition> tritoTrs) {
		this.sourceVertex = sourceVertex;
		this.targetVertex = targetVertex;
		this.guard = guard;
		this.nextStateFct = nextStateFct;
		this.legacyVertex = legacyVertex;
		this.legacyTransition = legacyTransition;
		this.idle = idle;
		
		protoTrs = new HashSet<ProtoTransition>();
		
		for (TritoTransition t : tritoTrs) {
			protoTrs.addAll(t.getProtoTrs());
		}
	}
	
	public NonaVertex getLegacyVertex() {
		return legacyVertex;
	}
	
	public NonaTransition getLegacyTransition() {
		return legacyTransition;
	}
	
	public List<FileLocation> getFileLocations() {
		if (legacyTransition != null) {
			return legacyTransition.getFileLocations();
		}
		
		return new ArrayList<FileLocation>(legacyVertex.getFileLocations());
	}
	
	public DecaVertex getSourceVertex() {
		return sourceVertex;
	}
	
	public DecaVertex getTargetVertex() {
		return targetVertex;
	}
	
	public ASALSymbolicValue getGuard() {
		return guard;
	}
	
	public NextStateFct getNextStateFct() {
		return nextStateFct;
	}
	
	public List<ASALSymbolicValue> getEqns() {
		List<ASALSymbolicValue> eqns = new ArrayList<ASALSymbolicValue>();
		
		for (NextStateFct.Entry expr : getNextStateFct().getEntries()) {
			eqns.add(expr.getValue());
		}
		
		return eqns;
	}
	
	public boolean isIdle() {
		return idle;
	}
	
	public Set<ProtoTransition> getProtoTrs() {
		return protoTrs;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		
		if (legacyVertex != null) {
			result.add(legacyVertex);
		}
		
		if (legacyTransition != null) {
			result.add(legacyTransition);
		}
		
		result.add(sourceVertex);
		result.add(targetVertex);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + sourceVertex.toString() + "]";
		result += "---" + Texts.concat(getFileLocations(), ";", (x) -> { return x.tiny(); }) + "-->";
		result += "[" + targetVertex.toString() + "]";
		return result;
	}
}



