package jlx.behave.proto;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.*;

import jlx.asal.j.*;
import jlx.asal.rewrite.ASALSymbolicValue;
import jlx.asal.vars.*;
import jlx.behave.StateMachine;
import jlx.common.reflection.*;
import jlx.models.UnifyingBlock.ReprPort;
import jlx.utils.*;
import jlx.utils.ldd.LDDEncoding;

/**
 * This state machines computes input valuations for guards, and
 * groups input valuations (locally, i.e. per vertex) so that
 * all input valuations in a group lead to the same target vertex/vertices.
 * 
 * (Disabled:)
 * Also applies (input port) priorities.
 */
public class DecaTritoStateMachine implements LineageObject {
	public final JScope scope;
	public final StateMachine instance;
	public final Map<DecaTwoVertex, DecaTritoVertex> vertices;
	public final DecaTritoVertex initialVertex;
	public final PulsePackMap initialInputs;
	public final Set<PulsePackMap> allInputs;
//	public final Map<ReprPort, Set<ASALSymbolicValue>> pvsPerInput;
	public final Map<ReprPort, Set<PulsePack>> pulsePacksPerInputPort;
	public final LDDEncoding<ReprPort, PulsePack> inputValEncoding;
	public final Set<DecaTritoTransition> transitions;
	public final Set<ProtoTransition> initProtoTransitions;
	public final Map<ASALPort, ASALPort> timeoutPortPerDurationPort;
	public final DecaTwoStateMachine legacy;
	
	public static boolean USE_PRIORITIES = false;
	
	public DecaTritoStateMachine(DecaTwoStateMachine source) throws ClassReflectionException, ModelException {
		instance = source.instance;
		scope = source.scope;
		legacy = source;
		
		timeoutPortPerDurationPort = new HashMap<ASALPort, ASALPort>(source.timeoutPortPerDurationPort);
		vertices = new HashMap<DecaTwoVertex, DecaTritoVertex>();
		
		for (DecaTwoVertex v : source.vertices.values()) {
			DecaTritoVertex w = new DecaTritoVertex(v);
			vertices.put(v, w);
		}
		
		initialVertex = vertices.get(source.initialVertex);
		initProtoTransitions = new HashSet<ProtoTransition>(source.initProtoTransitions);
		
		pulsePacksPerInputPort = new HashMap<ReprPort, Set<PulsePack>>();
		
		for (Map.Entry<ReprPort, Set<ASALSymbolicValue>> e : source.pvsPerInput.entrySet()) {
			if (JPulse.class.isAssignableFrom(e.getKey().getType())) {
				if (e.getValue().size() > 2) {
					throw new Error("Should not happen!");
				}
				
				Set<PulsePack> packs = new HashSet<PulsePack>();
				
				if (e.getValue().contains(ASALSymbolicValue.FALSE)) {
					packs.add(new PulsePack(e.getKey(), ASALSymbolicValue.FALSE));
				}
				
				if (e.getValue().contains(ASALSymbolicValue.TRUE)) {
					Map<ReprPort, Set<ASALSymbolicValue>> pvsPerDataPort = new HashMap<ReprPort, Set<ASALSymbolicValue>>();
					pvsPerDataPort.put(e.getKey(), Collections.singleton(ASALSymbolicValue.TRUE));
					
					for (ReprPort drp : e.getKey().getDataPorts()) {
						pvsPerDataPort.put(drp, source.pvsPerInput.get(drp));
					}
					
					for (Map<ReprPort, ASALSymbolicValue> inputVal : HashMaps.allCombinations(pvsPerDataPort)) {
						packs.add(new PulsePack(e.getKey(), inputVal));
					}
				}
				
				pulsePacksPerInputPort.put(e.getKey(), packs);
			} else {
				if (e.getKey().getPulsePort() == null) {
					Set<PulsePack> packs = new HashSet<PulsePack>();
					
					for (ASALSymbolicValue value : e.getValue()) {
						packs.add(new PulsePack(e.getKey(), value));
					}
					
					pulsePacksPerInputPort.put(e.getKey(), packs);
				}
			}
		}
		
		inputValEncoding = new LDDEncoding<ReprPort, PulsePack>(pulsePacksPerInputPort);
		
		initialInputs = PulsePackMap.from(source.initialInputs, Dir.IN);
		
		allInputs = new HashSet<PulsePackMap>();
		
		for (Map<ReprPort, ASALSymbolicValue> inputVal : HashMaps.allCombinations(source.pvsPerInput)) {
			allInputs.add(PulsePackMap.from(inputVal, Dir.IN));
		}
		
		Set<ASALSymbolicValue> guards = new HashSet<ASALSymbolicValue>();
		
		for (DecaTwoTransition t : source.transitions) {
			guards.add(t.getGuard());
		}
		
		Map<ASALSymbolicValue, Set<PulsePackMap>> inputValsPerGuard = new HashMap<ASALSymbolicValue, Set<PulsePackMap>>();
		
		for (Map<ReprPort, ASALSymbolicValue> inputVal : HashMaps.allCombinations(source.pvsPerInput)) {
			PulsePackMap ppm = PulsePackMap.from(inputVal, Dir.IN);
			
			for (ASALSymbolicValue guard : guards) {
				if (guard.substitute(inputVal).toBoolean()) {
					HashMaps.inject(inputValsPerGuard, guard, ppm);
				}
			}
		}
		
		
		
//		Map<Set<ASALSymbolicValue>, Set<PulsePackMap>> inputValsPerGuardSet = new HashMap<Set<ASALSymbolicValue>, Set<PulsePackMap>>();
//		
//		for (Map<ReprPort, ASALSymbolicValue> inputVal : HashMaps.allCombinations(source.pvsPerInput)) {
//			Set<ASALSymbolicValue> guardSet = new HashSet<ASALSymbolicValue>();
//			
//			for (ASALSymbolicValue guard : guards) {
//				if (guard.substitute(inputVal).toBoolean()) {
//					guardSet.add(guard);
//				}
//			}
//			
//			HashMaps.inject(inputValsPerGuardSet, guardSet, PulsePackMap.from(inputVal, Dir.IN));
//		}
		
		
		
		
		
		transitions = new HashSet<DecaTritoTransition>();
		
		for (DecaTritoVertex v : vertices.values()) {
			Map<PulsePackMap, Set<DecaTritoVertex>> tgtsPerInputVal = new HashMap<PulsePackMap, Set<DecaTritoVertex>>();
			
			for (DecaTwoTransition t : v.getLegacy().getOutgoing()) {
				for (PulsePackMap i : inputValsPerGuard.get(t.getGuard())) {
					DecaTritoVertex tgt = vertices.get(legacy.vertices.get(t.getTargetStateConfig()));
					HashMaps.inject(tgtsPerInputVal, i, tgt);
				}
			}
			
//			if (USE_PRIORITIES) {
//				for (Map.Entry<PulsePackMap, Set<DecaTwoBVertex>> e : tgtsPerInputVal.entrySet()) {
//					Set<DecaTwoBVertex> updatedTgts = tgtsPerInputVal.get(e.getKey().prioritize());
//					
//					if (updatedTgts == null) {
//						throw new Error("Should not happen!");
//					}
//					
//					e.setValue(updatedTgts);
//				}
//			}
			
			Map<Set<PulsePackMap>, Set<DecaTritoVertex>> tgtsPerInputVals = HashMaps.mergeKeysByValues(tgtsPerInputVal);
			
			for (Map.Entry<Set<PulsePackMap>, Set<DecaTritoVertex>> e : tgtsPerInputVals.entrySet()) {
				Set<DecaTwoTransition> legacyTrs = new HashSet<DecaTwoTransition>();
				
				for (DecaTwoTransition t : v.getLegacy().getOutgoing()) {
					DecaTritoVertex tgt = vertices.get(legacy.vertices.get(t.getTargetStateConfig()));
					
					if (e.getValue().contains(tgt)) {
						if (!Collections.disjoint(e.getKey(), inputValsPerGuard.get(t.getGuard()))) {
							legacyTrs.add(t);
						}
					}
				}
				
				transitions.add(new DecaTritoTransition(legacyTrs, v, e.getValue(), e.getKey()));
			}
			
//			Map<PulsePackMap, Set<DecaTwoTransition>> targetsPerInputVal = new HashMap<PulsePackMap, Set<DecaTwoTransition>>();
//			
//			for (DecaTwoTransition t : v.getLegacy().getOutgoing()) {
//				for (PulsePackMap i : inputValsPerGuard.get(t.getGuard())) {
//					HashMaps.inject(targetsPerInputVal, i, t);
//				}
//			}
//			
//			Map<PulsePackMap, Set<DecaTwoTransition>> prioritizedTargetsPerInputVal = new HashMap<PulsePackMap, Set<DecaTwoTransition>>();
//			prioritizedTargetsPerInputVal.putAll(targetsPerInputVal);
//			
////			for (Map.Entry<PulsePackMap, Set<DecaTwoTransition>> e : targetsPerInputVal.entrySet()) {
////				if (e.getValue().size() == 1) {
////					prioritizedTargetsPerInputVal.put(e.getKey(), e.getValue()); //TODO why?
////				} else {
////					Set<DecaTwoTransition> temp = targetsPerInputVal.get(e.getKey().prioritize());
////					
////					if (temp == null) {
////						throw new Error("Should not happen!");
////					}
////					
////					prioritizedTargetsPerInputVal.put(e.getKey(), temp);
////				}
////			}
//			
//			Map<Set<DecaTwoTransition>, Set<PulsePackMap>> inputValPerTrSet = new HashMap<Set<DecaTwoTransition>, Set<PulsePackMap>>();
//			
//			for (Map.Entry<PulsePackMap, Set<DecaTwoTransition>> e : prioritizedTargetsPerInputVal.entrySet()) {
//				HashMaps.inject(inputValPerTrSet, e.getValue(), e.getKey());
//			}
//			
//			for (Map.Entry<Set<DecaTwoTransition>, Set<PulsePackMap>> e : inputValPerTrSet.entrySet()) {
//				Set<DecaTwoBVertex> tgts = new HashSet<DecaTwoBVertex>();
//				
//				for (DecaTwoTransition t : e.getKey()) {
//					tgts.add(vertices.get(legacy.vertices.get(t.getTargetStateConfig())));
//				}
//				
//				transitions.add(new DecaTwoBTransition(e.getKey(), v, tgts, e.getValue()));
//			}
		}
		
		for (DecaTritoTransition t : transitions) {
			t.getSrc().getOutgoing().add(t);
		}
		
//		if (scope.getName().equals("p3")) {
//			List<String> elems = new ArrayList<String>();
//			
//			for (Map.Entry<ReprPort, Set<ASALSymbolicValue>> e : source.pvsPerInput.entrySet()) {
//				elems.add(e.getKey().getName() + " (x" + e.getValue().size() + ")");
//			}
//			
//			Collections.sort(elems);
//			
//			for (String elem : elems) {
//				System.out.println(elem);
//			}
//			
//			System.out.println("#vertices = " + vertices.size());
//			System.out.println("#transitions = " + transitions.size());
//			System.out.println("#allInputs = " + allInputs.size());
//			
//			System.out.println("#inputValsPerGuard = " + inputValsPerGuard.size());
//			System.out.println("pvsPerInput.#elems = " + source.pvsPerInput.size());
//			System.out.println("pvsPerInput.#perms = " + HashMaps.allCombinations(source.pvsPerInput).size());
//			
//			
//			System.exit(0);
//		}
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(initProtoTransitions);
		result.addAll(transitions);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	public Set<ProtoTransition> getAllProtoTransitions() {
		Set<ProtoTransition> result = new HashSet<ProtoTransition>();
		result.addAll(initProtoTransitions);
		
		for (DecaTritoTransition t : transitions) {
			result.addAll(t.getProtoTrs());
		}
		
		return result;
	}
	
	public void printGraphvizFile() {
		try {
			PrintStream ps = new PrintStream("deca-trito-" + scope.getName() + ".gv");
			printGraphvizFile(ps, transitions);
			ps.flush();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
	
	public void printGraphvizFile(PrintStream out, Set<DecaTritoTransition> trs) {
		Map<DecaTritoVertex, String> namePerVertex = new HashMap<DecaTritoVertex, String>();
		Map<DecaTritoTransition, String> namePerTransition = new HashMap<DecaTritoTransition, String>();
		Map<DecaTritoTransition, String> colorPerTransition = new HashMap<DecaTritoTransition, String>();
		Set<DecaTritoVertex> vtxs = new HashSet<DecaTritoVertex>();
		
		for (DecaTritoTransition t : trs) {
			vtxs.add(t.getSrc());
			vtxs.addAll(t.getTgts());
		}
		
		for (DecaTritoVertex v : vtxs) {
			namePerVertex.put(v, "V" + namePerVertex.size());
		}
		
		for (DecaTritoTransition t : trs) {
			namePerTransition.put(t, "T" + namePerTransition.size());
			colorPerTransition.put(t, Dot.getRandomColor());
		}
		
		TextOptions.select(TextOptions.GRAPHVIZ_MIN);
		
		out.println("// " + getClass().getCanonicalName());
		out.println("// " + scope.getName());
		out.println("digraph G {");
		
		{
			String s = "(initial)";
			
			for (Map.Entry<ASALVariable, ASALSymbolicValue> entry : initialVertex.getLegacy().getStateConfig().getValuation().entrySet()) {
				s += "\\n" + entry.getKey().getName() + " := " + entry.getValue().toString();
			}
			
			//out.println("\tI0 [label=\"" + s + "\", shape=circle, fontsize=10];");
			out.println("\tI0 [label=\"" + s + "\", shape=rectangle, fontsize=10];");
		}
		
		for (DecaTritoVertex v : vtxs) {
			String s = v.getName();
			
			for (Map.Entry<ASALVariable, ASALSymbolicValue> entry : v.getLegacy().getStateConfig().getValuation().entrySet()) {
				if (!JPulse.class.isAssignableFrom(entry.getKey().getType()) || entry.getValue().toBoolean()) {
					s += "\\n" + TextOptions.current().id(entry.getKey().getName()) + " := " + entry.getValue().toString();
				}
			}
			
			//out.println("\t" + namePerVertex.get(v) + " [label=\"" + s + "\", shape=ellipse];");
			out.println("\t" + namePerVertex.get(v) + " [label=\"" + s + "\", shape=rectangle];");
		}
		
		if ("points".equals("points")) {
			for (DecaTritoTransition t : trs) {
				out.println("\t" + namePerTransition.get(t) + " [label=\"\", shape=point, color=\"" + colorPerTransition.get(t) + "\", fontsize=10, fontcolor=\"" + colorPerTransition.get(t) + "\"];");
			}
		} else {
			for (DecaTritoTransition t : trs) {
				Set<PulsePackMap> localInputs = new HashSet<PulsePackMap>();
				
				for (DecaTritoTransition x : t.getSrc().getOutgoing()) {
					localInputs.addAll(x.getInputVals());
				}
				
				Set<PulsePackMap> zz = new HashSet<PulsePackMap>();
				
				for (PulsePackMap i : t.getInputVals()) {
					zz.add(i.extractMultiplePvsMap());
				}
				
				String s = "#vals = " + t.getInputVals().size() + " / " + localInputs.size() + " / " + allInputs.size();
				
				if (zz.size() <= 12) {
					for (PulsePackMap i : zz) {
						s += "\\n" + i.toString();
					}
				}
				
				out.println("\t" + namePerTransition.get(t) + " [label=\"" + s + "\", shape=rectangle, color=\"" + colorPerTransition.get(t) + "\", fontsize=10, fontcolor=\"" + colorPerTransition.get(t) + "\"];");
			}
		}
		
		for (DecaTritoTransition t : trs) {
			String t_name = namePerTransition.get(t);
			String sv_name = namePerVertex.get(t.getSrc());
			
			out.println("\t" + sv_name + " -> " + t_name + " [color=\"" + colorPerTransition.get(t) + "\"];");
			
			for (DecaTritoVertex tgt : t.getTgts()) {
				String tv_name = namePerVertex.get(tgt);
				out.println("\t" + t_name + " -> " + tv_name + " [color=\"" + colorPerTransition.get(t) + "\"];");
			}
		}
		
		if (vtxs.contains(initialVertex)) {
			out.println("\tI0 -> " + namePerVertex.get(initialVertex) + ";");
		}
		
		out.println("}");
	}
}

