package jlx.behave.proto;

import java.util.*;

import jlx.common.FileLocation;
import jlx.utils.Texts;

public class DecaTritoTransition implements LineageObject {
	private Set<DecaTwoTransition> legacy;
	private DecaTritoVertex src;
	private Set<DecaTritoVertex> tgts;
	private Set<PulsePackMap> inputVals;
	private Set<ProtoTransition> protoTrs;
	
	public DecaTritoTransition(Set<DecaTwoTransition> legacy, DecaTritoVertex src, Set<DecaTritoVertex> tgts, Set<PulsePackMap> inputVals) {
		this.legacy = legacy;
		this.src = src;
		this.tgts = tgts;
		this.inputVals = inputVals;
		
		protoTrs = new HashSet<ProtoTransition>();
		
		for (DecaTwoTransition t : legacy) {
			protoTrs.addAll(t.getProtoTrs());
		}
	}
	
	public Set<DecaTwoTransition> getLegacy() {
		return legacy;
	}
	
	public List<FileLocation> getFileLocations() {
		List<FileLocation> result = new ArrayList<FileLocation>();
		
		for (DecaTwoTransition t : legacy) {
			result.addAll(t.getFileLocations());
		}
		
		return result;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(legacy);
		result.add(src);
		
//		if (tgts.size() == 1) {
			result.addAll(tgts);
//		}
		
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	public DecaTritoVertex getSrc() {
		return src;
	}
	
	public Set<DecaTritoVertex> getTgts() {
		return tgts;
	}
	
	public Set<PulsePackMap> getInputVals() {
		return inputVals;
	}
	
	public Set<ProtoTransition> getProtoTrs() {
		return protoTrs;
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + src.toString() + "]";
		result += "---" + Texts.concat(getFileLocations(), ";", (x) -> { return x.tiny(); }) + "-->";
		result += "[" + Texts.concat(tgts, "+") + "]";
		return result;
	}
}

