package jlx.behave.proto;

import java.util.*;

import jlx.asal.parsing.api.*;
import jlx.common.FileLocation;
import jlx.utils.Texts;

public class OctoTransition implements LineageObject {
	private OctoStateMachine owner;
	private OctoStateConfig sourceStateConfig;
	private OctoStateConfig targetStateConfig;
	private ASALEvent event;
	private ASALExpr guard;
	private List<ASALStatement> effectSeq;
	private List<SeptaTransition> legacy;
	private Set<TritoTransition> protoTrs;
	
	public OctoTransition(OctoStateMachine owner, OctoStateConfig sourceStateConfig, SeptaTransition used) {
		this(owner, sourceStateConfig, used.getGuard(), Collections.singletonList(used), Collections.emptySet());
	}
	
	public OctoTransition(OctoStateMachine owner, OctoStateConfig sourceStateConfig, ASALExpr guard, List<SeptaTransition> used, Set<SeptaTransition> unused) {
		this.owner = owner;
		this.sourceStateConfig = sourceStateConfig;
		this.guard = guard;
		
		Set<SeptaVertex> states = new HashSet<SeptaVertex>(sourceStateConfig.states);
		
		for (SeptaTransition t : used) {
			states.removeAll(t.getMaxParallelVertices()); //Was t.getSourceVertices(), does not cover all circumstances.
		}
		
		for (SeptaTransition t : used) {
			states.addAll(t.getTargetVertices());
		}
		
		List<ASALStatement> effectElems = new ArrayList<ASALStatement>();
		
		for (SeptaTransition t : used) {
			for (ConditionalExitBehavior<SeptaVertex> ceb : t.getEffectPerm()) {
				if (ceb.getEffect().containsNonEmpty()) {
					if (ceb.overlapsWith(sourceStateConfig.states)) {
						effectElems.add(ceb.getEffect());
					}
				}
			}
		}
		
		targetStateConfig = new OctoStateConfig(states);
		event = removeFinalizedEvent(used.get(0).getEvent());
		effectSeq = Collections.unmodifiableList(effectElems);
		legacy = used;
		
		protoTrs = new HashSet<TritoTransition>();
		
		for (SeptaTransition t : used) {
			for (SeptaVertex srcVtx : sourceStateConfig.states) {
				Set<TritoTransition> temp = t.getProtoTrsPerSrcVertex().get(srcVtx);
				
				if (temp != null) {
					protoTrs.addAll(temp);
				}
			}
			
			protoTrs.addAll(t.getEnRouteProtoTrs());
			
			for (SeptaVertex tgtVtx : targetStateConfig.states) {
				Set<TritoTransition> temp = t.getProtoTrsPerTgtVertex().get(tgtVtx);
				
				if (temp != null) {
					protoTrs.addAll(temp);
				}
			}
		}
	}
	
	private static ASALEvent removeFinalizedEvent(ASALEvent e) {
		return e instanceof ASALFinalized ? null : e;
	}
	
	public List<SeptaTransition> getLegacy() {
		return legacy;
	}
	
	public List<FileLocation> getFileLocations() {
		List<FileLocation> result = new ArrayList<FileLocation>();
		
		for (SeptaTransition t : legacy) {
			result.addAll(t.getFileLocations());
		}
		
		return result;
	}
	
	public OctoStateConfig getSourceStateConfig() {
		return sourceStateConfig;
	}
	
	public OctoStateConfig getTargetStateConfig() {
		return targetStateConfig;
	}
	
	public ASALEvent getEvent() {
		return event;
	}
	
	public ASALExpr getGuard() {
		return guard;
	}
	
	public List<ASALStatement> getEffectSeq() {
		return effectSeq;
	}
	
	public Set<TritoTransition> getProtoTrs() {
		return protoTrs;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(legacy);
		result.add(owner.vertices.get(sourceStateConfig));
		result.add(owner.vertices.get(targetStateConfig));
		result.addAll(protoTrs);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + owner.vertices.get(sourceStateConfig).toString() + "]";
		result += "---" + Texts.concat(getFileLocations(), ";", (x) -> { return x.tiny(); }) + "-->";
		result += "[" + owner.vertices.get(targetStateConfig).toString() + "]";
		return result;
	}
}



