package jlx.behave.proto;

import java.util.*;

import jlx.asal.parsing.api.*;
import jlx.common.FileLocation;
import jlx.utils.*;

public class PentaTransition implements LineageObject {
	private Set<PentaVertex> sourceVertices;
	private Set<PentaVertex> maxParallelVertices;
	private Set<PentaVertex> targetVertices;
	private Set<List<ConditionalExitBehavior<PentaVertex>>> effectPerms;
	private ASALEvent event;
	private ASALExpr guard;
	private TetraTransition legacy;
	
	private Set<TritoTransition> enRouteProtoTrs;
	private Map<PentaVertex, Set<TritoTransition>> protoTrsPerSrcVertex;
	private Map<PentaVertex, Set<TritoTransition>> protoTrsPerTgtVertex;
	
	public PentaTransition(TetraTransition source, Map<TetraVertex, PentaVertex> newVertexPerOldVertex) {
		sourceVertices = HashSets.map(source.getSourceVertices(), newVertexPerOldVertex);
		maxParallelVertices = HashSets.map(source.getMaxParallelVertices(), newVertexPerOldVertex);
		targetVertices = HashSets.map(source.getTargetVertices(), newVertexPerOldVertex);
		event = source.getEvent();
		guard = source.getGuard();
		legacy = source;
		effectPerms = new HashSet<List<ConditionalExitBehavior<PentaVertex>>>();
		
		enRouteProtoTrs = new HashSet<TritoTransition>(source.getEnRouteProtoTrs());
		protoTrsPerSrcVertex = new HashMap<PentaVertex, Set<TritoTransition>>();
		
		for (Map.Entry<TetraVertex, Set<TritoTransition>> e : source.getProtoTrsPerSrcVertex().entrySet()) {
			protoTrsPerSrcVertex.put(newVertexPerOldVertex.get(e.getKey()), new HashSet<TritoTransition>(e.getValue()));
		}
		
		protoTrsPerTgtVertex = new HashMap<PentaVertex, Set<TritoTransition>>();
		
		for (Map.Entry<TetraVertex, Set<TritoTransition>> e : source.getProtoTrsPerTgtVertex().entrySet()) {
			protoTrsPerTgtVertex.put(newVertexPerOldVertex.get(e.getKey()), new HashSet<TritoTransition>(e.getValue()));
		}
		
		for (List<ConditionalExitBehavior<TetraVertex>> exitPerm : source.getExitBehaviorInOrder()) {
			for (List<ASALStatement> entryPerm : source.getEntryBehaviorInOrder()) {
				List<ConditionalExitBehavior<PentaVertex>> newEffectPerm = new ArrayList<ConditionalExitBehavior<PentaVertex>>();
				
				for (ConditionalExitBehavior<TetraVertex> e : exitPerm) {
					newEffectPerm.add(e.changeKey(newVertexPerOldVertex));
				}
				
				newEffectPerm.add(new ConditionalExitBehavior<PentaVertex>("EFFECT " + getFileLocation().tiny(), Collections.emptySet(), source.getStatement()));
				
				for (ASALStatement e : entryPerm) {
					newEffectPerm.add(new ConditionalExitBehavior<PentaVertex>("ENTRY " + e.getFileLocation().tiny(), Collections.emptySet(), e));
				}
				
				effectPerms.add(newEffectPerm);
			}
		}
	}
	
	public TetraTransition getLegacy() {
		return legacy;
	}
	
	public FileLocation getFileLocation() {
		return legacy.getFileLocation();
	}
	
	public ASALEvent getEvent() {
		return event;
	}
	
	public ASALExpr getGuard() {
		return guard;
	}
	
	public Set<List<ConditionalExitBehavior<PentaVertex>>> getEffectPerms() {
		return effectPerms;
	}
	
	public Set<PentaVertex> getSourceVertices() {
		return sourceVertices;
	}
	
	public Set<PentaVertex> getMaxParallelVertices() {
		return maxParallelVertices;
	}
	
	public Set<PentaVertex> getTargetVertices() {
		return targetVertices;
	}
	
	public Set<TritoTransition> getEnRouteProtoTrs() {
		return enRouteProtoTrs;
	}
	
	public Map<PentaVertex, Set<TritoTransition>> getProtoTrsPerSrcVertex() {
		return protoTrsPerSrcVertex;
	}

	public Map<PentaVertex, Set<TritoTransition>> getProtoTrsPerTgtVertex() {
		return protoTrsPerTgtVertex;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.add(legacy);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(sourceVertices);
		result.addAll(targetVertices);
		
		for (Set<TritoTransition> vs : getProtoTrsPerSrcVertex().values()) {
			result.addAll(vs);
		}
		
		for (Set<TritoTransition> vs : getProtoTrsPerTgtVertex().values()) {
			result.addAll(vs);
		}
		
		return result;
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + Texts.concat(sourceVertices, "+") + "]";
		result += "---" + legacy.getFileLocation().tiny() + "-->";
		result += "[" + Texts.concat(targetVertices, "+") + "]";
		return result;
	}
}

