package jlx.behave.proto;

import java.util.*;

import jlx.asal.j.*;
import jlx.asal.parsing.*;
import jlx.behave.CodeObject;
import jlx.common.FileLocation;
import jlx.common.reflection.ModelException;

public class ProtoTransition implements LineageObject {
	public final ProtoStateMachine stateMachine;
	public final ProtoVertex sourceState;
	public final ProtoVertex targetState;
	public final CodeObject legacy;
	public final String type;
	public final boolean isLocal;
	public final boolean isInitCode;
	
	public ProtoTransition(ProtoStateMachine stateMachine, ProtoVertex sourceState, ProtoVertex targetState, CodeObject codeObject, String type, boolean isLocal) {
		this.stateMachine = stateMachine;
		this.sourceState = sourceState;
		this.targetState = targetState;
		this.type = type;
		this.isLocal = isLocal;
		
		if (sourceState != null) {
			isInitCode = stateMachine.initialVertices.contains(sourceState);
		} else {
			isInitCode = false;
		}
		
		legacy = codeObject;
	}
	
	public FileLocation getFileLocation() {
		return legacy.getFileLocation();
	}
	
	public Set<ProtoVertex> getProtoVertices() {
		Set<ProtoVertex> result = new HashSet<ProtoVertex>();
		
		if (sourceState != null) {
			result.add(sourceState);
			result.addAll(sourceState.getContainers());
		}
		
		if (targetState != null) {
			result.add(targetState);
			result.addAll(targetState.getContainers());
		}
		
		return result;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		
		if (sourceState != null) {
			result.add(sourceState);
		}
		
		if (targetState != null) {
			result.add(targetState);
		}
		
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	public DeuteroTransition parse(JScope scope) throws ModelException {
		try {
			ASALSyntaxTree tree = legacy.treeObject.toSyntaxTree(scope);
			DeuteroTransition result = tree.createAPI(null, DeuteroTransition.class);
			result.validateAndCrossRef(scope, JVoid.class);
			result.setLegacy(this);
			return result;
		} catch (ASALException e) {
			throw new ModelException(legacy.getFileLocation(), e);
		}
	}
	
	@Override
	public String toString() {
		String result = "";
		
		if (sourceState != null) {
			result += "[" + sourceState.toString() + "]";
		} else {
			result += "[??]";
		}
		
		if (type != null && !type.isEmpty()) {
			result += "---" + legacy.getFileLocation().tiny() + "---" + type + "-->";
		} else {
			result += "---" + legacy.getFileLocation().tiny() + "-->";
		}
		
		if (targetState != null) {
			result += "[" + targetState.toString() + "]";
		} else {
			result += "[??]";
		}
		
		return result;
	}
}
