package jlx.behave.proto;

import java.util.*;

import jlx.asal.parsing.api.*;
import jlx.common.FileLocation;
import jlx.utils.*;

public class SeptaTransition implements LineageObject {
	private Set<SeptaVertex> sourceVertices;
	private Set<SeptaVertex> maxParallelVertices;
	private Set<SeptaVertex> targetVertices;
	private List<ConditionalExitBehavior<SeptaVertex>> effectPerm;
	private int legacyPermIndex;
	private ASALEvent event;
	private ASALExpr guard;
	private HexaTransition legacy;
	
	private Set<TritoTransition> enRouteProtoTrs;
	private Map<SeptaVertex, Set<TritoTransition>> protoTransitionsPerSrcVertex;
	private Map<SeptaVertex, Set<TritoTransition>> protoTransitionsPerTgtVertex;
	
	public SeptaTransition(HexaTransition source, int permIndex, List<ConditionalExitBehavior<HexaVertex>> pickedEffectPerm, Map<HexaVertex, SeptaVertex> newVertexPerOldVertex) {
		sourceVertices = HashSets.map(source.getSourceVertices(), newVertexPerOldVertex);
		maxParallelVertices = HashSets.map(source.getMaxParallelVertices(), newVertexPerOldVertex);
		targetVertices = HashSets.map(source.getTargetVertices(), newVertexPerOldVertex);
		effectPerm = translate(pickedEffectPerm, source.getSourceVertices(), newVertexPerOldVertex);
		event = source.getEvent();
		guard = source.getGuard();
		legacy = source;
		legacyPermIndex = permIndex;
		
		enRouteProtoTrs = new HashSet<TritoTransition>(source.getEnRouteProtoTrs());
		protoTransitionsPerSrcVertex = new HashMap<SeptaVertex, Set<TritoTransition>>();
		
		for (Map.Entry<HexaVertex, Set<TritoTransition>> e : source.getProtoTrsPerSrcVertex().entrySet()) {
			protoTransitionsPerSrcVertex.put(newVertexPerOldVertex.get(e.getKey()), e.getValue());
		}
		
		protoTransitionsPerTgtVertex = new HashMap<SeptaVertex, Set<TritoTransition>>();
		
		for (Map.Entry<HexaVertex, Set<TritoTransition>> e : source.getProtoTrsPerTgtVertex().entrySet()) {
			protoTransitionsPerTgtVertex.put(newVertexPerOldVertex.get(e.getKey()), e.getValue());
		}
	}
	
	private static List<ConditionalExitBehavior<SeptaVertex>> translate(List<ConditionalExitBehavior<HexaVertex>> xs, Set<HexaVertex> sourceVertices, Map<HexaVertex, SeptaVertex> newVertexPerOldVertex) {
		List<ConditionalExitBehavior<SeptaVertex>> result = new ArrayList<ConditionalExitBehavior<SeptaVertex>>();
		
		for (ConditionalExitBehavior<HexaVertex> x : xs) {
			if (x.getEffect().containsNonEmpty()) {
//				if (x.getExitedVertices().equals(sourceVertices)) {
//					//TODO I think this is wrong:
//					result.add(new ConditionalExitBehavior<SeptaVertex>(x.getDebugText(), Collections.emptySet(), x.getEffect()));
//				} else {
//					result.add(x.changeKey(newVertexPerOldVertex));
//				}
				
				result.add(x.changeKey(newVertexPerOldVertex));
			}
		}
		
		return result;
	}
	
	public HexaTransition getLegacy() {
		return legacy;
	}
	
	public List<FileLocation> getFileLocations() {
		return legacy.getFileLocations(legacyPermIndex);
	}
	
	public ASALEvent getEvent() {
		return event;
	}
	
	public ASALExpr getGuard() {
		return guard;
	}
	
	public List<ConditionalExitBehavior<SeptaVertex>> getEffectPerm() {
		return effectPerm;
	}
	
	public Set<SeptaVertex> getSourceVertices() {
		return sourceVertices;
	}
	
	public Set<SeptaVertex> getMaxParallelVertices() {
		return maxParallelVertices;
	}
	
	public Set<SeptaVertex> getTargetVertices() {
		return targetVertices;
	}
	
	public Set<TritoTransition> getEnRouteProtoTrs() {
		return enRouteProtoTrs;
	}
	
	public Map<SeptaVertex, Set<TritoTransition>> getProtoTrsPerSrcVertex() {
		return protoTransitionsPerSrcVertex;
	}
	
	public Map<SeptaVertex, Set<TritoTransition>> getProtoTrsPerTgtVertex() {
		return protoTransitionsPerTgtVertex;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(enRouteProtoTrs);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(sourceVertices);
		result.addAll(targetVertices);
		
		for (Set<TritoTransition> vs : getProtoTrsPerSrcVertex().values()) {
			result.addAll(vs);
		}
		
		for (Set<TritoTransition> vs : getProtoTrsPerTgtVertex().values()) {
			result.addAll(vs);
		}
		
		return result;
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + Texts.concat(sourceVertices, "+") + "]";
		result += "---" + Texts.concat(getFileLocations(), ";", (x) -> { return x.tiny(); }) + "-->";
		result += "[" + Texts.concat(targetVertices, "+") + "]";
		return result;
	}
}
