package jlx.behave.proto;

import java.util.*;

import jlx.asal.parsing.api.*;
import jlx.common.FileLocation;
import jlx.utils.Texts;

public class TetraTransition implements LineageObject {
	private Set<TetraVertex> sourceVertices;
	private Set<TetraVertex> maxParallelVertices;
	private Set<TetraVertex> targetVertices;
	private Set<List<ConditionalExitBehavior<TetraVertex>>> exitBehaviorInOrder;
	private Set<List<ASALStatement>> entryBehaviorInOrder;
	
	private Set<TritoTransition> enRouteProtoTrs;
	private Map<TetraVertex, Set<TritoTransition>> protoTrsPerSrcVertex;
	private Map<TetraVertex, Set<TritoTransition>> protoTrsPerTgtVertex;
	
	/**
	 * Timeout/null: occur non-deterministically
	 * Finalized: occur when final states have been reached, i.e. intersection(sourceVertices, stateConfig) may not contain final states
	 * Trigger: occur when trigger is detected
	 * Call: ... (no support yet) ...
	 */
	private ASALEvent event;
	private ASALExpr guard;
	private ASALStatement statement;
	private TritoTransition legacy;
	
	/**
	 * Should only be used for local (= entry/exit/do) transitions!
	 */
	public TetraTransition(TetraVertex localVertex, TritoTransition source) {
		this(Collections.singleton(localVertex), Collections.singleton(localVertex), Collections.singleton(localVertex), Collections.singleton(Collections.emptyList()), Collections.singleton(Collections.emptyList()), source, Collections.emptyMap(), Collections.emptyMap());
	}
	
	public TetraTransition(Set<TetraVertex> sourceVertices, Set<TetraVertex> maxParallelVertices, Set<TetraVertex> targetVertices, Set<List<ConditionalExitBehavior<TetraVertex>>> exitBehaviorInOrder, Set<List<ASALStatement>> entryBehaviorInOrder, TritoTransition source, Map<TetraVertex, Set<TritoTransition>> protoTrsPerSrcVertex, Map<TetraVertex, Set<TritoTransition>> protoTrsPerTgtVertex) {
		this.sourceVertices = sourceVertices;
		this.maxParallelVertices = maxParallelVertices;
		this.targetVertices = targetVertices;
		this.exitBehaviorInOrder = exitBehaviorInOrder;
		this.entryBehaviorInOrder = entryBehaviorInOrder;
		
		this.enRouteProtoTrs = Collections.singleton(source);
		this.protoTrsPerSrcVertex = protoTrsPerSrcVertex;
		this.protoTrsPerTgtVertex = protoTrsPerTgtVertex;
		
		event = source.getEvent();
		guard = source.getGuard();
		statement = source.getStatement();
		legacy = source;
	}
	
	public TritoTransition getLegacy() {
		return legacy;
	}
	
	public FileLocation getFileLocation() {
		return legacy.getFileLocation();
	}
	
	public ASALEvent getEvent() {
		return event;
	}
	
	public ASALExpr getGuard() {
		return guard;
	}
	
	public ASALStatement getStatement() {
		return statement;
	}
	
	public Set<TetraVertex> getSourceVertices() {
		return sourceVertices;
	}
	
	public Set<TetraVertex> getMaxParallelVertices() {
		return maxParallelVertices;
	}
	
	public Set<TetraVertex> getTargetVertices() {
		return targetVertices;
	}
	
	public Set<TritoTransition> getEnRouteProtoTrs() {
		return enRouteProtoTrs;
	}
	
	public Map<TetraVertex, Set<TritoTransition>> getProtoTrsPerSrcVertex() {
		return protoTrsPerSrcVertex;
	}
	
	public Map<TetraVertex, Set<TritoTransition>> getProtoTrsPerTgtVertex() {
		return protoTrsPerTgtVertex;
	}
	
	/**
	 * All possible orderings of (conditional) exit behavior.
	 * Never empty, even if it contains only an empty list.
	 */
	public Set<List<ConditionalExitBehavior<TetraVertex>>> getExitBehaviorInOrder() {
		return exitBehaviorInOrder;
	}
	
	/**
	 * All possible orderings of entry behavior.
	 * Never empty, even if it contains only an empty list.
	 */
	public Set<List<ASALStatement>> getEntryBehaviorInOrder() {
		return entryBehaviorInOrder;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(enRouteProtoTrs);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(sourceVertices);
		result.addAll(targetVertices);
		
		for (Set<TritoTransition> vs : getProtoTrsPerSrcVertex().values()) {
			result.addAll(vs);
		}
		
		for (Set<TritoTransition> vs : getProtoTrsPerTgtVertex().values()) {
			result.addAll(vs);
		}
		
		return result;
	}
	
	@Override
	public String toString() {
		String result = "";
		result += "[" + Texts.concat(sourceVertices, "+") + "]";
		result += "---" + legacy.getFileLocation().tiny() + "-->";
		result += "[" + Texts.concat(targetVertices, "+") + "]";
		return result;
	}
}
