package jlx.behave.proto.gui;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.*;

import jlx.behave.proto.*;
import jlx.behave.stable.DecaStableTransition;
import jlx.common.FileLocation;
import jlx.scenario.*;

public class DecaFourScenarioLog {
	private final Scenario scenario;
	private final boolean useAltSteps;
	private final List<String> lines;
	private final List<Set<DecaFourStateConfig>> potCfgsSeq;
	private final Set<DecaFourStateConfig> foundCfgs;
	private final DecaFourCoverageStats coverageStats;
	private final List<DecaStableTransition> stableTrs;
	private final List<PulsePackMap> inputValSeq;
	private final Set<DecaFourScenarioMismatch> mismatches;
	
	private FileLocation stepFileLocation;
	private int noStableTrCount;
	private int nonDetTrCount;
	
	public DecaFourScenarioLog(Scenario scenario, boolean useAltSteps, DecaFourStateConfig firstCfg) {
		this.scenario = scenario;
		this.useAltSteps = useAltSteps;
		
		lines = new ArrayList<String>();
		potCfgsSeq = new ArrayList<Set<DecaFourStateConfig>>();
		potCfgsSeq.add(Collections.singleton(firstCfg));
		foundCfgs = new HashSet<DecaFourStateConfig>();
		coverageStats = new DecaFourCoverageStats();
		stableTrs = new ArrayList<DecaStableTransition>();
		inputValSeq = new ArrayList<PulsePackMap>();
		mismatches = new HashSet<DecaFourScenarioMismatch>();
		
		noStableTrCount = 0;
		nonDetTrCount = 0;
	}
	
	public Scenario getScenario() {
		return scenario;
	}
	
	public boolean useAltSteps() {
		return useAltSteps;
	}
	
	public List<Set<DecaFourStateConfig>> getPotCfgsSeq() {
		return potCfgsSeq;
	}
	
	public Set<DecaFourStateConfig> getFoundCfgs() {
		return foundCfgs;
	}
	
	public Set<DecaFourStateConfig> getLastPotCfgs() {
		return potCfgsSeq.get(potCfgsSeq.size() - 1);
	}
	
	public List<PulsePackMap> getInputValSeq() {
		return inputValSeq;
	}
	
	public List<String> getLines() {
		return lines;
	}
	
	public Set<DecaFourScenarioMismatch> getMismatches() {
		return mismatches;
	}
	
	public FileLocation getStepFileLocation() {
		return stepFileLocation;
	}
	
	public void setStepFileLocation(FileLocation stepFileLocation) {
		this.stepFileLocation = stepFileLocation;
	}
	
	public DecaFourCoverageStats getCoverageStats() {
		return coverageStats;
	}
	
	public List<DecaStableTransition> getStableTrs() {
		return stableTrs;
	}
	
	public void incNoStableTrCount() {
		noStableTrCount++;
	}
	
	public int getNoStableTrCount() {
		return noStableTrCount;
	}
	
	public void incNonDetTrCount() {
		nonDetTrCount++;
	}
	
	public int getNonDetTrCount() {
		return nonDetTrCount;
	}
	
	public void populateCfgPairs(List<DecaFourCfgPair> dest) {
		for (int index = 2; index < potCfgsSeq.size(); index++) {
			Set<DecaFourStateConfig> cfgs1 = potCfgsSeq.get(index - 1);
			Set<DecaFourStateConfig> cfgs2 = potCfgsSeq.get(index);
			PulsePackMap diff = inputValSeq.get(index - 1).extractEventMap(inputValSeq.get(index - 2), false);
			
			DecaFourStateConfig cfg1 = cfgs1.iterator().next();
			DecaFourStateConfig cfg2 = cfgs2.iterator().next();
			dest.add(new DecaFourCfgPair(cfg1, cfg2, diff));
		}
	}
	
	public Set<DecaFourCfgPair> getCfgPairs() {
		Set<DecaFourCfgPair> result = new HashSet<DecaFourCfgPair>();
		
		//unifyingBlock.sms4.initialInputs.extractExternalMap();
//		PulsePackMap prevInputVal = initInputVal.deactivate();
		
		for (int index = 2; index < potCfgsSeq.size(); index++) {
			Set<DecaFourStateConfig> cfgs1 = potCfgsSeq.get(index - 1);
			Set<DecaFourStateConfig> cfgs2 = potCfgsSeq.get(index);
			PulsePackMap diff = inputValSeq.get(index - 1).extractEventMap(inputValSeq.get(index - 2), false);
			
			for (DecaFourStateConfig cfg1 : cfgs1) {
				for (DecaFourStateConfig cfg2 : cfgs2) {
					result.add(new DecaFourCfgPair(cfg1, cfg2, diff));
				}
			}
		}
		
		return result;
	}
	
	public void saveToFile(String filename) {
		try {
			int slashIndex = filename.lastIndexOf("/");
			
			if (slashIndex >= 0) {
				String dirName = filename.substring(0, slashIndex);
				File dir = new File(dirName);
				dir.mkdir();
			}
			
			PrintStream out = new PrintStream(filename);
			
			for (String line : getLines()) {
				out.println(line);
			}
			
			out.close();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
}

