package jlx.behave.stable;

import java.util.*;

import jlx.behave.proto.*;

public class DecaStablePath {
	private final List<DecaStableVertex> vtxs;
	private final List<PulsePack> remainingPacks;
	private final DecaFourCoverageStats coverageStats;
	private final DecaStableVertex lastVtx;
	private final int hashCode;
	
	public DecaStablePath(DecaStablePath source, DecaFourCoverageStats sourceCoverageStats) {
		vtxs = new ArrayList<DecaStableVertex>(source.getVtxs());
		remainingPacks = new ArrayList<PulsePack>(source.getRemainingPacks());
		coverageStats = new DecaFourCoverageStats(sourceCoverageStats);
		lastVtx = source.getLastVtx();
		hashCode = extractHashCode();
	}
	
	public DecaStablePath(DecaStableVertex startVtx, List<PulsePack> packs) {
		vtxs = new ArrayList<DecaStableVertex>();
		vtxs.add(startVtx);
		remainingPacks = new ArrayList<PulsePack>(packs);
		coverageStats = new DecaFourCoverageStats();
		lastVtx = startVtx;
		hashCode = extractHashCode();
	}
	
	public DecaStablePath(DecaStablePath source, DecaStableVertex nextVtx, DecaFourCoverageStats nextCoverageStats, List<PulsePack> packs) {
		vtxs = new ArrayList<DecaStableVertex>(source.getVtxs());
		vtxs.add(nextVtx);
		remainingPacks = new ArrayList<PulsePack>(packs);
		coverageStats = new DecaFourCoverageStats(source.coverageStats);
		coverageStats.addAll(nextCoverageStats);
		lastVtx = nextVtx;
		hashCode = extractHashCode();
	}
	
	private int extractHashCode() {
		return Objects.hash(vtxs, remainingPacks, coverageStats);
	}
	
	public List<DecaStableVertex> getVtxs() {
		return vtxs;
	}
	
	public List<PulsePack> getRemainingPacks() {
		return remainingPacks;
	}
	
	public DecaFourCoverageStats getCoverageStats() {
		return coverageStats;
	}
	
	public DecaStableVertex getLastVtx() {
		return lastVtx;
	}
	
	@Override
	public int hashCode() {
		return hashCode;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DecaStablePath other = (DecaStablePath) obj;
		return Objects.equals(coverageStats, other.coverageStats) && Objects.equals(lastVtx, other.lastVtx) && Objects.equals(remainingPacks, other.remainingPacks) && Objects.equals(vtxs, other.vtxs);
	}
}

