package jlx.behave.stable;

import java.io.*;
import java.time.LocalTime;
import java.util.*;

import jlx.asal.j.JPulse;
import jlx.asal.rewrite.ASALSymbolicValue;
import jlx.asal.vars.ASALPort;
import jlx.asal.vars.ASALProperty;
import jlx.behave.proto.*;
import jlx.models.UnifyingBlock.ReprPort;
import jlx.utils.*;

public class DecaStableStateMachine implements LineageObject {
	private final DecaFourCoverageStats coverageStats;
	
	public final DecaFourStateMachines legacy;
	public final NormMap<DecaStableVertex> vertices;
	public final DecaStableTransition initialTransition;
	public final Set<DecaStableTransition> transitions;
	public final Map<ReprPort, ASALSymbolicValue> initialInputs;
	public final Map<ASALPort, ASALPort> durationPortPerTimeoutPort;
	public final Set<DecaFourStateConfig> coveredCfgs;
	public final Set<DecaFourStateConfig> coveredStableCfgs;
	public final Set<Pair<DecaFourVertex>> vtxPairs;
	public final Set<DecaFourVertex> vtxs;
	public final Set<Pair<DecaFourStateConfig>> cfgPairs;
//	public final Map<Integer, Integer> cfgCountPerLevel;
	
	private int totalStableVtxsCount;
	private int totalStableTrsCount;
	
	public DecaStableStateMachine(DecaFourStateMachines source, boolean explore) {
		legacy = source;
		
		System.out.println("111");
		
		initialInputs = extractInitialInputs();
		durationPortPerTimeoutPort = new HashMap<ASALPort, ASALPort>();
		
		for (Map.Entry<ASALPort, ASALPort> e : source.timeoutPortPerDurationPort.entrySet()) {
			durationPortPerTimeoutPort.put(e.getValue(), e.getKey());
		}
		
		PulsePackMap initialExternalInputs = source.initialInputs.extractExternalMap();
		
		System.out.println("222");
		
		coverageStats = new DecaFourCoverageStats();
//		coverageStats.add(source.initProtoTransitions);
		
		Set<DecaFourStateConfig> cfgs = new HashSet<DecaFourStateConfig>();
		List<DecaFourStateConfig> cfgSeq;
		
		if (explore) {
			DecaFourStateConfigPath p = DecaFourStateConfig.followInputs(Collections.singleton(source.initCfg), initialExternalInputs, null);
			cfgSeq = p.getCfgs();
			coverageStats.addAll(p.getCoverageStats());
		} else {
			cfgSeq = Collections.singletonList(source.initCfg);
		}
		
		cfgs.addAll(cfgSeq);
		
		DecaStableVertex v1 = new DecaStableVertex(cfgSeq.get(cfgSeq.size() - 1), initialExternalInputs, 1);
		initialTransition = new DecaStableTransition(null, v1, initialExternalInputs, null, cfgSeq, new DecaFourCoverageStats(), false);
		
//		Set<DecaStableVertex> result = new HashSet<DecaStableVertex>();
		
		vertices = new NormMap<DecaStableVertex>();
		vertices.add(v1);
		
		Set<DecaStableVertex> unexhausted = new HashSet<DecaStableVertex>();
		unexhausted.add(v1);
		
//		Set<DecaStableVertex> beenHere = new HashSet<DecaStableVertex>();
//		beenHere.add(v1);
		
//		Set<DecaFourStateConfig> stableCfgs = new HashSet<DecaFourStateConfig>();
		
//		cfgCountPerLevel = new TreeMap<Integer, Integer>();
		
		Set<DecaStableVertex> fringe = new HashSet<DecaStableVertex>();
		Set<DecaStableVertex> newFringe = new HashSet<DecaStableVertex>();
		fringe.add(v1);
		
		WallClock.reset();
		int primOutCount = 0;
		int secOutCount = 0;
//		int level = 0;
		
		System.out.println("333");
		
		int exploreCount = Integer.MAX_VALUE;
		
		while (explore && fringe.size() > 0) {
			while (fringe.size() > 0) {
				ConcurrentWork<DecaStableVertex, ConcurrentWorker> cw = new ConcurrentWork<DecaStableVertex, ConcurrentWorker>();
				
				for (int i = 1; i <= 4; i++) {
					ConcurrentWorker w = new ConcurrentWorker("#tgts", "#stats", "#prim-out", "#sec-out");
					w.set("#prim-out", 0);
					w.set("#sec-out", secOutCount);
					w.set("#stats", new DecaFourCoverageStats());
					w.set("#tgts", new NormMap<DecaStableVertex>());
					cw.getWorkers().add(w);
				}
				
				cw.apply(fringe, (v, worker) -> {
					int prims = v.populatePrimaryOutgoing(legacy);
					worker.set("#prim-out", worker.get("#prim-out", Integer.class) + prims);
					
					for (DecaStableTransition t : v.getPrimaryOutgoing()) {
						worker.get("#stats", DecaFourCoverageStats.class).addAll(t.getCoverageStats());
						worker.getSet("#tgts", DecaStableVertex.class).add(t.getTgt());
					}
				});
				
				newFringe.clear();
				
				for (ConcurrentWorker w : cw.getWorkers()) {
					primOutCount += w.get("#prim-out", Integer.class);
					coverageStats.addAll(w.get("#stats", DecaFourCoverageStats.class));
					
					for (DecaStableVertex tgt : w.getSet("#tgts", DecaStableVertex.class)) {
						if (vertices.add(tgt)) {
							unexhausted.add(tgt);
							newFringe.add(tgt);
							cfgs.add(tgt.getCfg());
						}
					}
				}
				
				cw.apply(fringe, (v, worker) -> {
					v.normalizeSuccs(vertices);
				});
				
				fringe.clear();
				fringe.addAll(newFringe);
				
				System.out.println("[stable-prim-" + exploreCount + "][" + LocalTime.now() + "] #vtxs = " + vertices.size() + " (+" + fringe.size() + "); #cfgs = " + cfgs.size() + "; #prim-out = " + primOutCount + "; #sec-out = " + secOutCount);
			}
			
//			if ("no-timing".equals("no-timing")) {
//				break;
//			}
			
			
			
			if (exploreCount <= 0) {
				break;
			}
			
			exploreCount--;
		}
		
		System.out.println("444");
		
		System.out.println("#vtxs-non-unique = " + vertices.size());
		vertices.retainUnique();
		System.out.println("#vtxs-unique = " + vertices.size());
		
		transitions = new HashSet<DecaStableTransition>();
		cfgPairs = new HashSet<Pair<DecaFourStateConfig>>();
		
		for (DecaStableVertex v : vertices) {
			int hid = 0;
			
			for (DecaStableTransition t : v.getPrimaryOutgoing()) {
				if (t.isHiddenTimerTrigger()) {
					hid++;
				}
				
				if (!vertices.contains(t.getTgt())) {
					throw new Error("Should not happen!");
				}
				
				cfgPairs.add(new Pair<DecaFourStateConfig>(t.getSrc().getCfg(), t.getTgt().getCfg()));
			}
			
			if (hid > 1) {
				throw new Error("Should not happen!");
			}
			
			transitions.addAll(v.getPrimaryOutgoing());
		}
		
		vtxPairs = new HashSet<Pair<DecaFourVertex>>();
		vtxs = new HashSet<DecaFourVertex>();
		
		for (DecaStableTransition t : transitions) {
			vtxPairs.addAll(t.computeVtxPairs());
		}
		
		for (Pair<DecaFourVertex> vtxPair : vtxPairs) {
			vtxs.add(vtxPair.getElem1());
			vtxs.add(vtxPair.getElem2());
		}
		
		for (DecaStableTransition t : transitions) {
			t.getTgt().getPreds().add(t.getSrc());
		}
		
		coveredCfgs = new HashSet<DecaFourStateConfig>();
		coveredStableCfgs = new HashSet<DecaFourStateConfig>();
		
		for (DecaStableVertex v : vertices) {
			coveredCfgs.add(v.getCfg());
			coveredStableCfgs.add(v.getCfg());
		}
		
		final int stableCfgCount = coveredCfgs.size();
		
		for (DecaStableTransition t : transitions) {
			coveredCfgs.addAll(t.getSeq());
		}
		
		System.out.println("[stable][" + LocalTime.now() + "] #(cfg,input)s = " + vertices.size() + "; #transitions = " + transitions.size() + "; #stable-cfgs = " + stableCfgCount + "; #covered-cfgs = " + coveredCfgs.size());
		
//		//Sanity checking only (I think):
//		for (DecaStableVertex v : fringe) {
//			Set<Map<ReprPort, ASALSymbolicValue>> sicInputs = v.getCfg().getSelfLoopZeroPulseSicInputs(v.getExternalIncomingInputs().getValuePerPort(), legacy.timeoutPortPerDurationPort.values());
//			
//			for (DecaStableTransition t : v.getOutgoing()) {
//				if (t.getTgt().getCfg().equals(v.getCfg())) {
//					if (t.getSicInputs().withoutActivePulsesOrInternal().equals(t.getSicInputs().getValuePerPort())) {
//						if (!sicInputs.contains(t.getSicInputs().getValuePerPort())) {
//							System.out.println("Unexpectedly, self-loop is found generally but specially!");
//							CLI.waitForEnter();
//						}
//					}
//				}
//			}
//		}
		
		if ("input-enabledness-2".equals("off")) {
			Map<PulsePackMap, Set<DecaStableVertex>> cfgsPerOutputVal = new HashMap<PulsePackMap, Set<DecaStableVertex>>();
			Map<PulsePackMap, Set<DecaStableVertex>> cfgsPerExternalOutputVal = new HashMap<PulsePackMap, Set<DecaStableVertex>>();
			
			for (DecaStableVertex v : vertices) {
				HashMaps.inject(cfgsPerOutputVal, v.getCfg().getOutputVal(), v);
				HashMaps.inject(cfgsPerExternalOutputVal, v.getCfg().getOutputVal().extractExternalMap(), v);
			}
			
			int count = 0;
			
			for (Map.Entry<PulsePackMap, Set<DecaStableVertex>> e : cfgsPerExternalOutputVal.entrySet()) {
				Set<DecaStableInputChanges> inputChangesSet = new HashSet<DecaStableInputChanges>();
				
				for (DecaStableVertex v : e.getValue()) {
					for (DecaStableTransition t : v.getPrimaryOutgoing()) {
						inputChangesSet.add(t.getInputChanges1(this));
					}
				}
				
				for (DecaStableVertex v : e.getValue()) {
					Set<DecaStableInputChanges> outgoingInputChanges = v.getOutgoingInputChanges(this); 
					
					for (DecaStableInputChanges inputChanges : inputChangesSet) {
						if (!outgoingInputChanges.contains(inputChanges)) {
							PulsePackMap inputVal = inputChanges.applyToValuation(v.getExternalIncomingInputs());
							v.getPrimaryOutgoing().add(new DecaStableTransition(v, inputVal, null, false, false));
							count++;
						}
					}
				}
			}
			
			System.out.println("Added " + count + " no-input-change self-loops!");
		}
		
//		initStableCounts();
		
//		CLI.waitForEnter();
	}
	
	public int getTotalStableVtxsCount() {
		return totalStableVtxsCount;
	}
	
	public int getTotalStableTrsCount() {
		return totalStableTrsCount;
	}
	
	public Set<DecaFourCfgPair> getCfgPairs() {
		Set<DecaFourCfgPair> result = new HashSet<DecaFourCfgPair>();
		
		for (DecaStableTransition t : transitions) {
			if (t.getSicInputs1() != null) {
//				if (t.getTgt().getCfg().equals(t.getSrc().getCfg())) {
					//PulsePackMap inputVal = t.getSicInputs1().extractEventMap(t.getSrc().getExternalIncomingInputs(), false);
					PulsePackMap inputVal = PulsePackMap.EMPTY_IN;
					result.add(new DecaFourCfgPair(t.getSrc().getCfg(), t.getTgt().getCfg(), inputVal));
//				}
			}
		}
		
		return result;
	}
	
	private void initStableCounts() {
		ConcurrentWork<DecaFourStateConfig, ConcurrentWorker> cw = new ConcurrentWork<DecaFourStateConfig, ConcurrentWorker>();
		
		for (int index = 1; index <= 6; index++) {
			ConcurrentWorker w = new ConcurrentWorker("#stable-vtxs", "#stable-trs");
			w.set("#stable-vtxs", 0);
			w.set("#stable-trs", 0);
			cw.getWorkers().add(w);
		}
		
		cw.apply(legacy.configs, (cfg, w) -> {
			boolean hasStableSelfLoop = false;
			
			for (PulsePackMap val : cfg.computeStableSelfLoopInputVals()) {
				DecaStableVertex vtx = new DecaStableVertex(cfg, val, 0);
				vtx.populatePrimaryOutgoing(legacy);
				w.set("#stable-trs", w.get("#stable-trs", Integer.class) + vtx.getPrimaryOutgoing().size());
				hasStableSelfLoop = true;
			}
			
			if (hasStableSelfLoop) {
				w.set("#stable-vtxs", w.get("#stable-vtxs", Integer.class) + 1);
			}
		});
		
		totalStableVtxsCount = 0;
		totalStableTrsCount = 0;
		
		for (ConcurrentWorker w : cw.getWorkers()) {
			totalStableVtxsCount += w.get("#stable-vtxs", Integer.class);
			totalStableTrsCount += w.get("#stable-trs", Integer.class);
		}
	}
	
	public DecaStableTransition getIncomingTransition(DecaStableVertex tgt) {
		for (DecaStableVertex v : vertices) {
			if (v.getLevel() + 1 == tgt.getLevel()) {
				for (DecaStableTransition t : v.getPrimaryOutgoing()) {
					if (t.getTgt().equals(tgt)) {
						return t;
					}
				}
			}
		}
		
		return null;
	}
	
	public List<DecaStableTransition> getTransitionsToVertex(DecaStableVertex tgt) {
		List<DecaStableTransition> result = new ArrayList<DecaStableTransition>();
		DecaStableVertex curr = tgt;
		
		while (true) {
			DecaStableTransition t = getIncomingTransition(curr);
			
			if (t != null) {
				result.add(0, t);
				curr = t.getSrc();
			} else {
				break;
			}
		}
		
		return result;
	}
	
	private Map<ReprPort, ASALSymbolicValue> extractInitialInputs() {
		Map<ReprPort, ASALSymbolicValue> result = new HashMap<ReprPort, ASALSymbolicValue>();
		
		for (Map.Entry<ReprPort, ASALSymbolicValue> e : legacy.initialInputs.extractValuation().entrySet()) {
			if (e.getKey().isPortToEnvironment()) {
				if (JPulse.class.isAssignableFrom(e.getKey().getType())) {
					if (e.getValue().toBoolean()) {
						throw new Error("No support for initialization to TRUE of pulse port (" + e.getKey().getReprOwner().getName() + "::" + e.getKey().getName() + ")!");
					}
				} else {
					if (e.getKey().getPulsePort() == null) {
						result.put(e.getKey(), e.getValue());
					}
				}
			}
		}
		
		return result;
	}
	
	public Set<LineageObject> getLineageObjects() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(aggregateLineageObjects());
		return result;
	}
	
	public Set<LineageObject> getStableLineageObjects() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		
		for (LineageObject v : vertices) {
			result.addAll(v.aggregateLineageObjects());
		}
		
		return result;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(legacy.initProtoTransitions);
		result.addAll(coverageStats.getTransitions4());
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
	public void printGraphvizFile() {
		try {
			PrintStream ps = new PrintStream("deca-stable-overview.gv");
			printGraphvizFile(ps, true, true);
			ps.flush();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
	
	public void printGraphvizFile(PrintStream out, boolean includeCycles, boolean includeNonDet) {
		TextOptions.select(TextOptions.GRAPHVIZ_FULL);
		
		Map<DecaStableVertex, String> namePerVertex = new HashMap<DecaStableVertex, String>();
		Map<DecaStableTransition, String> namePerTransition = new HashMap<DecaStableTransition, String>();
		
		for (DecaStableVertex v : vertices) {
			namePerVertex.put(v, "V" + namePerVertex.size());
		}
		
		for (DecaStableTransition t : transitions) {
			if (includeCycles || t.getTgt() != t.getSrc()) {
				if (t.isNonDet()) {
					if (includeNonDet) {
						namePerTransition.put(t, "T" + namePerTransition.size() + "_nondet");
					}
				} else {
					namePerTransition.put(t, "T" + namePerTransition.size());
				}
			}
		}
		
		out.println("// " + getClass().getCanonicalName());
		out.println("digraph G {");
		out.println("\tI0 [label=\"initial\", style=none];");
		
		for (DecaStableVertex v : vertices) {
			List<String> elems = new ArrayList<String>();
			
			for (Class<?> clz : v.getClzs()) {
				elems.add("<<" + clz.getSimpleName() + ">>");
			}
			
			for (Map.Entry<ReprPort, ASALSymbolicValue> e : v.getExternalIncomingInputs().extractValuation().entrySet()) {
				elems.add("input " + e.getKey().getName() + " = " + e.getValue());
			}
			
			for (Map.Entry<ReprPort, ASALSymbolicValue> e : v.getOutputVal().extractValuation().entrySet()) {
				elems.add("output " + e.getKey().getName() + " = " + e.getValue());
			}
			
			for (Map.Entry<ASALProperty, ASALSymbolicValue> e : v.getCfg().getPropertyVal().entrySet()) {
				elems.add("property " + e.getKey().getName() + " = " + e.getValue());
			}
			
			Collections.sort(elems);
			String s = "";
//			s += v.getCfg().getDescription("\\n");
			s += Texts.concat(elems, "\\n");
			
			out.println("\t" + namePerVertex.get(v) + " [label=\"" + s + "\", shape=circle];");
			
//			for (DecaStableVertex succ : v.getSuccs()) {
//				if (succ != v) {
//					out.println("\t" + namePerVertex.get(v) + " -> " + namePerVertex.get(succ) + ";");
//				}
//			}
		}
		
		for (DecaStableTransition t : transitions) {
			if (t.getSicInputs1() != null) {
				if (includeCycles || t.getTgt() != t.getSrc()) {
					if (includeNonDet || !t.isNonDet()) {
						List<String> elems = new ArrayList<String>();
						String colors;
						
						if (t.isNonDet()) {
							elems.add("<<non-deterministic>>");
							colors = ", color=red, fontcolor=red";
						} else {
							colors = "";
						}
						
						for (Map.Entry<ReprPort, ASALSymbolicValue> e : t.getSicInputs1().extractValuation().entrySet()) {
							ASALSymbolicValue v = t.getSrc().getExternalIncomingInputs().getPortValue(e.getKey(), false);
							
							if (v == null || !v.equals(e.getValue())) {
								elems.add(e.getKey().getName() + " = " + e.getValue());
							}
						}
						
						for (DecaStableOutputEvolution evo : t.getOutputEvolutions()) {
							elems.add(evo.toString());
						}
						
						out.println("\t" + namePerTransition.get(t) + " [label=\"" + Texts.concat(elems, "\\n") + "\", shape=rect" + colors + "];");
						
						String v1 = namePerVertex.get(t.getSrc());
						String v2 = namePerTransition.get(t);
						String v3 = namePerVertex.get(t.getTgt());
						
						out.println("\t" + v1 + " -> " + v2 + " [style = solid" + colors + "];");
						out.println("\t" + v2 + " -> " + v3 + " [style = solid" + colors + "];");
					}
				}
			}
		}
		
//		for (DecaStableTransition t : transitions) {
//			if (t.getSicInputs1() != null) {
//				if (includeCycles || t.getTgt() != t.getSrc()) {
//					String v1 = namePerVertex.get(t.getSrc());
//					String v2 = namePerTransition.get(t);
//					String v3 = namePerVertex.get(t.getTgt());
//					
//					out.println("\t" + v1 + " -> " + v2 + ";");
//					out.println("\t" + v2 + " -> " + v3 + ";");
//				}
//			}
//		}
		
		out.println("\tI0 -> " + namePerVertex.get(initialTransition.getTgt()) + ";");
		out.println("}");
	}
	
	public void printGraphvizFile2(PrintStream out) {
		Set<Set<DecaStableVertex>> vtxGrps = new HashSet<Set<DecaStableVertex>>();
		Set<DecaStableVertex> remainingVtxs = new HashSet<DecaStableVertex>(vertices);
		
		while (remainingVtxs.size() > 0) {
			DecaStableVertex v = remainingVtxs.iterator().next();
			Set<DecaStableVertex> newVtxGrp = new HashSet<DecaStableVertex>();
			
			for (DecaStableVertex succ : v.getSuccs()) {
				if (succ.getSuccs().contains(v)) {
					if (remainingVtxs.contains(succ)) {
						newVtxGrp.add(v);
					}
				}
			}
			
			remainingVtxs.removeAll(newVtxGrp);
			vtxGrps.add(newVtxGrp);
		}
		
		Map<DecaStableVertex, Set<DecaStableVertex>> vtxGrpPerVtx = new HashMap<DecaStableVertex, Set<DecaStableVertex>>();
		
		for (Set<DecaStableVertex> vtxGrp : vtxGrps) {
			for (DecaStableVertex v : vtxGrp) {
				vtxGrpPerVtx.put(v, vtxGrp);
			}
		}
		
		Map<Set<DecaStableVertex>, Set<Set<DecaStableVertex>>> succVtxGrpsPerVtxGrp = new HashMap<Set<DecaStableVertex>, Set<Set<DecaStableVertex>>>();
		
		for (Set<DecaStableVertex> vtxGrp : vtxGrps) {
			Set<Set<DecaStableVertex>> succVtxGrps = new HashSet<Set<DecaStableVertex>>();
			
			for (DecaStableVertex v : vtxGrp) {
				for (DecaStableVertex succ : v.getSuccs()) {
					succVtxGrps.add(vtxGrpPerVtx.get(succ));
				}
			}
			
			succVtxGrpsPerVtxGrp.put(vtxGrp, succVtxGrps);
		}
		
		Map<Set<DecaStableVertex>, String> namePerVtxGrp = new HashMap<Set<DecaStableVertex>, String>();
		
		for (Set<DecaStableVertex> v : vtxGrps) {
			namePerVtxGrp.put(v, "V" + namePerVtxGrp.size());
		}
		
		out.println("// " + getClass().getCanonicalName());
		out.println("digraph G {");
		out.println("\tnode [label=\"\", shape=circle, width=0.1, fillcolor=black, style=filled];");
		out.println("\tI0 [style=none];");
		
		for (Set<DecaStableVertex> vtxGrp : vtxGrps) {
			for (Set<DecaStableVertex> succVtxGrp : succVtxGrpsPerVtxGrp.get(vtxGrp)) {
				if (succVtxGrp != vtxGrp) {
					out.println("\t" + namePerVtxGrp.get(vtxGrp) + " -> " + namePerVtxGrp.get(succVtxGrp) + ";");
				}
			}
		}
		
		out.println("\tI0 -> " + namePerVtxGrp.get(vtxGrpPerVtx.get(initialTransition.getTgt())) + ";");
		out.println("}");
	}
}

