package jlx.behave.stable;

import java.util.*;

import jlx.asal.vars.ASALPort;
import jlx.behave.proto.*;
import jlx.models.UnifyingBlock.ReprPort;
import jlx.utils.*;

public class DecaStableTransition implements LineageObject {
	private DecaStableVertex src;
	private DecaStableVertex tgt;
	private PulsePackMap sicInputs1;
	private PulsePackMap sicInputs2;
	private List<DecaFourStateConfig> seq;
	private boolean isHiddenTimerTrigger;
	private boolean isNonDet;
	private DecaFourCoverageStats coverageStats;
	
	public boolean tested;
	
	public DecaStableTransition(DecaStableVertex src, DecaStableVertex tgt, PulsePackMap sicInputs1, PulsePackMap sicInputs2, List<DecaFourStateConfig> seq, DecaFourCoverageStats coverageStats, boolean isNonDet) {
		this.src = src;
		this.tgt = tgt;
		this.sicInputs1 = sicInputs1;
		this.sicInputs2 = sicInputs2;
		this.seq = seq;
		this.coverageStats = coverageStats;
		this.isNonDet = isNonDet;
		
		isHiddenTimerTrigger = false;
	}
	
	public DecaStableTransition(DecaStableVertex src, PulsePackMap sicInputs1, PulsePackMap sicInputs2, boolean isHiddenTimerTrigger, boolean isNonDet) {
		this.src = src;
		this.sicInputs1 = sicInputs1;
		this.sicInputs2 = sicInputs2;
		this.isHiddenTimerTrigger = isHiddenTimerTrigger;
		this.isNonDet = isNonDet;
		
		tgt = src;
		seq = Collections.singletonList(src.getCfg());
		coverageStats = new DecaFourCoverageStats();
	}
	
	public DecaStableVertex getSrc() {
		return src;
	}
	
	public DecaStableVertex getTgt() {
		return tgt;
	}
	
	public PulsePackMap getSicInputs1() {
		return sicInputs1;
	}
	
	public PulsePackMap getSicInputs2() {
		return sicInputs2;
	}
	
	public List<DecaFourStateConfig> getSeq() {
		return seq;
	}
	
	public Set<Class<?>> getSysmlClzs() {
		Set<Class<?>> result = new HashSet<Class<?>>();
		
		for (DecaFourStateConfig cfg : seq) {
			result.addAll(cfg.getSysmlClzs());
		}
		
		return result;
	}
	
	public DecaFourCoverageStats getCoverageStats() {
		return coverageStats;
	}
	
	public boolean isHiddenTimerTrigger() {
		return isHiddenTimerTrigger;
	}
	
	public boolean isNonDet() {
		return isNonDet;
	}
	
	public void normalizeTgt(NormMap<DecaStableVertex> vertices) {
		tgt = vertices.get(tgt);
	}
	
	public Set<Pair<DecaFourVertex>> computeVtxPairs() {
		Set<Pair<DecaFourVertex>> result = new HashSet<Pair<DecaFourVertex>>();
		
		for (int index = 1; index < seq.size(); index++) {
			result.addAll(seq.get(index).computeVtxPairsFrom(seq.get(index - 1)));
		}
		
		return result;
	}
	
	public DecaStableInputChanges getInputChanges1(DecaStableStateMachine stableSm) {
		if (isHiddenTimerTrigger) {
			return new DecaStableInputChanges(PulsePackMap.EMPTY_IN, null, null, true);
		}
		
		PulsePackMap changes = getSicInputs1().extractEventMap(getSrc().getExternalIncomingInputs(), false);
		
		if (changes.getPackPerPort().isEmpty() && !src.equals(tgt)) {
			throw new Error("At least 1 input value should change!");
		}
		
		for (Map.Entry<ReprPort, PulsePack> e : changes.getPackPerPort().entrySet()) {
			ASALPort durationPort = stableSm.durationPortPerTimeoutPort.get(e.getKey());
			
			if (durationPort != null) {
				if (durationPort instanceof ReprPort) {
					ReprPort rp = (ReprPort)durationPort;
					return new DecaStableInputChanges(PulsePackMap.EMPTY_IN, e.getKey(), rp, false);
				} else {
					throw new Error("Should not happen!");
				}
			}
		}
		
		return new DecaStableInputChanges(changes, null, null, false);
	}
	
	/**
	 * Can be NULL.
	 */
	public DecaStableInputChanges getInputChanges2(DecaStableStateMachine stableSm) {
		if (getSicInputs2() == null) {
			return null;
		}
		
		PulsePackMap pred;
		
		if (isHiddenTimerTrigger) {
			pred = getSrc().getExternalIncomingInputs();
		} else {
			pred = getSicInputs1().deactivate();
		}
		
		PulsePackMap changes = getSicInputs2().extractEventMap(pred, false);
		
		if (changes.getPackPerPort().isEmpty() && !src.equals(tgt)) {
			throw new Error("At least 1 input value should change!");
		}
		
		for (Map.Entry<ReprPort, PulsePack> e : changes.getPackPerPort().entrySet()) {
			ASALPort durationPort = stableSm.durationPortPerTimeoutPort.get(e.getKey());
			
			if (durationPort != null) {
				if (durationPort instanceof ReprPort) {
					ReprPort rp = (ReprPort)durationPort;
					return new DecaStableInputChanges(PulsePackMap.EMPTY_IN, e.getKey(), rp, false);
				} else {
					throw new Error("Should not happen!");
				}
			}
		}
		
		return new DecaStableInputChanges(changes, null, null, false);
	}
	
	public Set<DecaStableOutputEvolution> getOutputEvolutions() {
		return DecaStableOutputEvolution.getOutputEvolutions(seq);
	}
	
	@Override
	public String toString() {
		String result = "";
		
		if (src != null) {
			result += src.getCfg().getDescription();
		} else {
			result += "[??]";
		}
		
		if (sicInputs1 != null) {
			if (src != null) {
				result += "\n---" + sicInputs1.extractEventMap(src.getExternalIncomingInputs(), false).toString() + "-->";
			} else {
				result += "\n---" + sicInputs1.toString() + "-->";
			}
		} else {
			result += "\n----->";
		}
		
		result += "\n" + tgt.getCfg().getDescription();
		return result;
	}
	
	@Override
	public Set<LineageObject> getImpliedElements() {
		Set<LineageObject> result = new HashSet<LineageObject>();
		result.addAll(coverageStats.getTransitions4());
		result.addAll(seq);
		
		if (src != null) {
			result.add(src);
		}
		
		result.add(tgt);
		return result;
	}
	
	@Override
	public Set<LineageObject> getInferredElements() {
		return Collections.emptySet();
	}
	
//	public int computeSecondaryTrs(DecaFourStateMachines legacy, NormMap<DecaStableVertex> vertices, DecaFourCoverageStats coverageStatsDest) {
//		int result = 0;
//		
//		if (sicInputs != null && src != null) {
//			PulsePackMap deactivatedSicInputs = sicInputs.deactivate();
//			
//			for (DecaFourStateConfig cfg : seq) {
//				if (!cfg.equals(src.getCfg()) && !cfg.equals(tgt.getCfg())) {
//					DecaStableVertex v = new DecaStableVertex(cfg, deactivatedSicInputs, 0);
//					result += v.populateOutgoing(legacy, src, vertices, coverageStatsDest, true);
//				}
//			}
//		}
//		
//		return result;
//	}
}

