package jlx.behave.stable.files;

import java.io.*;
import java.util.*;

import jlx.utils.*;

public class DecaStableFilePrinter extends DecaStableFileReader {
	public DecaStableFilePrinter() {
		//Empty.
	}
	
	public void printToFile(String filename) {
		try {
			PrintStream out = new PrintStream(filename);
			printToStream(out);
			out.flush();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
	}
	
	public void printToStream(PrintStream out) {
		Map<Vertex, List<String>> stepsPerVertex = new HashMap<Vertex, List<String>>();
		Map<Transition, List<String>> stepsPerTransition = new HashMap<Transition, List<String>>();
		int testIndex = 1;
		
		for (TestSuite ts : getTestSuites()) {
			for (Trace test : ts.getTests()) {
				int stepIndex = 1;
				
				for (Transition t : test.getTransitions()) {
					if (!t.getInputChanges1().isHiddenTimerTrigger()) {
						String step = testIndex + "." + stepIndex;
						stepIndex++;
						
						List<String> steps1 = stepsPerTransition.get(t);
						
						if (steps1 == null) {
							steps1 = new ArrayList<String>();
							stepsPerTransition.put(t, steps1);
						}
						
						steps1.add(step);
						
						List<String> steps2 = stepsPerVertex.get(t.getTgt());
						
						if (steps2 == null) {
							steps2 = new ArrayList<String>();
							stepsPerVertex.put(t.getTgt(), steps2);
						}
						
						steps2.add(step);
					}
				}
				
				testIndex++;
			}
		}
		
		out.println("digraph G {");
		out.println("\tI [label = \"(initial)\", shape = ellipse];");
		
		for (Vertex v : getVertices().values()) {
			printVertex(out, v, stepsPerVertex.get(v));
		}
		
		printTransition(out, getInitialTransition(), stepsPerTransition.get(getInitialTransition()));
		
		for (Transition t : getTransitions()) {
			if (!t.getInputChanges1().isHiddenTimerTrigger()) {
				printTransition(out, t, stepsPerTransition.get(t));
			}
		}
		
		out.println("}");
	}
	
	private static void printVertex(PrintStream out, Vertex v, List<String> steps) {
		List<String> elems = new ArrayList<String>();
		
		if (steps != null) {
			elems.add(Texts.concat(steps, ", "));
		}
		
		elems.add("V" + v.getId());
		
		for (Map.Entry<Scope, String> e : v.getStatePerScope().entrySet()) {
			elems.add(e.getValue());
		}
		
		for (Map.Entry<Port, String> e : v.getValuation().entrySet()) {
			elems.add(e.getKey().toString() + " = " + e.getValue().replace("\"", "\\\""));
		}
		
		out.println("\tV" + v.getId() + " [label = \"" + Texts.concat(elems, "\\n") + "\", shape = ellipse];");
	}
	
	private static void printTransition(PrintStream out, Transition t, List<String> steps) {
		final String src = t.getSrc() != null ? "V" + t.getSrc().getId() : "I";
		final String tid = t.getId() >= 0 ? "T" + t.getId() : "T";
		final String tgt = "V" + t.getTgt().getId();
		final String c = Dot.getRandomColor();
		
		List<String> elems = new ArrayList<String>();
		elems.add(tid);
		
		elems.add(t.getInputChanges1().toString().replace("\"", "\\\""));
		
		for (Map.Entry<Port, List<String>> e : t.constructChangesPerPort().entrySet()) {
			elems.add(e.getKey().toString() + " = " + Texts.concat(e.getValue(), " -> ").replace("\"", "\\\""));
		}
		
		out.println("\t" + tid + " [label = \"" + Texts.concat(elems, "\\n") + "\", shape = rect, color = \"" + c + "\", fontcolor = \"" + c + "\"];");
		
		if (steps != null) {
			out.println("\t" + src + " -> " + tid + " [color = \"" + c + "\", fontcolor = \"" + c + "\", taillabel = \"" + Texts.concat(steps, ", ") + "\"];");
		} else {
			out.println("\t" + src + " -> " + tid + " [color = \"" + c + "\", fontcolor = \"" + c + "\", taillabel = \"UNTOUCHED\"];");
		}
		
		out.println("\t" + tid + " -> " + tgt + " [color = \"" + c + "\"];");
	}
}
