package jlx.blocks.ibd1;

import java.util.*;

import jlx.asal.j.JStat;
import jlx.asal.j.JType;
import jlx.asal.parsing.ASALCode;
import jlx.behave.CodeObject;
import jlx.common.ReflectionUtils;

public class Operation<T extends JType> extends CodeObject {
	private final List<JType> params;
	private final boolean canInitialize;
	
	public Operation(String code0, String... code) {
		this(new ASALCode("STATS1", code0, code), false);
	}
	
	public Operation(JStat stat0, JStat... statements) {
		this(new ASALCode("STATS1", JStat.seq(stat0, statements)), false);
	}
	
	protected Operation(ASALCode treeObject, boolean canInitialize) {
		super(treeObject);
		
		params = new ArrayList<JType>();
		
		this.canInitialize = canInitialize;
	}
	
	private Operation(Operation<T> source, JType param) {
		super(source.treeObject);
		
		params = new ArrayList<JType>(source.params);
		params.add(param);
		
		canInitialize = source.canInitialize;
	}
	
	public List<JType> getParams() {
		return Collections.unmodifiableList(params);
	}
	
	public boolean canInitialize() {
		return canInitialize;
	}
	
	public Operation<T> withParam(String paramName, Class<? extends JType> paramType) {
		return withParam(JType.createParam(paramName, paramType));
	}
	
	public Operation<T> withParam(JType param) {
		if (param.getParamName() == null) {
			throw new Error("No parameter name!");
		}
		
		for (JType p : params) {
			if (param.getParamName().equals(p.getParamName())) {
				throw new Error("Parameter name \"" + param.getParamName() + "\" is already in use!");
			}
		}
		
		if (ReflectionUtils.isFinal(param.getClass())) {
			throw new Error("Parameter instance should not be final!");
		}
		
		return new Operation<T>(this, param);
	}
}

