package jlx.utils.ldd;

import java.util.*;

public class LDDNode {
	private final int key;
	private final int value;
	private final LDDNode nextValueNode;
	private final LDDNode nextKeyNode;
	private final int hashCode;
	
	public LDDNode(int key, int value, LDDNode nextValueNode, LDDNode nextKeyNode) {
		this.key = key;
		this.value = value;
		this.nextValueNode = nextValueNode;
		this.nextKeyNode = nextKeyNode;
		
		hashCode = extractHashCode();
	}
	
	private int extractHashCode() {
		return Objects.hash(key, value, nextValueNode, nextKeyNode);
	}
	
	public int getKey() {
		return key;
	}
	
	public int getValue() {
		return value;
	}
	
	public LDDNode getNextValueNode() {
		return nextValueNode;
	}
	
	public LDDNode getNextKeyNode() {
		return nextKeyNode;
	}
	
	public boolean hasMoreKeys() {
		return nextKeyNode != null;
	}
	
	@Override
	public int hashCode() {
		return hashCode;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		LDDNode other = (LDDNode) obj;
		return key == other.key && value == other.value && Objects.equals(nextKeyNode, other.nextKeyNode) && Objects.equals(nextValueNode, other.nextValueNode);
	}
	
	@Override
	public String toString() {
		String result = key + "=" + value + ",";
		
		if (nextValueNode != null) {
			result += nextValueNode.toString();
		} else {
			result += "-";
		}
		
		result += ",";
		
		if (nextKeyNode != null) {
			result += nextKeyNode.toString();
		} else {
			result += "-";
		}
		
		return "[" + result + "]";
	}
	
	public <K, V> boolean isWellFormed(LDDEncoding<K, V> encoding) {
		if (!encoding.isValidKeyId(key)) {
			return false;
		}
		
		if (value < 0) {
			if (value != -1) {
				return false;
			}
		} else {
			if (!encoding.isValidValueId(key, value)) {
				return false;
			}
		}
		
		if (key == encoding.getOrderedKeys().size() - 1) {
			if (nextKeyNode != null) {
				return false;
			}
		} else {
			if (nextKeyNode == null) {
				return false;
			}
		}
		
		if (nextValueNode != null && !nextValueNode.isWellFormed(encoding)) {
			return false;
		}
		
		if (nextKeyNode != null && !nextKeyNode.isWellFormed(encoding)) {
			return false;
		}
		
		return true;
	}
}

