package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDClearAll {
	public static LDDNode clearAll(LDDCache<?, ?> cache, LDDNode node, Set<Integer> vars) {
		if (vars.contains(node.getKey())) {
			LDDNode nextKeyNode;
			
			if (node.hasMoreKeys()) {
				nextKeyNode = clearAll(cache, node.getNextKeyNode(), vars);
				LDDNode curr = node.getNextValueNode();
				
				while (curr != null) {
					nextKeyNode = LDDUnion.union(cache, nextKeyNode, clearAll(cache, curr.getNextKeyNode(), vars));
					curr = curr.getNextValueNode();
				}
			} else {
				nextKeyNode = null;
			}
			
			return cache.createNode(node.getKey(), -1, null, nextKeyNode);
		} else {
			LDDNode nextValueNode;
			
			if (node.getNextValueNode() != null) {
				nextValueNode = clearAll(cache, node.getNextValueNode(), vars);
			} else {
				nextValueNode = null;
			}
			
			LDDNode nextKeyNode;
			
			if (node.getNextKeyNode() != null) {
				nextKeyNode = clearAll(cache, node.getNextKeyNode(), vars);
			} else {
				nextKeyNode = null;
			}
			
			return cache.createNode(node.getKey(), node.getValue(), nextValueNode, nextKeyNode);
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("CLEAR_ALL") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.clearAll(getTestMultiMap().keySet());
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				
				if (maps1.isEmpty()) {
					result.add(Collections.emptyMap());
				} else {
					for (Map<String, String> map : maps1) {
						Map<String, String> mapCopy = new HashMap<String, String>();
						
						for (Map.Entry<String, String> e : map.entrySet()) {
							if (!getTestMultiMap().containsKey(e.getKey())) {
								mapCopy.put(e.getKey(), e.getValue());
							}
						}
						
						result.add(mapCopy);
					}
				}
				
				return result;
			}
		};
	}
}
