package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDDiffElems {
	public static LDDNode diffElems(LDDCache<?, ?> cache, LDDNode node, Map<Integer, Set<Integer>> excludedValuesPerKey, int diffCount, int minDiffCount, int maxDiffCount) {
		int nextKeyDiffCount;
		
		if (excludedValuesPerKey.get(node.getKey()).contains(node.getValue())) {
			nextKeyDiffCount = diffCount + 1;
		} else {
			nextKeyDiffCount = diffCount;
		}
		
		LDDNode nextValueNode;
		
		if (node.getNextValueNode() != null) {
			nextValueNode = diffElems(cache, node.getNextValueNode(), excludedValuesPerKey, diffCount, minDiffCount, maxDiffCount);
		} else {
			nextValueNode = null;
		}
		
		if (nextKeyDiffCount <= maxDiffCount) {
			if (node.hasMoreKeys()) {
				LDDNode nextKeyNode = diffElems(cache, node.getNextKeyNode(), excludedValuesPerKey, nextKeyDiffCount, minDiffCount, maxDiffCount);
				
				if (nextKeyNode != null) {
					return cache.createNode(node.getKey(), node.getValue(), nextValueNode, nextKeyNode);
				} else {
					return nextValueNode;
				}
			} else {
				if (nextKeyDiffCount >= minDiffCount) {
					return cache.createNode(node.getKey(), node.getValue(), nextValueNode, null);
				} else {
					return nextValueNode;
				}
			}
		} else {
			return nextValueNode;
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("DIFF_ELEMS_0") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.diff(getTestMap(), 0, 0);
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				
				if (maps1.contains(getTestMap())) {
					result.add(getTestMap());
				}
				
				return result;
			}
		};
		
		new LDDSetTest("DIFF_ELEMS_1") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.diff(getTestMap(), 1, 1);
			}
			
			private static int getDiffCount(Map<String, String> map1, Map<String, String> map2) {
				Set<String> keys = new HashSet<String>(map1.keySet());
				keys.addAll(map2.keySet());
				int diffCount = 0;
				
				for (String key : keys) {
					if (!Objects.equals(map1.get(key), map2.get(key))) {
						diffCount++;
					}
				}
				
				return diffCount;
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				
				for (Map<String, String> map1 : maps1) {
					if (getDiffCount(map1, getTestMap()) == 1) {
						result.add(map1);
					}
				}
				
				return result;
			}
		};
	}
}

