package jlx.utils.ldd.ops;

import java.util.*;
import java.util.function.Consumer;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDFlatten {
	public static <K, V> Map<K, V> flatten(LDDEncoding<K, V> encoding, LDDNode node) {
		Map<K, V> result;
		
		if (node.hasMoreKeys()) {
			result = flatten(encoding, node.getNextKeyNode());
		} else {
			result = new HashMap<K, V>();
		}
		
		encoding.putDecoded(result, node.getKey(), node.getValue());
		return result;
	}
	
	public static <K, V> void flatten(LDDEncoding<K, V> encoding, LDDNode node, Map<K, V> soFar, Set<Map<K, V>> dest) {
		if (node.hasMoreKeys()) {
			LDDNode curr = node;
			
			while (curr != null) {
				Map<K, V> newSoFar = new HashMap<K, V>(soFar);
				encoding.putDecoded(newSoFar, curr.getKey(), curr.getValue());
				flatten(encoding, curr.getNextKeyNode(), newSoFar, dest);
				
				curr = curr.getNextValueNode();
			}
		} else {
			LDDNode curr = node;
			
			while (curr != null) {
				Map<K, V> newSoFar = new HashMap<K, V>(soFar);
				encoding.putDecoded(newSoFar, curr.getKey(), curr.getValue());
				dest.add(newSoFar);
				
				curr = curr.getNextValueNode();
			}
		}
	}
	
	public static <K, V> void flatten(LDDEncoding<K, V> encoding, LDDNode node, Map<K, V> soFar, Consumer<Map<K, V>> fct) {
		if (node.hasMoreKeys()) {
			LDDNode curr = node;
			
			while (curr != null) {
				Map<K, V> newSoFar = new HashMap<K, V>(soFar);
				encoding.putDecoded(newSoFar, curr.getKey(), curr.getValue());
				flatten(encoding, curr.getNextKeyNode(), newSoFar, fct);
				
				curr = curr.getNextValueNode();
			}
		} else {
			LDDNode curr = node;
			
			while (curr != null) {
				Map<K, V> newSoFar = new HashMap<K, V>(soFar);
				encoding.putDecoded(newSoFar, curr.getKey(), curr.getValue());
				fct.accept(newSoFar);
				
				curr = curr.getNextValueNode();
			}
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("FLATTEN") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return createFromMaps(ldd1.getElems());
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				return maps1;
			}
		};
	}
}
