package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDIntersection {
	public static LDDNode intersect(LDDCache<?, ?> cache, LDDNode node1, LDDNode node2, int maxIntersectKeyIndex) {
		if (node1.getValue() == node2.getValue()) {
			LDDNode nextValueNode;
			
			if (node1.getNextValueNode() != null && node2.getNextValueNode() != null) {
				nextValueNode = intersect(cache, node1.getNextValueNode(), node2.getNextValueNode(), maxIntersectKeyIndex);
			} else {
				nextValueNode = null;
			}
			
			if (node1.hasMoreKeys()) {
				if (node1.getKey() < maxIntersectKeyIndex) {
					LDDNode nextKeyNode = intersect(cache, node1.getNextKeyNode(), node2.getNextKeyNode(), maxIntersectKeyIndex);
					
					if (nextKeyNode != null) {
						return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, nextKeyNode);
					} else {
						return nextValueNode;
					}
				} else {
					return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, cache.deepCopyNode(node1.getNextKeyNode()));
				}
			} else {
				return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, null);
			}
		} else {
			if (node1.getValue() < node2.getValue()) {
				if (node1.getNextValueNode() != null) {
					return intersect(cache, node1.getNextValueNode(), node2, maxIntersectKeyIndex);
				} else {
					return null;
				}
			} else {
				if (node2.getNextValueNode() != null) {
					return intersect(cache, node1, node2.getNextValueNode(), maxIntersectKeyIndex);
				} else {
					return null;
				}
			}
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("INTERSECT_1") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.intersect(ldd2);
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				result.addAll(maps1);
				result.retainAll(maps2);
				return result;
			}
		};
		
		new LDDSetTest("INTERSECT_2") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.intersect(ldd2, getTestPrefix().get(getTestPrefix().size() - 1));
			}
			
			private boolean samePrefix(Map<String, String> map1, Map<String, String> map2) {
				for (String s : getTestPrefix()) {
					if (!Objects.equals(map1.get(s), map2.get(s))) {
						return false;
					}
				}
				
				return true;
			}
			
			private boolean samePrefix(Map<String, String> map1, Set<Map<String, String>> maps2) {
				for (Map<String, String> map2 : maps2) {
					if (samePrefix(map1, map2)) {
						return true;
					}
				}
				
				return false;
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				
				for (Map<String, String> map : maps1) {
					if (samePrefix(map, maps2)) {
						result.add(map);
					}
				}
				
				return result;
			}
		};
	}
}

