package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDProject {
	public static LDDNode project(LDDCache<?, ?> cache, LDDNode node, Set<Integer> retainedKeys, int nextKeyIndex) {
		if (!retainedKeys.contains(node.getKey())) {
			if (node.hasMoreKeys()) {
				LDDNode result = project(cache, node.getNextKeyNode(), retainedKeys, nextKeyIndex);
				LDDNode curr = node.getNextValueNode();
				
				while (curr != null) {
					LDDNode temp = project(cache, curr.getNextKeyNode(), retainedKeys, nextKeyIndex);
					
					if (temp != null) {
						if (result != null) {
							result = LDDUnion.union(cache, result, temp);
						} else {
							result = temp;
						}
					}
					
					curr = curr.getNextValueNode();
				}
				
				return result;
			} else {
				return null;
			}
		} else {
			LDDNode nextValueNode;
			
			if (node.getNextValueNode() != null) {
				nextValueNode = project(cache, node.getNextValueNode(), retainedKeys, nextKeyIndex);
			} else {
				nextValueNode = null;
			}
			
			LDDNode nextKeyNode;
			
			if (node.getNextKeyNode() != null) {
				nextKeyNode = project(cache, node.getNextKeyNode(), retainedKeys, nextKeyIndex + 1);
			} else {
				nextKeyNode = null;
			}
			
			return cache.createNode(nextKeyIndex, node.getValue(), nextValueNode, nextKeyNode);
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("PROJECT") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.createSubLDD(getTestMultiMap().keySet());
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				
				if (getTestMultiMap().isEmpty()) {
					//Do nothing.
				} else {
					for (Map<String, String> map : maps1) {
						Map<String, String> mapCopy = new HashMap<String, String>();
						
						for (Map.Entry<String, String> e : map.entrySet()) {
							if (getTestMultiMap().containsKey(e.getKey())) {
								mapCopy.put(e.getKey(), e.getValue());
							}
						}
						
						result.add(mapCopy);
					}
				}
				
				return result;
			}
			
			@Override
			public LDDEncoding<String, String> getNewEncoding(LDDEncoding<String, String> table) {
				return table.createSubEncoding(getTestMultiMap().keySet());
			}
		};
	}
}
