package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDRemoveAll {
	public static LDDNode removeAll(LDDCache<?, ?> cache, LDDNode node1, LDDNode node2) {
		if (node1.getValue() == node2.getValue()) {
			LDDNode nextValueNode;
			
			if (node1.getNextValueNode() != null) {
				if (node2.getNextValueNode() != null) {
					nextValueNode = removeAll(cache, node1.getNextValueNode(), node2.getNextValueNode());
				} else {
					nextValueNode = cache.deepCopyNode(node1.getNextValueNode());
				}
			} else {
				nextValueNode = null;
			}
			
			if (node1.hasMoreKeys()) {
				LDDNode nextKeyNode = removeAll(cache, node1.getNextKeyNode(), node2.getNextKeyNode());
				
				if (nextKeyNode != null) {
					return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, nextKeyNode);
				}
				
				return nextValueNode;
			} else {
				if (nextValueNode != null) {
					return nextValueNode;
				} else {
					return null;
				}
			}
		} else {
			if (node1.getValue() < node2.getValue()) {
				LDDNode nextValueNode;
				
				if (node1.getNextValueNode() != null) {
					nextValueNode = removeAll(cache, node1.getNextValueNode(), node2);
				} else {
					nextValueNode = null;
				}
				
				return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, cache.deepCopyNode(node1.getNextKeyNode()));
			} else {
				if (node2.getNextValueNode() != null) {
					return removeAll(cache, node1, node2.getNextValueNode());
				}
				
				return cache.deepCopyNode(node1);
			}
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("REMOVE_ALL") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.removeAll(ldd2);
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				result.addAll(maps1);
				result.removeAll(maps2);
				return result;
			}
		};
	}
}

