package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDSetAll {
	public static LDDNode setAll(LDDCache<?, ?> cache, LDDNode node, Map<Integer, List<Integer>> newValuesPerKey) {
		List<Integer> newValues = newValuesPerKey.get(node.getKey());
		
		if (newValues != null) {
			LDDNode nextKeyNode;
			
			if (node.hasMoreKeys()) {
				nextKeyNode = setAll(cache, node.getNextKeyNode(), newValuesPerKey);
				LDDNode curr = node.getNextValueNode();
				
				while (curr != null) {
					nextKeyNode = LDDUnion.union(cache, nextKeyNode, setAll(cache, curr.getNextKeyNode(), newValuesPerKey));
					curr = curr.getNextValueNode();
				}
			} else {
				nextKeyNode = null;
			}
			
			LDDNode result = null;
			
			for (int index = newValues.size() - 1; index >= 0; index--) {
				result = cache.createNode(node.getKey(), newValues.get(index), result, nextKeyNode);
			}
			
			return result; //Should not be null!
		} else {
			LDDNode nextValueNode;
			
			if (node.getNextValueNode() != null) {
				nextValueNode = setAll(cache, node.getNextValueNode(), newValuesPerKey);
			} else {
				nextValueNode = null;
			}
			
			LDDNode nextKeyNode;
			
			if (node.getNextKeyNode() != null) {
				nextKeyNode = setAll(cache, node.getNextKeyNode(), newValuesPerKey);
			} else {
				nextKeyNode = null;
			}
			
			return cache.createNode(node.getKey(), node.getValue(), nextValueNode, nextKeyNode);
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("SET_ALL") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.setAll(getTestMultiMap());
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				
				if (maps1.isEmpty()) {
					if (getTestMultiMap().isEmpty()) {
						//Do nothing.
					} else {
						result.addAll(getTestFlattenedMultiMap());
					}
				} else {
					for (Map<String, String> map1 : maps1) {
						for (Map<String, String> map2 : getTestFlattenedMultiMap()) {
							Map<String, String> map1Copy = new HashMap<String, String>(map1);
							map1Copy.putAll(map2);
							result.add(map1Copy);
						}
					}
				}
				
				return result;
			}
		};
	}
}
