package jlx.utils.ldd.ops;

import java.util.*;

import jlx.utils.ldd.*;
import jlx.utils.ldd.test.*;

public class LDDUnion {
	public static LDDNode union(LDDCache<?, ?> cache, LDDNode node1, LDDNode node2) {
		if (node1.getValue() == node2.getValue()) {
			LDDNode nextValueNode;
			
			if (node1.getNextValueNode() != null) {
				if (node2.getNextValueNode() != null) {
					nextValueNode = union(cache, node1.getNextValueNode(), node2.getNextValueNode());
				} else {
					nextValueNode = cache.deepCopyNode(node1.getNextValueNode());
				}
			} else {
				if (node2.getNextValueNode() != null) {
					nextValueNode = cache.deepCopyNode(node2.getNextValueNode());
				} else {
					nextValueNode = null;
				}
			}
			
			if (node1.hasMoreKeys()) {
				LDDNode nextKeyNode = union(cache, node1.getNextKeyNode(), node2.getNextKeyNode());
				return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, nextKeyNode);
			} else {
				return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, null);
			}
		} else {
			if (node1.getValue() < node2.getValue()) {
				LDDNode nextValueNode;
				
				if (node1.getNextValueNode() != null) {
					nextValueNode = union(cache, node1.getNextValueNode(), node2);
				} else {
					nextValueNode = cache.deepCopyNode(node2);
				}
				
				return cache.createNode(node1.getKey(), node1.getValue(), nextValueNode, cache.deepCopyNode(node1.getNextKeyNode()));
			} else {
				LDDNode nextValueNode;
				
				if (node2.getNextValueNode() != null) {
					nextValueNode = union(cache, node2.getNextValueNode(), node1);
				} else {
					nextValueNode = cache.deepCopyNode(node1);
				}
				
				return cache.createNode(node2.getKey(), node2.getValue(), nextValueNode, cache.deepCopyNode(node2.getNextKeyNode()));
			}
		}
	}
	
	public static void main(String[] args) {
		new LDDSetTest("UNION") {
			@Override
			public LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2) {
				return ldd1.union(ldd2);
			}
			
			@Override
			public Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2) {
				Set<Map<String, String>> result = new HashSet<Map<String, String>>();
				result.addAll(maps1);
				result.addAll(maps2);
				return result;
			}
		};
	}
}
