package jlx.utils.ldd.test;

import java.util.*;

import jlx.utils.ldd.*;

public abstract class LDDSetTest extends LDDTestAbstracted {
	public LDDSetTest(String testName) {
		super(testName);
	}
	
	@Override
	public void compare(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2, LDD<String, String> ldd1, LDD<String, String> ldd2) {
		Set<Map<String, String>> x = validate(maps1, maps2);
		LDD<String, String> ldd3 = test(ldd1, ldd2);
		Set<Map<String, String>> y = ldd3.getElems();
		
		if (x.equals(y)) {
			if (printSuccesses()) {
				System.out.println(getTestName() + " passed!");
				System.out.println("\tInput 1:");
				printMaps(System.out, maps1);
				System.out.println("\tInput 2:");
				printMaps(System.out, maps2);
				System.out.println("\tOutput:");
				printMaps(System.out, x);
			}
		} else {
			System.err.println(getTestName() + " failed!");
			System.err.println("\tInput 1:");
			printMaps(System.err, maps1);
			System.err.println("\t\t" + ldd1);
			System.err.println("\tInput 2:");
			printMaps(System.err, maps2);
			System.err.println("\t\t" + ldd2);
			System.err.println("\tTest map:");
			printMap(System.err, getTestMap());
			System.err.println("\tTest multi-map:");
			printMultiMap(System.err, getTestMultiMap());
			System.err.println("\tTest prefix:");
			System.err.println("\t\t" + getTestPrefix().toString());
			System.err.println("\tOutput expected:");
			printMaps(System.err, x);
			System.err.println("\t\t" + createFromMaps(x, getNewEncoding(ldd1.getEncoding())));
			System.err.println("\tBut found:");
			printMaps(System.err, y);
			System.err.println("\t\t" + ldd3);
			
			ldd1.toGraphVizFile().saveToFile("ldd1.gv");
			ldd2.toGraphVizFile().saveToFile("ldd2.gv");
			createFromMaps(x).toGraphVizFile().saveToFile("ldd3-expected.gv");
			ldd3.toGraphVizFile().saveToFile("ldd3-found.gv");
			
			throw new Error("LDD test failed!");
		}
	}
	
	public abstract LDD<String, String> test(LDD<String, String> ldd1, LDD<String, String> ldd2);
	public abstract Set<Map<String, String>> validate(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2);
	
	public LDDEncoding<String, String> getNewEncoding(LDDEncoding<String, String> encoding) {
		return encoding;
	}
}

