package jlx.utils.ldd.test;

import java.io.*;
import java.util.*;

import jlx.utils.ldd.*;

public abstract class LDDTestAbstracted {
	private final static int COUNT = 10000;
	private final static List<String> KEYS = createOrderedKeys();
	private final static Map<String, Set<String>> PVS = createPvsPerKey();
	private final static LDDEncoding<String, String> ENCODING = new LDDEncoding<String, String>(PVS, KEYS);
	
	private String testName;
	private Map<String, Set<String>> testMultiMap;
	private Set<Map<String, String>> testFlattenedMultiMap;
	private List<String> testPrefix;
	private Map<String, String> testMap;
	
	public LDDTestAbstracted(String testName) {
		System.out.println("Running " + testName + "[*] tests . . .");
		
		for (int index = 1; index <= COUNT; index++) {
			this.testName = testName + "[" + index + "]";
			this.testMultiMap = null;
			this.testFlattenedMultiMap = null;
			this.testPrefix = null;
			this.testMap = null;
			
			Set<Map<String, String>> maps1 = createRandomMaps();
			Set<Map<String, String>> maps2 = createRandomMaps();
			
			if (Math.random() < 0.5) {
				Map<String, String> r = createRandomMap(0.1);
				maps1.add(r);
				maps2.add(r);
			}
			
			LDD<String, String> ldd1 = new LDD<String, String>(maps1, ENCODING);
			LDD<String, String> ldd2 = new LDD<String, String>(maps2, ENCODING);
			compare(maps1, maps2, ldd1, ldd2);
			
//			LDDGraphVizFile x = ldd1.toGraphVizFile();
//			
//			if (x.isTag("dashed")) {
//				x.saveToFile("test.gv");
//				System.exit(0);
//			}
		}
		
		System.out.println("Running " + testName + "[*] tests . . . done!");
	}
	
	public String getTestName() {
		return testName;
	}
	
	public Map<String, Set<String>> getTestMultiMap() {
		if (testMultiMap == null) {
			testMultiMap = createRandomMultiMap();
		}
		
		return testMultiMap;
	}
	
	public Set<Map<String, String>> getTestFlattenedMultiMap() {
		if (testFlattenedMultiMap == null) {
			testFlattenedMultiMap = flatten(getTestMultiMap());
		}
		
		return testFlattenedMultiMap;
	}
	
	public List<String> getTestPrefix() {
		if (testPrefix == null) {
			testPrefix = KEYS.subList(0, (int)(Math.random() * KEYS.size()) + 1);
		}
		
		return testPrefix;
	}
	
	public Map<String, String> getTestMap() {
		if (testMap == null) {
			testMap = createRandomMap(0.05);
		}
		
		return testMap;
	}
	
	public boolean printSuccesses() {
		return false;
	}
	
	public static void printMultiMap(PrintStream out, Map<String, Set<String>> map) {
		List<String> elems = new ArrayList<String>();
		
		for (Map.Entry<String, Set<String>> e : map.entrySet()) {
			Iterator<String> q = e.getValue().iterator();
			String s = q.next();
			
			while (q.hasNext()) {
				s += "," + q.next();
			}
			
			elems.add(e.getKey() + " = " + s);
		}
		
		if (elems.isEmpty()) {
			out.println("\t\t{}");
		} else {
			String s = elems.get(0);
			
			for (int index = 1; index < elems.size(); index++) {
				s += ", " + elems.get(index);
			}
			
			out.println("\t\t{ " + s + " }");
		}
	}
	
	public static void printMap(PrintStream out, Map<String, String> map) {
		List<String> elems = new ArrayList<String>();
		
		for (Map.Entry<String, String> e : map.entrySet()) {
			elems.add(e.getKey() + " = " + e.getValue());
		}
		
		if (elems.isEmpty()) {
			out.println("\t\t{}");
		} else {
			String s = elems.get(0);
			
			for (int index = 1; index < elems.size(); index++) {
				s += ", " + elems.get(index);
			}
			
			out.println("\t\t{ " + s + " }");
		}
	}
	
	public static void printMaps(PrintStream out, Set<Map<String, String>> maps) {
		for (Map<String, String> map : maps) {
			printMap(out, map);
		}
	}
	
	public static LDD<String, String> createFromMaps(Set<Map<String, String>> maps) {
		return new LDD<String, String>(maps, ENCODING);
	}
	
	public static LDD<String, String> createFromMaps(Set<Map<String, String>> maps, LDDEncoding<String, String> newEncoding) {
		return new LDD<String, String>(maps, newEncoding);
	}
	
	private static List<String> createOrderedKeys() {
		List<String> result = new ArrayList<String>();
		result.add("A");
		result.add("B");
		result.add("C");
		result.add("D");
		result.add("E");
		result.add("F");
		return result;
	}
	
	private static Map<String, Set<String>> createPvsPerKey() {
		Map<String, Set<String>> result = new HashMap<String, Set<String>>();
		result.put("A", Set.of("1", "2", "3", "4"));
		result.put("B", Set.of("5"));
		result.put("C", Set.of("6", "7", "8", "9", "10", "11", "12", "13"));
		result.put("D", Set.of("14", "15", "16"));
		result.put("E", Set.of("17", "18", "19", "20"));
		result.put("F", Set.of("21", "22"));
		return result;
	}
	
	private static Set<Map<String, String>> flatten(Map<String, Set<String>> pvsPerKey) {
		Set<Map<String, String>> result = new HashSet<Map<String, String>>();
		flatten(pvsPerKey, 0, new HashMap<String, String>(), result);
		return result;
	}
	
	private static void flatten(Map<String, Set<String>> pvsPerKey, int keyIndex, Map<String, String> soFar, Set<Map<String, String>> dest) {
		if (keyIndex < KEYS.size()) {
			String key = KEYS.get(keyIndex);
			Set<String> pvs = pvsPerKey.get(key);
			
			if (pvs != null) {
				for (String pv : pvs) {
					Map<String, String> newSoFar = new HashMap<String, String>(soFar);
					newSoFar.put(key, pv);
					
					flatten(pvsPerKey, keyIndex + 1, newSoFar, dest);
				}
			} else {
				flatten(pvsPerKey, keyIndex + 1, soFar, dest);
			}
		} else {
			dest.add(soFar);
		}
	}
	
	private static Map<String, Set<String>> createRandomMultiMap() {
		Map<String, Set<String>> result = new TreeMap<String, Set<String>>();
		
		for (Map.Entry<String, Set<String>> e : PVS.entrySet()) {
			if (e.getValue().size() > 0 && Math.random() < 0.33) {
				List<String> xs = new ArrayList<String>(e.getValue());
				
				for (int index = (int)(Math.random() * xs.size()); index > 0; index--) {
					xs.remove((int)(Math.random() * xs.size()));
				}
				
				result.put(e.getKey(), new HashSet<String>(xs));
			}
		}
		
		return result;
	}
	
	private static Map<String, String> createRandomMap(double nullFrac) {
		Map<String, String> result = new TreeMap<String, String>();
		
		for (Map.Entry<String, Set<String>> e : PVS.entrySet()) {
			if (e.getValue().size() > 0 && Math.random() < 1.0 - nullFrac) {
				List<String> xs = new ArrayList<String>(e.getValue());
				result.put(e.getKey(), xs.get((int)(Math.random() * xs.size())));
			}
		}
		
		return result;
	}
	
	private static Set<Map<String, String>> createRandomMaps() {
		Set<Map<String, String>> result = new HashSet<Map<String, String>>();
		
		for (int index = (int)(Math.random() * 15); index >= 1; index--) {
			result.add(createRandomMap(0.1));
		}
		
		return result;
	}
	
	public abstract void compare(Set<Map<String, String>> maps1, Set<Map<String, String>> maps2, LDD<String, String> ldd1, LDD<String, String> ldd2);
}
