package models.elx.p.v2x7x0A.scenarios;

import jlx.asal.j.*;
import jlx.scenario.*;
import models.elx.generic.types.EST_EfeS_State;
import models.elx.generic.types.PDI_Checksum_Data;
import models.elx.generic.types.PDI_Checksum_Result;
import models.elx.generic.types.PDI_Connection_State;
import models.elx.generic.types.PDI_Version;
import models.elx.p.types.Detection_State;
import models.elx.p.types.PM2_Activation;
import models.elx.p.types.PointPos;
import models.elx.p.v2x7x0A.DeutscheBahnPointModel;

/**
 * Inspired by DB_VAL_SCI-P_015.
 * 
 * Goal: See if Subsystem - Point detects the initial end positions correctly.
 * Extended with T5 = TRUE, so that
 * the initialization sequence is completed and the point position is reported.
 * 
 * X = NO_END_POSITION
 * Y = LEFT
 */
public class Scen015_Booting_2pm extends SubS_P_SD {
	public Scen015_Booting_2pm(DeutscheBahnPointModel m) {
		super(m);
	}

	@Override
	public Step[] getSteps() {
		return new Step[] {
			setInput(get_sec().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D32_Con_007600, JBool.FALSE),
			setInput(get_p3().D22_PM2_Position, new PointPos.LEFT()),
			setInput(get_p3().D34_Con_008000, JBool.TRUE),
			setInput(get_p3().D37_Con_008400, JBool.FALSE),
			setInput(get_sec().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_prim().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_est().D20_Con_MDM_Used, JBool.FALSE),
			setInput(get_p3().D35_Con_008200, JBool.FALSE),
			setInput(get_p3().D33_Con_007900, JBool.FALSE),
			setInput(get_p3().D4_Con_tmax_Point_Operation, new JInt._12000()),
			setInput(get_sec().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_prim().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D13_PM2_Activation, new PM2_Activation.ACTIVE()),
			setInput(get_p3().D36_Con_008300, JBool.FALSE),
			setInput(get_prim().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_p3().D21_PM1_Position, new PointPos.NO_END_POSITION()),
			setInput(get_prim().D2_Con_tmax_PDI_Connection, new JInt._20000()),
			setInput(get_p3().D30_Con_007000, JBool.FALSE),
			setInput(get_p3().D38_Con_008500, JBool.FALSE),
			expectStateChange(get_prim(), "prim[0][PDI_CONNECTION_CLOSED]", "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[0][NO_OPERATING_VOLTAGE]"),
			expectStateChange(get_p3(), "p3[0][STOPPED+WAITING_FOR_INITIALISING]"),
			expectStateChange(get_sec(), "sec[0][NOT_READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.TRUE), Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.FALSE)),
			stabilize(),
			setInput(get_est().T1_Power_On_Detected, JPulse.TRUE),
			expectStateChange(get_prim(), "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[0][NO_OPERATING_VOLTAGE]", "est[1][BOOTING]"),
			expectStateChange(get_p3(), "p3[0][STOPPED+WAITING_FOR_INITIALISING]"),
			expectStateChange(get_sec(), "sec[0][NOT_READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.NO_OPERATING_VOLTAGE()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING())),
			stabilize(),
			setInput(get_est().T4_Booted, JPulse.TRUE),
			expectStateChange(get_prim(), "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[1][BOOTING]", "est[4][INITIALISING]", "est[7][INITIALISING]"),
			expectStateChange(get_p3(), "p3[0][STOPPED+WAITING_FOR_INITIALISING]", "p3[3][STOPPED+NO_END_POSITION]", "p3[8][STOPPED+NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[0][NOT_READY_FOR_CONNECTION]", "sec[1][READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.TRUE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T4_Information_No_End_Position, JPulse.FALSE), Output.from(get_p3().T4_Information_No_End_Position, JPulse.TRUE), Output.from(get_p3().T4_Information_No_End_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.UNKNOWN()), Output.from(get_p3().D20_Point_Position, new PointPos.NO_END_POSITION())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY())),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING())),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.FALSE()), Output.from(get_p3().D6_Detection_State, new Detection_State.NO_END_POSITION())),
			stabilize(),
			setInput(get_prim().T5_SCP_Connection_Established, JPulse.TRUE),
			setInput(get_sec().T5_SCP_Connection_Established, JPulse.TRUE),
			expectStateChange(get_prim(), "prim[2][PDI_CONNECTION_CLOSED]", "prim[1][WAITING_FOR_VERSION_CHECK]", "prim[6][WAITING_FOR_VERSION_CHECK]", "prim[3][WAITING_FOR_INITIALISATION]", "prim[13][WAITING_FOR_INITIALISATION]", "prim[14][RECEIVING_STATUS]", "prim[16][PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]", "sp[4][STATUS_REPORTED]", "sp[22][STATUS_REPORTED]", "sp[12][PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_fp(), "fp[0][WATING]", "fp[1][REPORT_STATUS]", "fp[9][REPORT_STATUS]", "fp[3][STATUS_REPORTED]", "fp[36][STATUS_REPORTED]", "fp[35][PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[7][INITIALISING]", "est[5][OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[8][STOPPED+NO_END_POSITION]", "p3[11][STOPPED+NO_END_POSITION]", "p3[8][STOPPED+NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[1][READY_FOR_CONNECTION]", "sec[2][READY_FOR_VERSION_CHECK]", "sec[5][READY_FOR_INITIALISATION]", "sec[9][READY_FOR_INITIALISATION]", "sec[8][SENDING_STATUS]", "sec[17][SENDING_STATUS]", "sec[16][PDI_CONNECTION_ESTABLISHED]", "sec[18][PDI_CONNECTION_ESTABLISHED]"),
			expectOutput(Output.from(get_sec().T6_Start_Status_Report, JPulse.FALSE), Output.from(get_sec().T6_Start_Status_Report, JPulse.TRUE), Output.from(get_sec().T6_Start_Status_Report, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().T7_Cd_PDI_Version_Check, JPulse.FALSE), Output.from(get_prim().DT7_PDI_Version, new PDI_Version.V1(), get_prim().T7_Cd_PDI_Version_Check, JPulse.TRUE), Output.from(get_prim().T7_Cd_PDI_Version_Check, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().D2_Point_Position, new PointPos.NONE()), Output.from(get_fp().D2_Point_Position, new PointPos.NO_END_POSITION())),
			expectOutput(Output.from(get_sp().T20_Point_Position, JPulse.FALSE), Output.from(get_sp().DT20_Point_Position, new PointPos.NO_END_POSITION(), get_sp().T20_Point_Position, JPulse.TRUE), Output.from(get_sp().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_REQUESTED()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.WAITING_FOR_VERSION_CHECK()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.WAITING_FOR_INITIALISATION()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.RECEIVING_STATUS()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.ESTABLISHED())),
			expectOutput(Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.FALSE), Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.TRUE), Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T13_Msg_PDI_Version_Check, JPulse.FALSE), Output.from(get_sec().DT13_Result, new PDI_Checksum_Result.Match(), get_sec().T13_Msg_PDI_Version_Check, JPulse.TRUE, get_sec().DT13_Checksum_Data, new PDI_Checksum_Data.NotApplicable()), Output.from(get_sec().T13_Msg_PDI_Version_Check, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T40_Send_Status_Report, JPulse.FALSE), Output.from(get_fp().T40_Send_Status_Report, JPulse.TRUE), Output.from(get_fp().T40_Send_Status_Report, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.FALSE), Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.TRUE), Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T11_PDI_Connection_Established, JPulse.FALSE), Output.from(get_sec().T11_PDI_Connection_Established, JPulse.TRUE), Output.from(get_sec().T11_PDI_Connection_Established, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.FALSE), Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.TRUE), Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.FALSE), Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.TRUE), Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.FALSE)),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.OPERATIONAL())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.READY_FOR_VERSION_CHECK()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.READY_FOR_INITIALISATION()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.SENDING_STATUS()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.ESTABLISHED())),
			stabilize()
		};
	}
}
