package models.elx.p.v2x7x0A.scenarios;

import jlx.asal.j.*;
import jlx.scenario.*;
import models.elx.generic.types.EST_EfeS_State;
import models.elx.generic.types.PDI_Checksum_Data;
import models.elx.generic.types.PDI_Checksum_Result;
import models.elx.generic.types.PDI_Connection_State;
import models.elx.generic.types.PDI_Version;
import models.elx.p.types.Detection_State;
import models.elx.p.types.DriveState;
import models.elx.p.types.PM2_Activation;
import models.elx.p.types.PointPos;
import models.elx.p.v2x7x0A.DeutscheBahnPointModel;

/**
 * Inspired by DB_VAL_SCI-P_057.
 * 
 * 1. Start in end position X.
 * 2. Initialize the system; end position X is reported.
 * 3. EIL instructs end position Y; point machines are instructed.
 * 4. Point machines achieve NO_END_POSITION; NO_END_POSITION is reported.
 * 5. - Point machines achieve end position Y.
 *    - EIL instructs end position Z.
 * 6. - End position X is reported.
 *    - Point machine are instructed.
 * 7. End position Z is achieved.
 * 8. End position Z is reported. 
 * 
 * X = LEFT
 * Y = RIGHT
 * Z = LEFT
 */
public class Scen058_Reversal_NonDet_1pm extends SubS_P_SD {
	public Scen058_Reversal_NonDet_1pm(DeutscheBahnPointModel m) {
		super(m);
	}

	@Override
	public Step[] getSteps() {
		return new Step[] {
			setInput(get_sec().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D32_Con_007600, JBool.FALSE),
			setInput(get_p3().D22_PM2_Position, new PointPos.NO_END_POSITION()),
			setInput(get_p3().D34_Con_008000, JBool.TRUE),
			setInput(get_p3().D37_Con_008400, JBool.FALSE),
			setInput(get_sec().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_prim().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_est().D20_Con_MDM_Used, JBool.FALSE),
			setInput(get_p3().D35_Con_008200, JBool.FALSE),
			setInput(get_p3().D33_Con_007900, JBool.FALSE),
			setInput(get_p3().D4_Con_tmax_Point_Operation, new JInt._12000()),
			setInput(get_sec().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_prim().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D13_PM2_Activation, new PM2_Activation.INACTIVE()),
			setInput(get_p3().D36_Con_008300, JBool.FALSE),
			setInput(get_prim().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_p3().D21_PM1_Position, new PointPos.LEFT()),
			setInput(get_prim().D2_Con_tmax_PDI_Connection, new JInt._20000()),
			setInput(get_p3().D30_Con_007000, JBool.FALSE),
			setInput(get_p3().D38_Con_008500, JBool.FALSE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=TRUE]", "prim[PDI_CONNECTION_CLOSED][T6=FALSE]"),
			expectStateChange(get_est(), "est[NO_OPERATING_VOLTAGE][T18=FALSE,T22=FALSE]"),
			expectStateChange(get_p3(), "p3[STOPPED+WAITING_FOR_INITIALISING][D20=UNKNOWN,D6=False,Mem=NO_END_POSITION,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[NOT_READY_FOR_CONNECTION][T17=FALSE]"),
			expectOutput(Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.TRUE), Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.FALSE)),
			stabilize(),
			setInput(get_est().T1_Power_On_Detected, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=FALSE]"),
			expectStateChange(get_est(), "est[NO_OPERATING_VOLTAGE][T18=FALSE,T22=FALSE]", "est[BOOTING][T18=FALSE,T22=FALSE]"),
			expectStateChange(get_p3(), "p3[STOPPED+WAITING_FOR_INITIALISING][D20=UNKNOWN,D6=False,Mem=NO_END_POSITION,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[NOT_READY_FOR_CONNECTION][T17=FALSE]"),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.NO_OPERATING_VOLTAGE()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING())),
			stabilize(),
			setInput(get_est().T4_Booted, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=FALSE]"),
			expectStateChange(get_est(), "est[BOOTING][T18=FALSE,T22=FALSE]", "est[INITIALISING][T21=TRUE]", "est[INITIALISING][T21=FALSE]"),
			expectStateChange(get_p3(), "p3[STOPPED+WAITING_FOR_INITIALISING][D20=UNKNOWN,D6=False,Mem=NO_END_POSITION,T20=FALSE,T30=FALSE]", "p3[ALL_LEFT+STOPPED][T20=TRUE,T30=FALSE,T5=TRUE]", "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[NOT_READY_FOR_CONNECTION][T17=FALSE]", "sec[READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.UNKNOWN()), Output.from(get_p3().D20_Point_Position, new PointPos.LEFT())),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.FALSE()), Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY())),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING())),
			expectOutput(Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.TRUE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			stabilize(),
			setInput(get_prim().T5_SCP_Connection_Established, JPulse.TRUE),
			setInput(get_sec().T5_SCP_Connection_Established, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]", "sp[STATUS_REPORTED][DT20=LEFT,Mem=LEFT,T20=TRUE]", "sp[STATUS_REPORTED][DT20=,Mem=LEFT,T20=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=FALSE]", "prim[WAITING_FOR_VERSION_CHECK][T7=TRUE]", "prim[WAITING_FOR_VERSION_CHECK][T7=FALSE]", "prim[WAITING_FOR_INITIALISATION][T8=TRUE]", "prim[WAITING_FOR_INITIALISATION][T8=FALSE]", "prim[RECEIVING_STATUS]", "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[INITIALISING][T21=FALSE]", "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]", "p3[ALL_LEFT+STOPPED][T20=TRUE,T30=FALSE,T5=FALSE]", "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]", "fp[REPORT_STATUS][D2=,Mem=NO_END_POSITION,T40=TRUE]", "fp[REPORT_STATUS][D2=,Mem=NO_END_POSITION,T40=FALSE]", "fp[STATUS_REPORTED][D2=LEFT,Mem=LEFT,T23=TRUE,T2=TRUE]", "fp[STATUS_REPORTED][D2=LEFT,Mem=LEFT,T23=FALSE,T2=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[READY_FOR_CONNECTION]", "sec[READY_FOR_VERSION_CHECK]", "sec[READY_FOR_INITIALISATION][DT13=match,T13=TRUE]", "sec[READY_FOR_INITIALISATION][DT13=,T13=FALSE]", "sec[SENDING_STATUS][T14=TRUE,T6=TRUE]", "sec[SENDING_STATUS][T14=FALSE,T6=FALSE]", "sec[PDI_CONNECTION_ESTABLISHED][T11=TRUE,T15=TRUE]", "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sec().T6_Start_Status_Report, JPulse.FALSE), Output.from(get_sec().T6_Start_Status_Report, JPulse.TRUE), Output.from(get_sec().T6_Start_Status_Report, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T11_PDI_Connection_Established, JPulse.FALSE), Output.from(get_sec().T11_PDI_Connection_Established, JPulse.TRUE), Output.from(get_sec().T11_PDI_Connection_Established, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_REQUESTED()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.WAITING_FOR_VERSION_CHECK()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.WAITING_FOR_INITIALISATION()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.RECEIVING_STATUS()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.ESTABLISHED())),
			expectOutput(Output.from(get_prim().T7_Cd_PDI_Version_Check, JPulse.FALSE), Output.from(get_prim().DT7_PDI_Version, new PDI_Version.V1(), get_prim().T7_Cd_PDI_Version_Check, JPulse.TRUE), Output.from(get_prim().T7_Cd_PDI_Version_Check, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.FALSE), Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.TRUE), Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.FALSE)),
			expectOutput(Output.from(get_sp().T20_Point_Position, JPulse.FALSE), Output.from(get_sp().DT20_Point_Position, new PointPos.LEFT(), get_sp().T20_Point_Position, JPulse.TRUE), Output.from(get_sp().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.FALSE), Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.TRUE), Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T13_Msg_PDI_Version_Check, JPulse.FALSE), Output.from(get_sec().DT13_Result, new PDI_Checksum_Result.Match(), get_sec().T13_Msg_PDI_Version_Check, JPulse.TRUE, get_sec().DT13_Checksum_Data, new PDI_Checksum_Data.NotApplicable()), Output.from(get_sec().T13_Msg_PDI_Version_Check, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.FALSE), Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.TRUE), Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().D2_Point_Position, new PointPos.NONE()), Output.from(get_fp().D2_Point_Position, new PointPos.LEFT())),
			expectOutput(Output.from(get_fp().T40_Send_Status_Report, JPulse.FALSE), Output.from(get_fp().T40_Send_Status_Report, JPulse.TRUE), Output.from(get_fp().T40_Send_Status_Report, JPulse.FALSE)),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.OPERATIONAL())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.READY_FOR_VERSION_CHECK()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.READY_FOR_INITIALISATION()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.SENDING_STATUS()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.ESTABLISHED())),
			expectOutput(Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.FALSE), Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.TRUE), Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.FALSE)),
			stabilize(),
			setInput(get_sp().DT10_Move_Point, new PointPos.RIGHT()),
			setInput(get_sp().T10_Move_Point, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=RIGHT,DT20=,T1=TRUE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]", "p3[ALL_LEFT+MOVING_RIGHT][T20=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=RIGHT,T10=TRUE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE), Output.from(get_sp().T1_Cd_Move_Point, JPulse.TRUE, get_sp().DT1_Move_Point_Target, new PointPos.RIGHT()), Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D11_Move_Right, JBool.FALSE), Output.from(get_p3().D11_Move_Right, JBool.TRUE)),
			expectOutput(Output.from(get_fp().T10_Move, JPulse.FALSE), Output.from(get_fp().DT10_Move_Target, new PointPos.RIGHT(), get_fp().T10_Move, JPulse.TRUE), Output.from(get_fp().T10_Move, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D5_Drive_State, new DriveState.STOPPED()), Output.from(get_p3().D5_Drive_State, new DriveState.MOVING())),
			stabilize(),
			setInput(get_p3().D21_PM1_Position, new PointPos.NO_END_POSITION()),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=NO_END_POSITION,T1=FALSE,T20=TRUE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_LEFT+MOVING_RIGHT][T20=FALSE,T5=FALSE]", "p3[MOVING_RIGHT+NO_END_POSITION][T20=TRUE,T4=TRUE]", "p3[MOVING_RIGHT+NO_END_POSITION][T20=FALSE,T4=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=TRUE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_fp().D2_Point_Position, new PointPos.LEFT()), Output.from(get_fp().D2_Point_Position, new PointPos.NO_END_POSITION())),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.LEFT()), Output.from(get_p3().D20_Point_Position, new PointPos.NO_END_POSITION())),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION()), Output.from(get_p3().D6_Detection_State, new Detection_State.NO_END_POSITION())),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T4_Information_No_End_Position, JPulse.FALSE), Output.from(get_p3().T4_Information_No_End_Position, JPulse.TRUE), Output.from(get_p3().T4_Information_No_End_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_sp().T20_Point_Position, JPulse.FALSE), Output.from(get_sp().DT20_Point_Position, new PointPos.NO_END_POSITION(), get_sp().T20_Point_Position, JPulse.TRUE), Output.from(get_sp().T20_Point_Position, JPulse.FALSE)),
			stabilize(),
			setInput(get_p3().D21_PM1_Position, new PointPos.RIGHT()),
			setInput(get_sp().DT10_Move_Point, new PointPos.LEFT()),
			setInput(get_sp().T10_Move_Point, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=LEFT,DT20=,T1=TRUE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[MOVING_RIGHT+NO_END_POSITION][T20=FALSE,T4=FALSE]", "p3[ALL_RIGHT+STOPPED][T20=TRUE,T30=FALSE,T5=TRUE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE), Output.from(get_sp().T1_Cd_Move_Point, JPulse.TRUE, get_sp().DT1_Move_Point_Target, new PointPos.LEFT())),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE)),
			expectOutput(Output.from(get_p3().D11_Move_Right, JBool.TRUE), Output.from(get_p3().D11_Move_Right, JBool.FALSE)),
			expectOutput(Output.from(get_p3().D5_Drive_State, new DriveState.MOVING()), Output.from(get_p3().D5_Drive_State, new DriveState.STOPPED())),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.NO_END_POSITION()), Output.from(get_p3().D20_Point_Position, new PointPos.RIGHT())),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.NO_END_POSITION()), Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION())),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE)),
			step(),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=LEFT,DT20=,T1=TRUE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_RIGHT+STOPPED][T20=TRUE,T30=FALSE,T5=TRUE]", "p3[ALL_RIGHT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=RIGHT,DT10=,T10=FALSE,T2=TRUE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_sp().T1_Cd_Move_Point, JPulse.TRUE, get_sp().DT1_Move_Point_Target, new PointPos.LEFT()), Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE)),
			expectOutput(Output.from(get_fp().D2_Point_Position, new PointPos.NO_END_POSITION()), Output.from(get_fp().D2_Point_Position, new PointPos.RIGHT())),
			step(),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=RIGHT,T1=FALSE,T20=TRUE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_RIGHT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=RIGHT,DT10=,T10=FALSE,T2=TRUE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=RIGHT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sp().T20_Point_Position, JPulse.FALSE), Output.from(get_sp().DT20_Point_Position, new PointPos.RIGHT(), get_sp().T20_Point_Position, JPulse.TRUE), Output.from(get_sp().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE)),
			stabilize()
		};
	}
	
	@Override
	public Step[] getAltSteps() {
		return new Step[] {
			setInput(get_sec().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D32_Con_007600, JBool.FALSE),
			setInput(get_p3().D22_PM2_Position, new PointPos.NO_END_POSITION()),
			setInput(get_p3().D34_Con_008000, JBool.TRUE),
			setInput(get_p3().D37_Con_008400, JBool.FALSE),
			setInput(get_sec().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_prim().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_est().D20_Con_MDM_Used, JBool.FALSE),
			setInput(get_p3().D35_Con_008200, JBool.FALSE),
			setInput(get_p3().D33_Con_007900, JBool.FALSE),
			setInput(get_p3().D4_Con_tmax_Point_Operation, new JInt._12000()),
			setInput(get_sec().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_prim().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D13_PM2_Activation, new PM2_Activation.INACTIVE()),
			setInput(get_p3().D36_Con_008300, JBool.FALSE),
			setInput(get_prim().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_p3().D21_PM1_Position, new PointPos.LEFT()),
			setInput(get_prim().D2_Con_tmax_PDI_Connection, new JInt._20000()),
			setInput(get_p3().D30_Con_007000, JBool.FALSE),
			setInput(get_p3().D38_Con_008500, JBool.FALSE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=TRUE]", "prim[PDI_CONNECTION_CLOSED][T6=FALSE]"),
			expectStateChange(get_est(), "est[NO_OPERATING_VOLTAGE][T18=FALSE,T22=FALSE]"),
			expectStateChange(get_p3(), "p3[STOPPED+WAITING_FOR_INITIALISING][D20=UNKNOWN,D6=False,Mem=NO_END_POSITION,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[NOT_READY_FOR_CONNECTION][T17=FALSE]"),
			expectOutput(Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.TRUE), Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.FALSE)),
			stabilize(),
			setInput(get_est().T1_Power_On_Detected, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=FALSE]"),
			expectStateChange(get_est(), "est[NO_OPERATING_VOLTAGE][T18=FALSE,T22=FALSE]", "est[BOOTING][T18=FALSE,T22=FALSE]"),
			expectStateChange(get_p3(), "p3[STOPPED+WAITING_FOR_INITIALISING][D20=UNKNOWN,D6=False,Mem=NO_END_POSITION,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[NOT_READY_FOR_CONNECTION][T17=FALSE]"),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.NO_OPERATING_VOLTAGE()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING())),
			stabilize(),
			setInput(get_est().T4_Booted, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=FALSE]"),
			expectStateChange(get_est(), "est[BOOTING][T18=FALSE,T22=FALSE]", "est[INITIALISING][T21=TRUE]", "est[INITIALISING][T21=FALSE]"),
			expectStateChange(get_p3(), "p3[STOPPED+WAITING_FOR_INITIALISING][D20=UNKNOWN,D6=False,Mem=NO_END_POSITION,T20=FALSE,T30=FALSE]", "p3[ALL_LEFT+STOPPED][T20=TRUE,T30=FALSE,T5=TRUE]", "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]"),
			expectStateChange(get_sec(), "sec[NOT_READY_FOR_CONNECTION][T17=FALSE]", "sec[READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.UNKNOWN()), Output.from(get_p3().D20_Point_Position, new PointPos.LEFT())),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.FALSE()), Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY())),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING())),
			expectOutput(Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.TRUE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			stabilize(),
			setInput(get_prim().T5_SCP_Connection_Established, JPulse.TRUE),
			setInput(get_sec().T5_SCP_Connection_Established, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[REPORT_STATUS][Mem=NO_END_POSITION]", "sp[STATUS_REPORTED][DT20=LEFT,Mem=LEFT,T20=TRUE]", "sp[STATUS_REPORTED][DT20=,Mem=LEFT,T20=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_CLOSED][T6=FALSE]", "prim[WAITING_FOR_VERSION_CHECK][T7=TRUE]", "prim[WAITING_FOR_VERSION_CHECK][T7=FALSE]", "prim[WAITING_FOR_INITIALISATION][T8=TRUE]", "prim[WAITING_FOR_INITIALISATION][T8=FALSE]", "prim[RECEIVING_STATUS]", "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[INITIALISING][T21=FALSE]", "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]", "p3[ALL_LEFT+STOPPED][T20=TRUE,T30=FALSE,T5=FALSE]", "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[WATING][D2=,Mem=NO_END_POSITION]", "fp[REPORT_STATUS][D2=,Mem=NO_END_POSITION,T40=TRUE]", "fp[REPORT_STATUS][D2=,Mem=NO_END_POSITION,T40=FALSE]", "fp[STATUS_REPORTED][D2=LEFT,Mem=LEFT,T23=TRUE,T2=TRUE]", "fp[STATUS_REPORTED][D2=LEFT,Mem=LEFT,T23=FALSE,T2=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[READY_FOR_CONNECTION]", "sec[READY_FOR_VERSION_CHECK]", "sec[READY_FOR_INITIALISATION][DT13=match,T13=TRUE]", "sec[READY_FOR_INITIALISATION][DT13=,T13=FALSE]", "sec[SENDING_STATUS][T14=TRUE,T6=TRUE]", "sec[SENDING_STATUS][T14=FALSE,T6=FALSE]", "sec[PDI_CONNECTION_ESTABLISHED][T11=TRUE,T15=TRUE]", "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sec().T6_Start_Status_Report, JPulse.FALSE), Output.from(get_sec().T6_Start_Status_Report, JPulse.TRUE), Output.from(get_sec().T6_Start_Status_Report, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T11_PDI_Connection_Established, JPulse.FALSE), Output.from(get_sec().T11_PDI_Connection_Established, JPulse.TRUE), Output.from(get_sec().T11_PDI_Connection_Established, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_REQUESTED()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.WAITING_FOR_VERSION_CHECK()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.WAITING_FOR_INITIALISATION()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.RECEIVING_STATUS()), Output.from(get_prim().D50_PDI_Connection_State, new PDI_Connection_State.ESTABLISHED())),
			expectOutput(Output.from(get_prim().T7_Cd_PDI_Version_Check, JPulse.FALSE), Output.from(get_prim().DT7_PDI_Version, new PDI_Version.V1(), get_prim().T7_Cd_PDI_Version_Check, JPulse.TRUE), Output.from(get_prim().T7_Cd_PDI_Version_Check, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.FALSE), Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.TRUE), Output.from(get_prim().T8_Cd_Initialisation_Request, JPulse.FALSE)),
			expectOutput(Output.from(get_sp().T20_Point_Position, JPulse.FALSE), Output.from(get_sp().DT20_Point_Position, new PointPos.LEFT(), get_sp().T20_Point_Position, JPulse.TRUE), Output.from(get_sp().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.FALSE), Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.TRUE), Output.from(get_sec().T14_Msg_Start_Initialisation, JPulse.FALSE)),
			expectOutput(Output.from(get_sec().T13_Msg_PDI_Version_Check, JPulse.FALSE), Output.from(get_sec().DT13_Result, new PDI_Checksum_Result.Match(), get_sec().T13_Msg_PDI_Version_Check, JPulse.TRUE, get_sec().DT13_Checksum_Data, new PDI_Checksum_Data.NotApplicable()), Output.from(get_sec().T13_Msg_PDI_Version_Check, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.FALSE), Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.TRUE), Output.from(get_fp().T23_Sending_Status_Report_Completed, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().D2_Point_Position, new PointPos.NONE()), Output.from(get_fp().D2_Point_Position, new PointPos.LEFT())),
			expectOutput(Output.from(get_fp().T40_Send_Status_Report, JPulse.FALSE), Output.from(get_fp().T40_Send_Status_Report, JPulse.TRUE), Output.from(get_fp().T40_Send_Status_Report, JPulse.FALSE)),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.OPERATIONAL())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.READY_FOR_VERSION_CHECK()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.READY_FOR_INITIALISATION()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.SENDING_STATUS()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.ESTABLISHED())),
			expectOutput(Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.FALSE), Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.TRUE), Output.from(get_sec().T15_Msg_Initialisation_Completed, JPulse.FALSE)),
			stabilize(),
			setInput(get_sp().DT10_Move_Point, new PointPos.RIGHT()),
			setInput(get_sp().T10_Move_Point, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=RIGHT,DT20=,T1=TRUE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_LEFT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]", "p3[ALL_LEFT+MOVING_RIGHT][T20=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=RIGHT,T10=TRUE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE), Output.from(get_sp().T1_Cd_Move_Point, JPulse.TRUE, get_sp().DT1_Move_Point_Target, new PointPos.RIGHT()), Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D11_Move_Right, JBool.FALSE), Output.from(get_p3().D11_Move_Right, JBool.TRUE)),
			expectOutput(Output.from(get_fp().T10_Move, JPulse.FALSE), Output.from(get_fp().DT10_Move_Target, new PointPos.RIGHT(), get_fp().T10_Move, JPulse.TRUE), Output.from(get_fp().T10_Move, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D5_Drive_State, new DriveState.STOPPED()), Output.from(get_p3().D5_Drive_State, new DriveState.MOVING())),
			stabilize(),
			setInput(get_p3().D21_PM1_Position, new PointPos.NO_END_POSITION()),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=NO_END_POSITION,T1=FALSE,T20=TRUE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_LEFT+MOVING_RIGHT][T20=FALSE,T5=FALSE]", "p3[MOVING_RIGHT+NO_END_POSITION][T20=TRUE,T4=TRUE]", "p3[MOVING_RIGHT+NO_END_POSITION][T20=FALSE,T4=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=LEFT,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=TRUE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_fp().D2_Point_Position, new PointPos.LEFT()), Output.from(get_fp().D2_Point_Position, new PointPos.NO_END_POSITION())),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.LEFT()), Output.from(get_p3().D20_Point_Position, new PointPos.NO_END_POSITION())),
			expectOutput(Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.TRUE), Output.from(get_fp().T2_Msg_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION()), Output.from(get_p3().D6_Detection_State, new Detection_State.NO_END_POSITION())),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T4_Information_No_End_Position, JPulse.FALSE), Output.from(get_p3().T4_Information_No_End_Position, JPulse.TRUE), Output.from(get_p3().T4_Information_No_End_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_sp().T20_Point_Position, JPulse.FALSE), Output.from(get_sp().DT20_Point_Position, new PointPos.NO_END_POSITION(), get_sp().T20_Point_Position, JPulse.TRUE), Output.from(get_sp().T20_Point_Position, JPulse.FALSE)),
			stabilize(),
			setInput(get_p3().D21_PM1_Position, new PointPos.RIGHT()),
			setInput(get_sp().DT10_Move_Point, new PointPos.LEFT()),
			setInput(get_sp().T10_Move_Point, JPulse.TRUE),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=LEFT,DT20=,T1=TRUE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[MOVING_RIGHT+NO_END_POSITION][T20=FALSE,T4=FALSE]", "p3[ALL_RIGHT+STOPPED][T20=TRUE,T30=FALSE,T5=TRUE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE), Output.from(get_sp().T1_Cd_Move_Point, JPulse.TRUE, get_sp().DT1_Move_Point_Target, new PointPos.LEFT())),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE)),
			expectOutput(Output.from(get_p3().D11_Move_Right, JBool.TRUE), Output.from(get_p3().D11_Move_Right, JBool.FALSE)),
			expectOutput(Output.from(get_p3().D5_Drive_State, new DriveState.MOVING()), Output.from(get_p3().D5_Drive_State, new DriveState.STOPPED())),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.NO_END_POSITION()), Output.from(get_p3().D20_Point_Position, new PointPos.RIGHT())),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.NO_END_POSITION()), Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION())),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE)),
			step(),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=LEFT,DT20=,T1=TRUE,T20=FALSE,T30=FALSE]", "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_RIGHT+STOPPED][T20=TRUE,T30=FALSE,T5=TRUE]", "p3[ALL_RIGHT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=LEFT,T10=TRUE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_sp().T1_Cd_Move_Point, JPulse.TRUE, get_sp().DT1_Move_Point_Target, new PointPos.LEFT()), Output.from(get_sp().T1_Cd_Move_Point, JPulse.FALSE)),
			expectOutput(Output.from(get_fp().T10_Move, JPulse.FALSE), Output.from(get_fp().DT10_Move_Target, new PointPos.LEFT(), get_fp().T10_Move, JPulse.TRUE)),
			step(),
			expectStateChange(get_sp(), "sp[PDI_CONNECTION_ESTABLISHED][DT1=,DT20=,T1=FALSE,T20=FALSE,T30=FALSE]"),
			expectStateChange(get_prim(), "prim[PDI_CONNECTION_ESTABLISHED]"),
			expectStateChange(get_est(), "est[OPERATIONAL]"),
			expectStateChange(get_p3(), "p3[ALL_RIGHT+STOPPED][T20=FALSE,T30=FALSE,T5=FALSE]", "p3[ALL_RIGHT+MOVING_LEFT][T20=FALSE,T5=FALSE]"),
			expectStateChange(get_fp(), "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=LEFT,T10=TRUE,T2=FALSE,T3=FALSE]", "fp[PDI_CONNECTION_ESTABLISHED][D2=NO_END_POSITION,DT10=,T10=FALSE,T2=FALSE,T3=FALSE]"),
			expectStateChange(get_sec(), "sec[PDI_CONNECTION_ESTABLISHED][T11=FALSE,T15=FALSE]"),
			expectOutput(Output.from(get_p3().D10_Move_Left, JBool.FALSE), Output.from(get_p3().D10_Move_Left, JBool.TRUE)),
			expectOutput(Output.from(get_fp().DT10_Move_Target, new PointPos.LEFT(), get_fp().T10_Move, JPulse.TRUE), Output.from(get_fp().T10_Move, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D5_Drive_State, new DriveState.STOPPED()), Output.from(get_p3().D5_Drive_State, new DriveState.MOVING())),
			stabilize()
		};
	}
}
