package models.elx.p.v2x7x0A.scenarios;

import jlx.asal.j.*;
import jlx.scenario.*;
import models.elx.generic.types.EST_EfeS_State;
import models.elx.generic.types.PDI_Checksum_Data;
import models.elx.generic.types.PDI_Connection_State;
import models.elx.generic.types.PDI_Version;
import models.elx.p.types.Detection_State;
import models.elx.p.types.PM2_Activation;
import models.elx.p.types.PointPos;
import models.elx.p.v2x7x0A.DeutscheBahnPointModel;

/**
 * Inspired by DB_VAL_SCI-P_091.
 * 
 * 1. Start in end position X.
 * 2. Power, boot.
 * 3. T10_SCP_Connection_Terminated.
 * 
 * X = RIGHT
 */
public class Scen091_Disconnect_1pm extends SubS_P_SD {
	public Scen091_Disconnect_1pm(DeutscheBahnPointModel m) {
		super(m);
	}

	@Override
	public Step[] getSteps() {
		return new Step[] {
			setInput(get_sec().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D32_Con_007600, JBool.FALSE),
			setInput(get_p3().D22_PM2_Position, new PointPos.NO_END_POSITION()),
			setInput(get_p3().D34_Con_008000, JBool.TRUE),
			setInput(get_p3().D37_Con_008400, JBool.FALSE),
			setInput(get_sec().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_prim().D4_Con_Checksum_Data, new PDI_Checksum_Data.D1()),
			setInput(get_est().D20_Con_MDM_Used, JBool.FALSE),
			setInput(get_p3().D35_Con_008200, JBool.FALSE),
			setInput(get_p3().D33_Con_007900, JBool.FALSE),
			setInput(get_p3().D4_Con_tmax_Point_Operation, new JInt._12000()),
			setInput(get_sec().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_prim().D3_Con_PDI_Version, new PDI_Version.V1()),
			setInput(get_p3().D13_PM2_Activation, new PM2_Activation.INACTIVE()),
			setInput(get_p3().D36_Con_008300, JBool.FALSE),
			setInput(get_prim().D23_Con_Checksum_Data_Used, JBool.FALSE),
			setInput(get_p3().D21_PM1_Position, new PointPos.RIGHT()),
			setInput(get_prim().D2_Con_tmax_PDI_Connection, new JInt._20000()),
			setInput(get_p3().D30_Con_007000, JBool.FALSE),
			setInput(get_p3().D38_Con_008500, JBool.FALSE),
			expectStateChange(get_prim(), "prim[0][PDI_CONNECTION_CLOSED]", "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[0][NO_OPERATING_VOLTAGE]"),
			expectStateChange(get_p3(), "p3[0][STOPPED+WAITING_FOR_INITIALISING]"),
			expectStateChange(get_sec(), "sec[0][NOT_READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.TRUE), Output.from(get_prim().T6_Establish_SCP_Connection, JPulse.FALSE)),
			stabilize(),
			setInput(get_est().T1_Power_On_Detected, JPulse.TRUE),
			expectStateChange(get_prim(), "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[0][NO_OPERATING_VOLTAGE]", "est[1][BOOTING]"),
			expectStateChange(get_p3(), "p3[0][STOPPED+WAITING_FOR_INITIALISING]"),
			expectStateChange(get_sec(), "sec[0][NOT_READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.NO_OPERATING_VOLTAGE()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING())),
			stabilize(),
			setInput(get_est().T4_Booted, JPulse.TRUE),
			expectStateChange(get_prim(), "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[1][BOOTING]", "est[4][INITIALISING]", "est[7][INITIALISING]"),
			expectStateChange(get_p3(), "p3[0][STOPPED+WAITING_FOR_INITIALISING]", "p3[5][STOPPED+ALL_RIGHT]", "p3[23][STOPPED+ALL_RIGHT]"),
			expectStateChange(get_sec(), "sec[0][NOT_READY_FOR_CONNECTION]", "sec[1][READY_FOR_CONNECTION]"),
			expectOutput(Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.TRUE), Output.from(get_p3().T5_Info_End_Position_Arrived, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D20_Point_Position, new PointPos.UNKNOWN()), Output.from(get_p3().D20_Point_Position, new PointPos.RIGHT())),
			expectOutput(Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.TRUE), Output.from(get_est().T21_Ready_For_PDI_Connection, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().T20_Point_Position, JPulse.FALSE), Output.from(get_p3().T20_Point_Position, JPulse.TRUE), Output.from(get_p3().T20_Point_Position, JPulse.FALSE)),
			expectOutput(Output.from(get_p3().D6_Detection_State, new Detection_State.FALSE()), Output.from(get_p3().D6_Detection_State, new Detection_State.END_POSITION())),
			expectOutput(Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED()), Output.from(get_sec().D50_PDI_Connection_State, new PDI_Connection_State.CLOSED_READY())),
			expectOutput(Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.BOOTING()), Output.from(get_est().D51_EST_EfeS_State, new EST_EfeS_State.INITIALISING())),
			stabilize(),
			setInput(get_prim().T10_SCP_Connection_Terminated, JPulse.TRUE),
			setInput(get_sec().T10_SCP_Connection_Terminated, JPulse.TRUE),
			expectStateChange(get_prim(), "prim[2][PDI_CONNECTION_CLOSED]"),
			expectStateChange(get_sp(), "sp[0][REPORT_STATUS]"),
			expectStateChange(get_fp(), "fp[0][WATING]"),
			expectStateChange(get_est(), "est[7][INITIALISING]"),
			expectStateChange(get_p3(), "p3[23][STOPPED+ALL_RIGHT]"),
			expectStateChange(get_sec(), "sec[1][READY_FOR_CONNECTION]"),
			stabilize()
		};
	}
}
