package models.fmics;

import java.util.*;

import jlx.behave.stable.files.*;
import jlx.behave.stable.files.DecaStableFile.*;
import jlx.behave.stable.testgen.DecaStableTestGen5;
import jlx.common.reflection.ReflectionException;
import models.elx.p.v2x7x0A.DeutscheBahnPointModel;

public class Point_Compute_SIC_DFSM2 {
	/**
	 * Generates the FSM, saves it to a file:
	 */
	public static void main0(String[] args) throws ReflectionException {
		DeutscheBahnPointModel m = new DeutscheBahnPointModel(true, true, false, true);
		new DecaStableFileBuilder(m.get_ub().stableSm, false).saveToFile("sttt", "point.xsic");
	}
	
	/**
	 * Generates the SIC-DFSM, saves it to a file:
	 */
	public static void main1(String[] args) throws ReflectionException {
		DeutscheBahnPointModel m = new DeutscheBahnPointModel(true, true, false, true);
		new DecaStableFileBuilder(m.get_ub().stableSm, false).saveToFile("sttt", "point.xsic");
	}
	
	/**
	 * Reduces the SIC-DFSM modulo strong bisimilarity:
	 */
	public static void main2(String[] args) throws ReflectionException {
		DecaStableFileBisimReducer r = new DecaStableFileBisimReducer();
		r.loadFromFile("sttt", "point.xsic", false);
		
		DecaStableFileWriter dest = new DecaStableFileWriter();
		r.writeTo(dest);
		dest.saveToFile("sttt", "point.bisim.xsic");
	}
	
	/**
	 * Generates two test suites, one covering edges and one coveraging edges *labels*.
	 */
	public static void main3(String[] args) throws ReflectionException {
		DecaStableFileWriter r1 = new DecaStableFileWriter();
		r1.loadFromFile("sttt", "point.bisim.xsic", false);
		new DecaStableTestGen5(r1, 0.1f, false).populateTests("trcov");
		r1.saveToFile("sttt", "point.bisim.trcov.xsic");
		
		DecaStableFileWriter r2 = new DecaStableFileWriter();
		r2.loadFromFile("sttt", "point.bisim.xsic", false);
		new DecaStableTestGen5(r2, 0.11f, true).populateTests("trlabcov");
		r2.saveToFile("sttt", "point.bisim.trlabcov.xsic");
	}
	
	/**
	 * Compares the coverage of the two test suites.
	 */
	public static void main(String[] args) throws ReflectionException {
		DecaStableFileWriter r1 = new DecaStableFileWriter();
		r1.loadFromFile("sttt", "point.bisim.trcov.xsic", false);
		
		DecaStableFileWriter r2 = new DecaStableFileWriter();
		r2.loadFromFile("sttt", "point.bisim.trlabcov.xsic", false);
		
		ProtoData data = new ProtoData("all", r1.getProto().values());
		data.print();
		
		Set<String> transitions1 = analyze(r1, "r1");
		Set<String> transitions2 = analyze(r2, "r2");
		
		for (String t0 : data.transitions) {
			if (!transitions1.contains(t0)) {
				System.out.println("DIFF0 " + t0);
			}
		}
		
		for (String t1 : transitions1) {
			if (!transitions2.contains(t1)) {
				System.out.println("DIFF1 " + t1);
			}
		}
	}
	
	private static Set<String> analyze(DecaStableFileWriter r, String name) {
		Set<Proto> ps = new HashSet<Proto>();
		
		for (TestSuite ts : r.getTestSuites()) {
			for (Trace t : ts.getTests()) {
				for (Transition step : t.getTransitions()) {
					ps.addAll(step.getProtoSet().getElems());
				}
			}
		}
		
		ProtoData data = new ProtoData(name, ps);
		data.print();
		return data.transitions;
	}
	
	public static class ProtoData {
		public final String name;
		public final Set<String> simpleStates;
		public final Set<String> transitions;
		public final Map<String, Set<String>> transitionsPerCat;
		public final Set<String> pairs;
		
		public ProtoData(String name, Collection<Proto> ps) {
			this.name = name;
			
			simpleStates = new HashSet<String>();
			
			for (Proto p : ps) {
				if (p.getDescription().endsWith("::SimpleState")) {
					int colonIndex = p.getDescription().indexOf("::");
					String stateName = p.getDescription().substring(0, colonIndex);
					simpleStates.add("[" + stateName + "]");
				}
			}
			
			transitions = new HashSet<String>();
			transitionsPerCat = new HashMap<String, Set<String>>();
			transitionsPerCat.put("do", new HashSet<String>());
			transitionsPerCat.put("entry", new HashSet<String>());
			transitionsPerCat.put("exit", new HashSet<String>());
			transitionsPerCat.put("-->", new HashSet<String>());
			pairs = new HashSet<String>();
			
			for (Proto p : ps) {
				if (p.getDescription().endsWith("::SimpleState")) {
					//Do nothing.
				} else {
					if (p.getDescription().contains("-->")) {
						transitions.add(p.getDescription());
						
						int bracketIndex1 = p.getDescription().indexOf("]---") + 1;
						int bracketIndex2 = p.getDescription().indexOf("-->[") + 3;
						
						String src = p.getDescription().substring(0, bracketIndex1);
						String tgt = p.getDescription().substring(bracketIndex2, p.getDescription().length());
						
						if (!src.equals("[??]") && !tgt.equals("[??]")) {
							if (simpleStates.contains(src) || simpleStates.contains(tgt)) {
								pairs.add(src + " -> " + tgt);
							}
						}
						
						if (p.getDescription().contains("---do-->")) {
							transitionsPerCat.get("do").add(p.getDescription());
						} else {
							if (p.getDescription().contains("---entry-->")) {
								transitionsPerCat.get("entry").add(p.getDescription());
							} else {
								if (p.getDescription().contains("---exit-->")) {
									transitionsPerCat.get("exit").add(p.getDescription());
								} else {
									transitionsPerCat.get("-->").add(p.getDescription());
								}
							}
						}
					}
				}
			}
		}
		
		public void print() {
			System.out.println(name + ".#simple-states = " + simpleStates.size());
			System.out.println(name + ".#transitions = " + transitions.size());
			
			for (Map.Entry<String, Set<String>> e : transitionsPerCat.entrySet()) {
				System.out.println(name + ".#transitions[" + e.getKey() + "] = " + e.getValue().size());
			}
			
			System.out.println(name + ".#pairs = " + pairs.size());
		}
	}
}

