package models.sttt.point._2pm;

import jlx.behave.stable.files.DecaStableScenario;
import jlx.behave.stable.files.DecaStableScenarioException;

import jlx.behave.proto.DecaFourScenarioPlayerComp;
import jlx.behave.proto.DecaFourTransition;
import jlx.behave.proto.DecaFourVertex;
import jlx.behave.proto.ProtoTransition;
import jlx.behave.proto.ProtoVertex;
import jlx.behave.proto.gui.DecaFourScenarioLog;
import jlx.behave.proto.gui.DecaFourScenarioPlayer;
import jlx.behave.stable.files.DecaStableFile.TestSuite;
import jlx.behave.stable.files.DecaStableFile.Trace;
import jlx.behave.stable.files.DecaStableFileReader;
import jlx.common.reflection.ReflectionException;
import models.elx.p.v2x7x0A.DeutscheBahnPointModel;

public class Step5_CompareTestSuites {
	/**
	 * Compares the coverage of the two test suites.
	 */
	public static void main(String[] args) throws ReflectionException, DecaStableScenarioException {
		System.out.println(Step5_CompareTestSuites.class.getName());
		
		//We are not comparing to the DB test suite here, since it assumes 1 PM.
		//Note that we also enable the 3 error ports.
		DeutscheBahnPointModel model = new DeutscheBahnPointModel(true, true, false, true);
		
		DecaFourScenarioPlayer trCovPlayer = new DecaFourScenarioPlayer(model.get_ub());
		trCovPlayer.setName("trcov");
		
		DecaStableFileReader reader = new DecaStableFileReader();
		reader.loadFromFile("sttt", "point.2pm.bisim.2suite.sic", false);
		
		TestSuite ts1 = reader.getTestSuites().get(0);
		TestSuite ts2 = reader.getTestSuites().get(1);
		
		System.out.println("Found " + ts1.getTests().size() + " tests (" + ts1.getDescription() + ")!");
		
		for (int i = 1; i <= ts1.getTests().size(); i++) {
			Trace t = ts1.getTests().get(i - 1);
			System.out.println("Running test " + i + " of " + ts1.getTests().size() + "...");
			
			DecaStableScenario scenario = new DecaStableScenario(model.get_ub(), t);
			trCovPlayer.play(scenario, "sttt/failures-2pm/trcov-test-" + i + "-log.txt");
			
			System.out.println("Running test " + i + " of " + ts1.getTests().size() + "... done!");
			i++;
		}
		
		DecaFourScenarioPlayer trLabCovPlayer = new DecaFourScenarioPlayer(model.get_ub());
		trLabCovPlayer.setName("trlabcov");
		
		trLabCovPlayer.setCfgConsumer((cfg, log) -> {
			if (!trCovPlayer.getPotVisitedCfgs().contains(cfg)) {
				System.out.println("Illegal cfg!!");
				
				if (model.get_ub().stableSm.coveredCfgs.contains(cfg)) {
					System.out.println("Illegal cfg is in the stableSm.coveredCfgs!!");
				}
				
				if (model.get_ub().stableSm.coveredStableCfgs.contains(cfg)) {
					System.out.println("Illegal cfg is in the stableSm.coveredStableCfgs!!");
				}
				
				log.saveToFile("sttt/illegal-cfg-log.txt");
				throw new Error("Entered illegal configuration!!");
			}
		});
		
		System.out.println("Found " + ts2.getTests().size() + " tests (" + ts2.getDescription() + ")!");
		
		for (int i2 = 1; i2 <= ts2.getTests().size(); i2++) {
			Trace t = ts1.getTests().get(i2 - 1);
			System.out.println("Running test " + i2 + " of " + ts2.getTests().size() + "...");
			
			DecaStableScenario scenario = new DecaStableScenario(model.get_ub(), t);
			trLabCovPlayer.play(scenario, "sttt/failures-2pm/trlabcov-test-" + i2 + "-log.txt");
			
			System.out.println("Running test " + i2 + " of " + ts2.getTests().size() + "... done!");
			i2++;
		}
		
		DecaFourScenarioPlayer p = new DecaFourScenarioPlayer(model.get_ub());
		p.setName("p");
		p.getLookedOutForCfgs().addAll(new DecaFourScenarioPlayerComp(trCovPlayer, trLabCovPlayer).addCfgComparison(model.get_ub().sms4, 20));
		
		if (p.getLookedOutForCfgs().size() > 0) {
			System.out.println("Found " + p.getLookedOutForCfgs().size() + " anomalies!");
			System.out.println("Revisiting " + ts2.getTests().size() + " tests!");
			int confirmedAnomalyCount = 0;
			
			for (int j = 1; j <= ts2.getTests().size(); j++) {
				Trace t = ts1.getTests().get(j - 1);
				System.out.println("Re-running test " + j + " of " + ts2.getTests().size() + "...");
				
				DecaStableScenario scenario = new DecaStableScenario(model.get_ub(), t);
				DecaFourScenarioLog log = p.play(scenario, null);
				
				if (log.getFoundCfgs().size() > 0) {
					log.saveToFile("sttt/anomalies-2pm/anomaly-test-" + j + "-log.txt");
					confirmedAnomalyCount++;
				}
				
				System.out.println("Re-running test " + j + " of " + ts2.getTests().size() + "... done!");
				j++;
			}
			
			System.out.println("Investigated " + p.getLookedOutForCfgs().size() + " anomalies!");
			System.out.println("Found " + confirmedAnomalyCount + " confirmations of anomalies!");
		}
		
		DecaFourScenarioPlayerComp comp = new DecaFourScenarioPlayerComp(trCovPlayer, trLabCovPlayer);
		comp.addComparison(ProtoVertex.class, 10);
		comp.addComparison(ProtoTransition.class, 10);
		comp.addComparison(DecaFourVertex.class, 10);
		comp.addComparison(DecaFourTransition.class, 10);
		comp.addCfgComparison(model.get_ub().sms4, 10);
		comp.saveToFile("sttt/test-suite-comp.pm2.txt", 100);
		comp.saveToFile("sttt/test-suite-comp-summary.pm2.txt", 0);
	}
}
